
zc = zc or {}

-- 根据弧度求角度
function CC_RADIANS_TO_DEGREES( angel )
    return angel * 57.29577951;
end

-- 根据角度求弧度
function CC_DEGREES_TO_RADIANS( angel )
    return angel * 0.01745329252;
end

function zc.doFuncForAllChild( sNode, sFunc )
    local function _func_( _node ) 
        if sFunc then sFunc(_node) end
        if not tolua.isnull(_node) then
            local _children = _node:getChildren()
            for k,node in pairs(_children) do
                _func_(node)
            end
        end
    end
    _func_(sNode)
end

--[[
    src,
    plist,
    ap,
    pos,
]]
function zc.getSprite( params )
    local _plist = params.plist == nil and false or params.plist
    local _file = params.src
    local _sp 
    if _plist then
        _sp = zc.createSpriteWithFrameName(_file)
    else
        _sp = zc.createSprite(_file)
    end

    if params.ap then
        _sp:setAnchorPoint(params.ap)
    end

    if params.pos then
        _sp:setPosition(params.pos)
    end
    if params.name then
        _sp:setName(params.name)
    end
    if params.scale then
        _sp:setScale(params.scale)
    end

    return _sp
end

function zc.createSprite( file )
    local _s = nil
    if file then _s = cc.Sprite:create(file) end
    if not _s then
        if file then
            -- print("wm-----errTip : " .. tostring(file) .. " notExit")
        end
        _s = cc.Sprite:create()
    end
    return _s
end

function zc.createSpriteWithFrameName( name )
    local _s = nil
    if name then
        if cc.SpriteFrameCache:getInstance():getSpriteFrameByName(name) then
            _s = cc.Sprite:createWithSpriteFrameName(name)
        end
    end
    if not _s then
        -- print("wm-----errFameTip : " .. tostring(name) .. " notExit")
         _s = cc.Sprite:create()
    end
    return _s
end

function zc.removeEventListener(eventName)
    cc.Director:getInstance():getEventDispatcher():removeCustomEventListeners(eventName)
end

--[[
{
name = "xxx",
callback = function(event)
end
}
name:字符串，事件名称
callback:
    function (event)
    end)
]]
function zc.addEventListener(params)
    local eventName         = params.name
    local callback          = params.callback
    zc.removeEventListener(eventName)
    
    local custom_listener = cc.EventListenerCustom:create(eventName,callback)
    local eventDispatcher = cc.Director:getInstance():getEventDispatcher()
    eventDispatcher:addEventListenerWithFixedPriority(custom_listener, 1)
end

--[[
添加在节点上的监听，如果传入node为nil，走zc.addEventListener
节点上的监听在节点被移掉的时候会自动移掉。
]]
function zc.addEventListenerWithNode(params)
    local eventName         = params.name
    local callback          = params.callback
    local node              = params.node
    
    local custom_listener = cc.EventListenerCustom:create(eventName,callback)
    local eventDispatcher = cc.Director:getInstance():getEventDispatcher()
    if node then
        eventDispatcher:addEventListenerWithSceneGraphPriority(custom_listener, node)
    else
        eventDispatcher:addEventListenerWithFixedPriority(custom_listener, 1)
    end
end

--[[
发送自定义的引擎消息
一般用于刷新数据
name: 事件名称
data:携带的数据(在接收时可以使用)
{
name = "xxx",
data = "xxx"
}
]]
function zc.dispatchEvent(params)
    local eventName = params.name
    local data      = params.data

    local event     = cc.EventCustom:new(eventName)
    event.data      = data
    local eventDispatcher = cc.Director:getInstance():getEventDispatcher()
    eventDispatcher:dispatchEvent(event)
end



--[[
--创建默认参数
local defaultParams = {
text = "",
fontSize = nil,--字体大小
size = nil,--字体大小，同上
ttf = nil,--自定义字体，默认字体使用:Helvetica
pos = cc.p(0,0),
color = cc.c3b(255, 255, 255),
anchor = cc.p(0.5,0.5),--锚点
needSwallow = false,--是否需要吞噬事件，默认不吞噬
clickable = true,--是否可以点击
beganCallback = nil,--点击事件的按下回调
endCallback = nil,--点击事件的抬起回调
touchSize = cc.size(0,0)--点击区域，如果没有传值，则默认点击区域为getBoundingBox()
}

[color=ffffffff]test[/color] 控制字体颜色，颜色值按aa rr gg bb排列
[b][/b] 字体加粗
[image=path offsety=-10 w=20 h= 20][/image] 图标 ,其中offsety如果为负数，则代表图标向下偏移10像素，w代表图标宽度(图标将强制变成此宽度)
[u][/u]下划线
[i][/i] 文字倾斜 汉字可能没有效果

设置颜色渐变 : setGradientColor(const ccColor4B& start, const ccColor4B& end, const CCPoint& v);
设置阴影：obj:enableShadow( cc.size(1,-1), 0xff0000ff, 1 )
]]
function zc.createRichLabel(params)
    local platform = cc.Application:getInstance():getTargetPlatform()
    if platform == cc.PLATFORM_OS_ANDROID then
        params.ttf = zc.getFont(1)
    else
        params.ttf = zc.getFontName(1)
    end
    return ZCRichLabelTTF:createWithParams(params)
end
--[[
--创建默认参数
local defaultParams = {
text = "",
fontSize = nil,--字体大小
size = nil,--字体大小，同上
ttf = nil,--自定义字体，默认字体使用:Helvetica
fnt             = nil,
kerning         = 0,----左右字距
pos = cc.p(0,0),
color = cc.c3b(255, 255, 255),
anchor = cc.p(0.5,0.5),--锚点
needSwallow = false,--是否需要吞噬事件，默认不吞噬
clickable = true,--是否可以点击
beganCallback = nil,--点击事件的按下回调
endCallback = nil,--点击事件的抬起回调
touchSize = cc.size(0,0)--点击区域，如果没有传值，则默认点击区域为getBoundingBox()
}
]]
zc.labelWarmColor = cc.c3b(137, 74, 6)
zc.labelColdColor = cc.c3b(6, 83, 137)
zc.tabLabNormalColor1 = cc.c3b( 122, 42, 0 )
zc.tabLabNormalColor2 = cc.c3b( 255, 232, 128 )
zc.tabLabselectColor2 = cc.c3b( 72, 37, 0 )

function zc.createLabel( params )
    local name      = params.name or nil
    local width     = params.width or nil
    local tag       = params.tag or nil
    local fontName  = params.ttf or params.font or params.fontName or zc.getFont(1)
    local ap        = params.ap or cc.p(0.5, 0.5)
    local opacity   = params.opacity or 255

    params.ttf      = fontName
    params.anchor   = ap

    local label     = ZCLabel:createWithParams(params)
    if name     then label:setName(name) end
    if tag      then label:setTag(tag) end
    -- 换行的话
    if width    then label:setWidth(width) end
    label:setOpacity(opacity)

    function label:setText(text)
        label:setString(text)
    end
    if params.color == display.COLOR_BLACK then
        params.stroke = false
    end
    if params.stroke and params.stroke == true then
        local outlineColor  = params.color2 or display.COLOR_BLACK
        local shadowSize    = params.shadowSize or cc.size(2, -2)
        label:enableShadow(cc.c4b(outlineColor.r,outlineColor.g,outlineColor.b,255), shadowSize)
    end
    if params.shadow and params.shadow == true then
        local outlineColor  = params.color2 or display.COLOR_BLACK
        local shadowSize    = params.shadowSize or cc.size(2, -2)
        label:enableShadow(cc.c4b(outlineColor.r,outlineColor.g,outlineColor.b,255), shadowSize)
    end
    if params.outLine and params.outLine == true then
        local outlineColor  = params.color2 or display.COLOR_BLACK
        local pixel = params.pixel or 1
        label:enableOutline(cc.c4b(outlineColor.r,outlineColor.g,outlineColor.b,255), pixel)  
    end
    return label
end
--[[
加载动画放入缓存中，使用时请通过zc.getAnimation获取
]]
function zc.loadAnimation(path, name, indexA, indexB, delay)
    local delay = delay or 0.1

    local SpriteManager = cc.SpriteFrameCache:getInstance()
    local AnimManager = cc.AnimationCache:getInstance()

    if path ~= nil and name ~= nil then
        local frames = {}
        local texture = SpriteManager:addSpriteFrames(path .. name .. ".plist")
        for j = indexA, indexB do
            if SpriteManager:getSpriteFrame(name.."_"..tostring(j)..".png") then
                frames[#frames+1] = SpriteManager:getSpriteFrame(name.."_"..tostring(j)..".png")
            end
        end
        local newAnimation = cc.Animation:createWithSpriteFrames(frames, delay)
        AnimManager:addAnimation(newAnimation,name)
    end
end

function zc.getSpriteWithAnimationFrames( params )
    local _sp = zc.createSprite()
    local _action = zc.getAnimationBySpriteFrame(params)
    _sp:runAction(cc.RepeatForever:create(_action))
    return _sp
end

function zc.getAnimationBySpriteFrame( params )
    local _path = params.path
    local _preFix = params.preFix
    local _startIndex = params.startIndex
    local _endIndex = params.endIndex
    local _perUnit = params.perUnit or 2.8 / 14.0
    local _spFrame = cc.SpriteFrameCache:getInstance()
    _spFrame:addSpriteFrames(_path..".plist", _path..".png")
    local animation = cc.Animation:create()
    for i = _startIndex , _endIndex do
        local filepath = _preFix .. i .. ".png"
        animation:addSpriteFrame(_spFrame:spriteFrameByName(filepath))
    end
    animation:setDelayPerUnit( _perUnit )
    animation:setRestoreOriginalFrame(true)
    local action = cc.Animate:create(animation)
    return action
end

function zc.getAnimation(name, imgType)
    local imageType
    if imgType ~= nil then
        imageType = "."..imgType
    else
        imageType = ".png"
    end
    local sprite = cc.Sprite:createWithSpriteFrameName(name.."_1"..imageType)
    local animation = cc.AnimationCache:getInstance():getAnimation(name)
    return sprite, animation
end

function zc.unloadAnimation(path, name, part, imgType)
    local imageType
    if imgType~= nil then
        imageType = "."..imgType
    else
        imageType = ".png"
    end
    if path ~= nil and name ~= nil then
        cc.AnimationCache:getInstance():removeAnimationByName(name)
        if part ~= nil then
            for i = 1,part do
                cc.SpriteFrameCache:getInstance():removeSpriteFramesFromFile(path..name..i..".plist")
                cc.TextureCache:getInstance():dumpCachedTextureInfo()
                cc.TextureCache:getInstance():removeTextureForKey(path..name..i..imageType)
            end
        else
            cc.AnimationCache:getInstance() :removeAnimationByName(name)
            cc.SpriteFrameCache:getInstance():removeSpriteFramesFromFile(path..name..".plist")
            cc.TextureCache:getInstance():removeTextureForKey(path..name..imageType)
        end
    end
end

function zc.getDynamicTime( distance, perPx )
    distance = math.abs(distance)
    return distance / perPx;
end

function zc.getSpine( fileName, scale )
    local params = {
        fileName = fileName,
        scale = scale,
    }
    return zc.createSpine(params)
end

function zc.createSpine( params )
    local _scale = tonumber(params.scale) or 1.0
    local _fileName = params.fileName
    local _actName = params.actName
    local autoRemove = params.autoRemove
    local _loop = params.loop == nil and true or params.loop
    local _removeCallback = params.removeCallback
    local _needInit = params.needInit
    local _path = params.path
    if not _path then
        _path = "res/animation/".._fileName.."/"
    end
    local _pathJson = _path .. _fileName ..".json"
    local _atlasJson = _path .. _fileName ..".atlas"
    local _spine = SpineBase:createWithParams({
        jsonFile = _pathJson, 
        atlasFile = _atlasJson, 
        scale = _scale,
        needInit = _needInit,
    })
    
    if _actName then
        _spine:playAction(_actName, _loop)
    end

    if autoRemove then
        _spine:registerSpineEventHandler( function ( event )
            -- _spine:unregisterSpineEventHandler(sp.EventType.ANIMATION_COMPLETE)
            performWithDelay(_spine, function()
                _spine:removeFromParent()
                if _removeCallback then
                    _removeCallback()
                end
            end, 0.1)
        end, sp.EventType.ANIMATION_COMPLETE)
    end
    return _spine
end

function zc.createAnimationArmature( params )
    params.parentPath = "res/animation/" .. tostring(params.fileName) .. "/"
    params.autoRelease = true
    return zc.createArmature(params)
end

local ARM_COUNT = {}
local ARM_USEFUL = {}

local function _armCountAdd(resFile, autoRelease)
    if autoRelease ~= true then
        return false
    end
    local _resFile = tostring(resFile)
    ARM_COUNT[_resFile] = ARM_COUNT[_resFile] or 0
    ARM_COUNT[_resFile] = ARM_COUNT[_resFile] + 1
    print("wm----_armCountAdd : ", ARM_COUNT[_resFile], " , ", _resFile)   
    return true
end

local function _armCountPlus(resFile, autoRelease)
    if autoRelease ~= true then
        return
    end
    local _resFile = tostring(resFile)
    ARM_COUNT[_resFile] = ARM_COUNT[_resFile] or 0
    ARM_COUNT[_resFile] = ARM_COUNT[_resFile] - 1
    ARM_COUNT[_resFile] = ARM_COUNT[_resFile] < 0 and 0 or ARM_COUNT[_resFile]
    print("wm----_armCountPlus : ", ARM_COUNT[_resFile], " , ", _resFile) 
    if ARM_COUNT[_resFile] == 0 then
        if not inArray(_resFile, ARM_USEFUL) then
            ccs.ArmatureDataManager:getInstance():removeArmatureFileInfo(_resFile, true)
        end
    end  
end

--[[
@ fileName          = --文件名，不包括父目录以及后缀名
@ actName           = --需要播放的动作名,如果此参数为nil，则代表只创建骨骼动画
@ loop              = --是否循环，默认不循环
@ parentPath        = --父目录
@ suffix            = --后缀
@ completeCallback  = --动作完成后的回调事件
@ eventCallback     = --帧事件
@ moveCallback      = --
@ autoRemove        = ----动作complete后是否自动删除
]]
function zc.createArmature(params)
    local time              = socket.gettime()
    local readFileTime      = time
    local creatTime         = time
    local fileName          = tostring(params.fileName)--[[文件名，不包括父目录以及后缀名]]
    local actName           = params.actName
    local loop              = params.loop
    local parentPath        = params.parentPath--[[父目录]]
    local suffix            = params.suffix--[[后缀]]
    local completeCallback  = params.completeCallback--[[动作完成后的回调事件]]
    local eventCallback     = params.eventCallback
    local moveCallback      = params.moveCallback
    local autoRemove        = params.autoRemove--[[--动作complete后是否自动删除]]
    local async             = params.async
    local asyncCallback     = params.asyncCallback
    local autoRelease       = params.autoRelease
    if autoRemove == nil then autoRemove = false end
    if parentPath == nil then parentPath = "res/image/battle/hero/" end
    if suffix     == nil then suffix = ".csb" end
    local resFile           = parentPath..tostring(fileName)..suffix
    print("resFile="..resFile..",fileName="..tostring(fileName))
    local armatureDataManager = ccs.ArmatureDataManager:getInstance()
    local armature          = ccs.Armature:create()
    local _haveAdd = false
    if cc.FileUtils:getInstance():isFileExist(resFile) == true then
        --[[目前只做简单的异步加载]]
        if async == true then
            --[[如果异步加载失败，就删除掉该记录]]
            armatureDataManager:addArmatureFileInfoAsync(resFile, function(num) 
                if armatureDataManager:getAnimationData(fileName) and armatureDataManager:getArmatureData(fileName) then
                    -- print("异步加载成功："..resFile..",fileName="..fileName)
                else
                    print("异步加载失败："..resFile..",fileName="..fileName)
                end
                local _armature_ = nil
                if not tolua.isnull(armature) then 
                    armature:init(fileName) 
                    _armature_ = armature
                    _haveAdd = _armCountAdd(resFile, autoRelease)
                end
                if asyncCallback then asyncCallback(_armature_ , fileName , num) end
            end)
        else
            armatureDataManager:addArmatureFileInfo(resFile)
            readFileTime = socket.gettime() - time
            xpcall(function ( ... )
                armature            = ccs.Armature:create(fileName)
                creatTime           = socket.gettime() - time - readFileTime
                if armature then
                    _haveAdd = _armCountAdd(resFile, autoRelease)
                end
            end, function ( msg )
                print("----------------------------------------createArmature error")
                print("LUA ERROR: " .. tostring(msg) .. "\n")
                print(debug.traceback())
                print("----------------------------------------")
                -- armatureDataManager:removeArmatureFileInfo(resFile)
                -- armatureDataManager:addArmatureFileInfo(resFile)
                armature            = ccs.Armature:create()
            end)
            if armature and actName and string.len(tostring(actName)) > 0 then
                actName             = tostring(actName)
                local animation     = armature:getAnimation()
                --[[--如果为空，则默认不循环]]
                if loop == nil or tostring(loop) == "false" then
                    loop = 0
                elseif tostring(loop) == "true" or tonumber(loop) > 0 then
                    loop = 1
                end
                animation:play(actName,-1,loop)
                
                if moveCallback == nil then
                    moveCallback = function (armatureBack,movementType,movementID)
                        if movementType == ccs.MovementEventType.complete then
                            if completeCallback then
                                completeCallback()
                            end
                            if autoRemove == true and loop == 0 then 
                                xpcall(function() 
                                    if not tolua.isnull(armature) then
                                        armature:removeFromParent()
                                    end
                                end, function ()
                                    print("armature已被删除")
                                end)
                            end
                        end
                    end
                end
                animation:setMovementEventCallFunc(moveCallback)
                if eventCallback then
                    animation:setFrameEventCallFunc(eventCallback)
                end
            end
        end
    else
        print("************************************文件"..resFile.."不存在")
    end
    -- 处理自动释放
    local _sp = cc.Node:create()
    _sp:onNodeEvent("cleanup", function()
        xpcall(function() 
            if tolua.isnull(armature) then
                if not tolua.isnull(_sp) then
                    _sp:removeFromParent()
                end
                return
            end
            local _bone = armature:getBone("weapons")
            local node = _bone:getDisplayRenderNode()
            if node and node:getChildByName("cleanSp") then
                node:getChildByName("cleanSp"):removeFromParent()
            end
        end, function() end)
        if _haveAdd then
            xpcall(function() 
                _armCountPlus(resFile, autoRelease)
            end, function() end)
        end
    end)
    _sp:setName("cleanSp")
    armature:addChild(_sp)
    -- print("*************************"..tostring(socket.gettime() - time)..",readFileTime="..tostring(readFileTime)..",creatTime="..tostring(creatTime)..",resFile="..resFile)
    return armature
end





helper = helper or {}

helper.releaseSpineCaches = function()
    zc.dispatchEvent({name = "REMOVE_UNUSED_SPINES"})
    zc.dispatchEvent({name = "REMOVE_UNUSED_SPINE_DATA"})
end

local function _releaseMemory( removeCSB )
    if removeCSB then
        zc.dispatchEvent({name = "CLEAR_CACHED_CSB_DATA"})
        local manager = ccs.ArmatureDataManager:getInstance()
        xpcall(function()
            manager:removeAllArmatureFileInfo(true)
            print("!!!!!!!!!!!!!!!!!!!清空所有csb数据成功")
        end,function( ... )
            print("!!!!!!!!!!!!!!!!!!!清空所有csb数据失败")
        end)
        ARM_COUNT = {}
    end
    helper.releaseSpineCaches()
    cc.SpriteFrameCache:getInstance():removeUnusedSpriteFrames()
    cc.Director:getInstance():getTextureCache():removeUnusedTextures()
    collectgarbage("collect")
    print("*************清理纹理*************")
end

helper.reloadAllFile = function()
    local _global = package.loaded
    for key, value in pairs(_global) do
        if string.find(tostring(key), "src/") and string.find(tostring(key), ".lua") then
            print("---------------------->"..tostring(key))
            package.loaded[key] = nil;   --赋值为nil的目的是为了重新加载.lua文件，值为true是标记已经加载过了
        end
    end
    --[[异步更新后才需要清除动画的配置文件]]
    _releaseMemory(true)
end

helper.getNowCacheInfo = function()
    local textureCache = cc.Director:getInstance():getTextureCache()
    local memInfo = textureCache:getCachedTextureInfo()
    local _s = string.find(memInfo, "%(")
    _s = tonumber(_s) or 0
    local _e = string.find(memInfo, "MB)")
    _e = tonumber(_e) or 2
    local ueserMem = string.sub(memInfo, _s + 1, _e - 1)
    ueserMem = tonumber(ueserMem) or 0
    print("----now collectMemory : " .. tostring(ueserMem))
    return ueserMem, memInfo
end

-- 主动释放不用的spriteframe和texture  addby wangming 20150813 
-- tip : 请谨慎使用, 小心刚加载，还没被引用就被释放掉得情况出现
helper.collectMemory = function( isForcible, _sNum, removeCSB )
    local memoryTop = tonumber(_sNum) or 70 
    -- 纯纹理内存控制的最大值，大约为实际内存占用量的一半左右，到达则做释放无用纹理处理，可根据做成变量，不同环境具体调整参数
    if isForcible then
        _releaseMemory(removeCSB)
    else
        local ueserMem = helper.getNowCacheInfo()
        if ueserMem >= memoryTop then
           _releaseMemory(removeCSB)
        end
    end
end

-- 封装了错误处理，收集错误信息
helper.xpcall = function( funcA, funcB )
    xpcall(function()
        if funcA then
            funcA()
        end
    end, function( errorMsg )
        if funcB then
            funcB()
        end
    end)
end