local RewardLayer = class("RewardLayer",function ()
	return ZCPopLayer:create()
end)

function RewardLayer:ctor(params)
	musicManager.playEffect( zc.effect.btnFetch )
    self:setOpacity(200)
	self:initData(params)
	self:initUI()

	local _callBack = params.callback
	self:setHideCallback(function()
		if _callBack and type(_callBack) == "function" then
			_callBack()
		end
	end)
	self:show()
end

function RewardLayer:initData(params)
	dump(params)
	self.id_list = params.id
	self.turnTableData = params.turnTableData
end
function RewardLayer:initUI()
	-- 背景
	local background = cc.Sprite:create( "res/Main/battle/battleresult/battle_bg.png" )
	background:setPosition( zc.width/2, zc.height/2 - 30 )
	self:addContent( background )
	self._bg = background
	self._bgSize = background:getContentSize()
    local close = zc.createPushButton({
        touchSize = cc.size(10000,10000),
        needSwallow = true,
        anchor = cc.p(0.5, 0.5),
        pos = cc.p(self._bgSize.width/2, self._bgSize.height/2),
        endCallback = function() end,
    })
    background:addChild(close)
    self:runAction(cc.Sequence:create(
        cc.DelayTime:create(0.8),
        cc.CallFunc:create(function() close:setTouchEndedCallback(function() self:hide() end) end)
    ))
	-- 奖励
    local armature = zc.createArmature({
        parentPath = "res/Main/battle/battleresult/win/",
        fileName = "win",
        actName = "SWL_star_3",
        eventCallback = function() end,
        completeCallback = function(armatureBack,movementType,movementID)
            self._armature:getAnimation():play("SWL_idle_3", -1, 0)
        end
    })
    armature:setPosition(self._bgSize.width/2, self._bgSize.height)
    background:addChild(armature)
    self._armature = armature

	local closeLabel = zc.createLabel({
	    text = zc.lang("tips_click_empty_continue"),
	    outLine = true,
	    fontSize = 22,
	    ap = cc.p(0.5, 0),
	    pos = cc.p(self._bgSize.width/2 + 20, -25),
	    pixel = 1.3
	})
	background:addChild(closeLabel)
	closeLabel:setCascadeOpacityEnabled(true)
    closeLabel:setOpacity(255)
	closeLabel:runAction(cc.RepeatForever:create(cc.Sequence:create(cc.FadeIn:create(1),cc.FadeOut:create(1))))

    local bg = background
	performWithDelay(self, function()
		self:createRewardTablview(reward)
    end, 0.1)
end
function RewardLayer:createRewardTablview(reward)
	local list_pos = {
		cc.p(400,360),cc.p(500,360),cc.p(600,360),cc.p(700,360),cc.p(800,360),
		cc.p(400,240),cc.p(500,240),cc.p(600,240),cc.p(700,240),cc.p(800,240)
	}
	for i = 1,10 do
		local item_id = self.id_list[(i)]
		if item_id then
			local data = self.turnTableData[tostring(item_id)].reward
			local list = string.split(data or "","|")
			local _item = zc.createIconByData({
	            data = {
					mainType = list[1],
					type = list[2],
	                id = list[2],
	                num = list[3],
				},
				showCount = true,
	            pos = list_pos[i],
	        })
	        _item:setScale(0.7)
	        self:addChild(_item)
		end
	end
end
function RewardLayer:buildCell(cell, idx, cellWidth, cellHeight)
	cell._item = {}
	for i = 1, 6 do
	    local posX = cellWidth/12*(2*i - 1)
	    local posY = cellHeight/2
		local item = zc.createIconByData({
	        data = {
	            mainType = 1,
	            type = 1, 
	            id = 1,
	            num = 0,
	        }, 
	        showDetailPop = true,
	        pos = cc.p(posX, posY),
	    })
	    cell:addChild(item)
	    local _desc = zc.createLabel({
	        outLine = true,
	        fontSize = 24,
	        pos = cc.p(item:getContentSize().width/2, item:getContentSize().height/2 - 80),
	        pixel = 1.3
	    })
	    item:addChild(_desc)
	    item._desc = _desc
	    item:setCascadeOpacityEnabled( true )
	    cell._item[i] = item
	end
end
function RewardLayer:updateCell(cell, idx)
	for i = 1, 6 do
		local _index = idx*6 + i
		local cellData = self._reward[_index]
		local item = cell._item[i]
		if not cellData then
			item:setVisible( false )
		else
			item:setVisible( true )
			local _itemType = tonumber(cellData.itemType) or 0
			local _itemId = tonumber(cellData.itemId) or 0

		    info = zc.updateIconByData(item, {
		        data = {
		            mainType = cellData.itemType,
		            type = cellData.itemId, 
		            id = cellData.itemId,
		            num = cellData.changeNum,
		            level = (_itemType == zc.mainTypeWeapon) and zc.getLevelTipShow()..self:findWeapon(_itemId) or nil
		        }, 
		        showLevel = (_itemType == zc.mainTypeWeapon) and true or false,
		        showDetailPop = true,
		    })

		    --如果是转盘，则显示获得的倍数
		    if self.__TurnInfo then
		    	local multiple = self.__TurnInfo.multiple or 1
		    	local _str = multiple.. zc.lang("beijiangli")
		    	item._desc:setString(_str)
		    	item._desc:setColor(display.COLOR_GOLD)
		    end

		    if not self._isinit then
			    item:setScale(1.0)
			    item:setOpacity(0)

				item:runAction(
					cc.Sequence:create(
						cc.DelayTime:create( 0.1*_index ),
						cc.Spawn:create(
							cc.FadeIn:create( 0.2 ),
							cc.Sequence:create(
								cc.DelayTime:create(0.1),
								cc.ScaleTo:create(0.15,0.8)
							)
						)
					)
				)
			else
				item:setScale(0.8)
			end
		end
	end
end

function RewardLayer:create(params,callback)
	return self.new(params)
end

function RewardLayer:onCleanup()
    local textureCache = cc.Director:getInstance():getTextureCache()
    textureCache:removeTextureForKey("res/plugin/RewardLayer/reward_bg.png")
    textureCache:removeTextureForKey("res/plugin/RewardLayer/reward_reel.png")
end
return RewardLayer