--[[
	演武榜排行榜
	唐实聪
	2016.3.18
]]
local ArenaRankPop = class("ArenaRankPop", function()
	return ZCPopLayer:create()
end)

function ArenaRankPop:create( params )
	return ArenaRankPop.new( params )
end

function ArenaRankPop:ctor( params )
	self:initData( params )
	self:initUI()
	self:show()
end

function ArenaRankPop:initData( params )
	-- dump( params, "ArenaRankPop" )
	local _type = params.type or 1
	self._type = _type
	if inArray( _type, { 1, 2, 3} ) then
		self._typeConfig = {
			1, 2, 3
		}
		self._index = _type
	else
		self._typeConfig = {
			11, 12
		}
		self._index = _type - 10
	end

	self._rank = params.rank or 0
	self._score = params.score or 0

	self._rankData = params.data.ranks
	for k, v in pairs( self._rankData ) do
		table.sort(v, function( a, b )
			return a.rank < b.rank
		end)
	end
	
	-- dump( self._rankData, "self._rankData" )
	self._size = self:getContentSize()
	self._rightSize = cc.size( 700, 450 )
end

function ArenaRankPop:initUI()
	-- 标题背景云
	local titleCloud = cc.Sprite:create( "res/Main/common/title_bg_cloud.png" )
	self:addContent( titleCloud )
	-- 背景
	-- local background = cc.Sprite:create( "res/Main/common/popBg/popBg_red.png" )
	local background = zc.createNormalPop({
		src = "res/Main/common/popBg/popBg_red.png",
		bianZOrder = 0,
		height = 527 + 31,
	})
	background:setPosition( self._size.width/2, self._size.height/2 )
	self:addContent( background )
	self._bg = background
	self._bgSize = background:getContentSize()

	titleCloud:setAnchorPoint( 0, 0.5 )
	titleCloud:setPosition( self._size.width/2 - background:getContentSize().width/2 - 80, self._size.height/2 + background:getContentSize().height/2 )
	-- 标题背景
	local titleBg = cc.Sprite:create( "res/Main/common/title_bg_shadow.png" )
	titleBg:setAnchorPoint( 0, 0.5 )
	titleBg:setPosition( -30, self._bgSize.height )
	self._bg:addChild( titleBg )
	-- 标题
	local title = cc.Sprite:create( "res/Main/arena/rank/title.png" )
	title:setAnchorPoint( 0, 0.5 )
	title:setPosition( -5, self._bgSize.height )
	self._bg:addChild( title )
	-- 左边标签
	self:initLeft()
	-- 右边
	self:initRight()
	-- 关闭按钮
	local closeBtn = zc.createBackButton({
		pos = cc.p( self._bgSize.width, self._bgSize.height ),
		listener = function ()
			--功能引导
			
			self:hide()
			local _currentGID = getGuideManager():getCurrentGID()
			if _currentGID == GuideKey.KEY_12207 then
				zc.createArenaShopLayer()
			end
		end
	})
	self._bg:addChild( closeBtn )
    self._guide_btn = closeBtn

	-- 入场动画
	self:initAnimation()
end
-- 初始化左侧
function ArenaRankPop:initLeft()
	self._typeBtn = {}
	-- 文字,key值为type,
	local textTmp = {
		["1"] = LANGUAGE_ARENA.bang_text1,
		["2"] = LANGUAGE_ARENA.bang_text2,
		["3"] = LANGUAGE_ARENA.bang_text3,
		["11"] = LANGUAGE_ARENA.bang_text11,
		["12"] = LANGUAGE_ARENA.bang_text12,
	}
	for i = 1, #self._typeConfig do
		-- 按钮
		local typeBtn = zc.createTypeBtn({
	        btnText = textTmp[tostring(self._typeConfig[i])],
	        pos = cc.p( 86, 500 - 72*i ),
	        endCallback = function() end,
		})
		self._bg:addChild( typeBtn )
		self._typeBtn[i] = typeBtn

		-- 监听事件
		typeBtn:setTouchEndedCall(function()
			if self._index ~= i then
	   			self._typeBtn[i]:setSelected(true)
	   			self._typeBtn[self._index]:setSelected(false)
	   			if not self._rankData["type"..self._typeConfig[i]] or #self._rankData["type"..self._typeConfig[i]] == 0 then
            		-- 没有数据，需要请求
	            	zc.sendHttp({
	            		apiName = "Arena_rank",
	            		params = {types=self._typeConfig[i]},
	            		successCallback = function(data)
				        	for k, v in pairs( data.ranks ) do
				        		table.sort(v, function( a, b )
									return a.rank < b.rank
								end)
				        		self._rankData[k] = v
				        	end
							self._index = i
							self:initAnimation()
						end,
				        failCallback = function()
				        	self._typeBtn[i]:setSelected(false)
				        	self._typeBtn[self._index]:setSelected(true)
				        end,
				        isSilent = true
			        })
				else
					-- 有数据，直接用
					self._index = i
					self:initAnimation()
				end
			end
        end)
	end
	self._typeBtn[self._index]:setSelected( true )
end
-- 初始化右侧
function ArenaRankPop:initRight()
	-- 尺寸
	local topHeight = 40
	local middleHeight = 355
	local bottomHeight = self._rightSize.height - topHeight - middleHeight
	-- 容器
	local rightContainer = cc.Sprite:create()
	rightContainer:setContentSize( self._rightSize )
	rightContainer:setAnchorPoint( 0, 0.5 )
	rightContainer:setPosition( 163, 245 )
	self._bg:addChild( rightContainer )
	self._rightContainer = rightContainer
	-- 背景
    local rightBg = ccui.Scale9Sprite:create( cc.rect( 25, 26, 3, 2 ), "res/Main/common/common_inner20.png" )
    rightBg:setContentSize( self._rightSize.width + 2, self._rightSize.height )
    rightBg:setPosition( self._rightSize.width/2, self._rightSize.height/2 )
    rightContainer:addChild( rightBg )

    -- 顶部
    -- 顶部背景
    local topBg = cc.Sprite:create( "res/Main/arena/rank/topBg.png" )
    topBg:setPosition( self._rightSize.width/2, self._rightSize.height - topHeight/2 )
    rightContainer:addChild( topBg )
    -- 排名文字
    local rankText = zc.createLabel({
    	text = LANGUAGE_ARENA.rank,
        fontSize = 18,
        color = cc.c3b( 76, 18, 18 ),
        pos = cc.p( 95, topHeight/2 - 3 ),
	})
	topBg:addChild( rankText )
	-- 名字文字
    local nameText = zc.createLabel({
    	text = LANGUAGE_ARENA.name,
        fontSize = 18,
        color = cc.c3b( 76, 18, 18 ),
        pos = cc.p( 360, topHeight/2 - 3 ),
	})
	topBg:addChild( nameText )
	-- 积分文字
    local scoreText = zc.createLabel({
    	text = LANGUAGE_ARENA.score,
        fontSize = 18,
        color = cc.c3b( 76, 18, 18 ),
        pos = cc.p( 560, topHeight/2 - 3 ),
	})
	topBg:addChild( scoreText )

    -- 中间
    -- 顶部滑动阴影
    local topShadow = cc.Sprite:create( "res/Main/arena/rank/shadow.png" )
    topShadow:setAnchorPoint( 0.5, 1 )
    topShadow:setPosition( self._rightSize.width/2, self._rightSize.height - topHeight )
    rightContainer:addChild( topShadow, 1 )
    self._topShadow = topShadow
    topShadow:setVisible( false )
    local topArrow = cc.Sprite:create( "res/Main/common/arrow_left.png" )
    topArrow:setRotation( 90 )
    topArrow:setScale( 0.5 )
    topArrow:setAnchorPoint( 0, 0.5 )
    topArrow:setPosition( topShadow:getContentSize().width/2, topShadow:getContentSize().height )
    topShadow:addChild( topArrow )
    topArrow:runAction(
    	cc.RepeatForever:create(
    		cc.Sequence:create(
	    		cc.MoveTo:create(
	    			2, cc.p( topShadow:getContentSize().width/2, topShadow:getContentSize().height - 10 )
				),
	    		cc.MoveTo:create(
	    			2, cc.p( topShadow:getContentSize().width/2, topShadow:getContentSize().height )
				)
			)
		)
	)
    -- 底部滑动阴影
    local bottomShadow = cc.Sprite:create( "res/Main/arena/rank/shadow.png" )
    bottomShadow:setFlippedY( true )
    bottomShadow:setAnchorPoint( 0.5, 0 )
    bottomShadow:setPosition( self._rightSize.width/2, bottomHeight )
    rightContainer:addChild( bottomShadow, 1 )
    self._bottomShadow = bottomShadow
    bottomShadow:setVisible( false )
    local bottomArrow = cc.Sprite:create( "res/Main/common/arrow_left.png" )
    bottomArrow:setRotation( -90 )
    bottomArrow:setScale( 0.5 )
    bottomArrow:setAnchorPoint( 0, 0.5 )
    bottomArrow:setPosition( topShadow:getContentSize().width/2, 0 )
    bottomShadow:addChild( bottomArrow )
    bottomArrow:runAction(
    	cc.RepeatForever:create(
    		cc.Sequence:create(
	    		cc.MoveTo:create(
	    			2, cc.p( topShadow:getContentSize().width/2, 10 )
				),
	    		cc.MoveTo:create(
	    			2, cc.p( topShadow:getContentSize().width/2, 0 )
				)
			)
		)
	)
	-- tableview
	self._tableviewSize = cc.size( self._rightSize.width, middleHeight )
	local rankTabelview = zc.ZCTableView:create( self._tableviewSize )
    rankTabelview:setBounceable( true )
    rankTabelview:setInertia( true )
    rankTabelview:setDirection( cc.SCROLLVIEW_DIRECTION_VERTICAL )
    rankTabelview:setPosition( cc.p( 0, bottomHeight ) )
    rankTabelview:setDelegate()
    rankTabelview:setVerticalFillOrder( cc.TABLEVIEW_FILL_TOPDOWN )
    local cellWidth = self._rightSize.width
    local cellHeight = 90
    -- 屏幕上显示的cell数量
    local cellShowNum = math.ceil( self._tableviewSize.height/cellHeight )
    local function cellSizeForTable(table, idx) 
        return cellHeight, cellWidth
    end
    local function tableCellAtIndex(table, cellIndex)
        local cell = rankTabelview:dequeueCell()
        -- 创建模板
        if cell == nil then
            cell = zc.ZCTableViewCell:new()
            self:buildCell( cell, cellIndex, cellWidth, cellHeight )
        end
        cell._cellBg:setScale( 1 )
        -- 更新界面
        self:updateCell( cell, cellIndex, cellHeight )
        return cell
    end
    local function numberOfCellsInTableView()
    	if self._rankData["type"..self._typeConfig[self._index]] then
    		return #self._rankData["type"..self._typeConfig[self._index]]
    	else
    		return 0
		end
    end
    local function scorllviewScriptScroll( table )
    	local cellNum = numberOfCellsInTableView()
    	if cellNum > cellShowNum then
        	self:scorllviewScriptScroll( table, cellNum )
        end
    end
    rankTabelview:registerScriptHandler( cellSizeForTable, cc.TABLECELL_SIZE_FOR_INDEX )
    rankTabelview:registerScriptHandler( tableCellAtIndex, cc.TABLECELL_SIZE_AT_INDEX )
    rankTabelview:registerScriptHandler( numberOfCellsInTableView, cc.NUMBER_OF_CELLS_IN_TABLEVIEW )
    rankTabelview:registerScriptHandler( scorllviewScriptScroll, zctech.SCROLLVIEW_SCRIPT_SCROLL )
    rightContainer:addChild(rankTabelview)
    self._rankTabelview = rankTabelview
    
    -- 底部
    -- 我的排名
    local myRankText = zc.createLabel({
    	text = LANGUAGE_ARENA.rank_mine..":",
        fontSize = 18,
        color = cc.c3b( 76, 18, 18 ),
        ap = cc.p( 1, 0.5 ),
        pos = cc.p( 130, bottomHeight/2 ),
	})
	rightContainer:addChild( myRankText )
    local myRankNum = zc.createLabel({
    	text = self._rank,
        fontSize = 24,
        color = display.COLOR_YELLOW,
        outLine = true, 
        pixel = 1.3,
        ap = cc.p( 0, 0.5 ),
        pos = cc.p( myRankText:getPositionX(), myRankText:getPositionY() + 3 ),
	})
	rightContainer:addChild( myRankNum )
	if self._typeConfig[self._index] ~= 11 then
		-- 天榜不显示积分
		-- 我的积分
	    local myScoreText = zc.createLabel({
	    	text = LANGUAGE_ARENA.score_mine..":",
	        fontSize = 18,
        	color = cc.c3b( 76, 18, 18 ),
	        ap = cc.p( 1, 0.5 ),
	        pos = cc.p( 315, bottomHeight/2 ),
		})
		rightContainer:addChild( myScoreText )
	    local myScoreNum = zc.createLabel({
	    	text = self._score,
	        fontSize = 24,
	        color = display.COLOR_YELLOW,
	        outLine = true,
	        pixel = 1.3,
	        ap = cc.p( 0, 0.5 ),
	        pos = cc.p( myScoreText:getPositionX(), myScoreText:getPositionY() + 3 ),
		})
		rightContainer:addChild( myScoreNum )
	end
	-- 奖励预览
	local rewardBtn = zc.createPushButton({
        normalFile = "res/Main/common/button/btn_sec_u.png",
        selectedFile = "res/Main/common/button/btn_sec_d.png",
        text = LANGUAGE_ARENA.reward_preview,
        fontSize = 30,
        scale = 0.7,
        outLine = true,
        color2 = cc.c3b(76, 18, 18),
        pos = cc.p( self._rightSize.width - 210, bottomHeight/2 ),
        endCallback = function()
	    	if tonumber(self._type) == tonumber(self._typeConfig[self._index]) then
	    		zc.createArenaRewardPop({rank = self._rank, type = self._typeConfig[self._index]})
	    	else
	    		local nowType = self._typeConfig[self._index] or 1
	    		local tipStr = LANGUAGE_ARENA["arena_type_"..nowType] or ""
	    		zc.tipInfo(LANGUAGE_ARENA.un_into .. tipStr .. "!")
	    	end
        end
    })
	rightContainer:addChild( rewardBtn )
	-- 规则说明
	local rulesBtn = zc.createPushButton({
        normalFile = "res/Main/common/button/btn_sec_u.png",
        selectedFile = "res/Main/common/button/btn_sec_d.png",
        text = LANGUAGE_ARENA.rule,
        fontSize = 30,
        scale = 0.7,
        outLine = true,
        color2 = cc.c3b(76, 18, 18),
        pos = cc.p( self._rightSize.width - 75, bottomHeight/2 ),
        endCallback = function()
        	zc.createRulePop( 1 )
        end
    })
	rightContainer:addChild( rulesBtn )
end
function ArenaRankPop:buildCell( cell, cellIndex, cellWidth, cellHeight )
	local middleY = cellHeight/2
	-- 背景
	local cellBg = cc.Sprite:create()
	cellBg:setContentSize( 694, 87 )
	cellBg:setPosition( cellWidth/2, middleY )
	cell:addChild( cellBg )
	cell._cellBg = cellBg
	-- 排名数字
	local rankNum = zc.createFntLabel({
		fnt = "res/Main/arena/rank/fnt/arenaRank_num.fnt",
        pos = cc.p( 95, middleY ),
	})
	cellBg:addChild( rankNum )
	cell._rankNum = rankNum
	-- 排名图标
	local rankIcon = cc.Sprite:create()
	rankIcon:setContentSize( 72, 67 )
	rankIcon:setPosition( rankNum:getPosition() )
	cellBg:addChild( rankIcon )
	cell._rankIcon = rankIcon
	-- 分隔线
	local split = cc.Sprite:create( "res/Main/arena/rank/split.png" )
	split:setPosition( 177, middleY )
	cellBg:addChild( split )
	-- 名字
	local name = zc.createLabel({
        fontSize = 20,
        color = cc.c3b( 76, 18, 18 ),
        ap = cc.p( 0, 0.5 ),
        pos = cc.p( 338, middleY ),
	})
	cellBg:addChild( name )
	cell._name = name
	-- 积分
	local score = zc.createLabel({
        fontSize = 20,
        outLine = true,
        pixel = 1.3,
        ap = cc.p( 0, 0.5 ),
        pos = cc.p( 545, middleY ),
	})
	cellBg:addChild( score )
	cell._score = score
	-- btn
	local btn = ccui.Widget:create()
	btn:setContentSize( cellBg:getContentSize() )
	btn:setPosition( cellBg:getContentSize().width/2, cellBg:getContentSize().height/2 )
	btn:setTouchEnabled( true )
	btn:setSwallowTouches( false )
	btn:onTouch(function(event)
		if event.name == "began" then
            cellBg:runAction(
                cc.ScaleTo:create( 0.05, 0.9 )
            )
        elseif event.name == "moved" then
        elseif event.name == "ended" then
            cellBg:runAction(
                cc.ScaleTo:create( 0.05, 1 )
            )
            local _data = self._rankData["type"..self._typeConfig[self._index]][cell._index] or {}
        	zc.createOtherPlayerSelect( _data.userId )
        elseif event.name == "cancelled" then
            cellBg:runAction(
                cc.ScaleTo:create( 0.05, 1 )
            )
        end
	end)
	cellBg:addChild( btn )
end
function ArenaRankPop:updateCell( cell, cellIndex, cellHeight )
	cell._index = cellIndex + 1
	-- 数据
	local data = self._rankData["type"..self._typeConfig[self._index]][cellIndex + 1]
	-- dump( data, "data" )
	-- 背景，排名图标，排名数字
	if data.rank >= 1 and data.rank <= 3 then
		cell._cellBg:setTexture( "res/Main/arena/rank/bg_"..data.rank..".png" )
		cell._rankIcon:setVisible( true )
		cell._rankIcon:setTexture( "res/Main/arena/rank/icon_"..data.rank..".png" )
		cell._rankNum:setVisible( false )
	else
		cell._cellBg:setTexture( "res/Main/arena/rank/bg_4.png" )
		cell._rankIcon:setVisible( false )
		cell._rankNum:setVisible( true )
		cell._rankNum:setString( data.rank )
	end
	-- 头像
	local _iconId = 1
	if cell._heroIcon then
		cell._heroIcon:removeFromParent()
		cell._heroIcon = nil
	end
    if tonumber( data.userId ) > 0 then
    	_iconId = data.iconId
    else
    	_iconId = tonumber( data.iconId ) or 101
    	_iconId = _iconId%100
    -- 	local _weaponId = tonumber( data.weaponId ) or 1001
    -- 	_weaponId = math.floor( _weaponId/1000 )
    -- 	local _sex = tonumber( data.iconId ) or 101
    -- 	_sex = _sex%100
    -- 	_iconId = _weaponId*2 + _sex - 2
    end
	local heroIcon = zc.createPlayerHead({iconId = _iconId, level = data.level})
    heroIcon:setPosition( 232, cellHeight/2 )
    heroIcon:setScale( 0.85 )
    cell._cellBg:addChild( heroIcon )
    cell._heroIcon = heroIcon
	-- 名字
	cell._name:setText( data.userName )
	-- 积分
	cell._score:setText( data.scores )
end

-- 入场动画
function ArenaRankPop:initAnimation()
	self._topShadow:setVisible( false )
	self._rankTabelview:reloadData()
    local cellHeight = 90
    local cellNum = math.ceil( self._tableviewSize.height/cellHeight )
    for i = 1, cellNum do
    	local cell = self._rankTabelview:cellAtIndex( i - 1 )
    	if cell then
    		cell:stopAllActions()
	    	cell:setPosition( 0, self._rankTabelview:getContentSize().height - cellHeight - self._tableviewSize.height )
	    	cell:runAction(
	    		cc.Sequence:create(
	    			cc.DelayTime:create( i*0.15 ),--math.sqrt( ( i - 1 )*0.05 ) ),
					cc.EaseExponentialOut:create( cc.MoveTo:create( 0.8, cc.p( 0, self._rankTabelview:getContentSize().height - cellHeight*i ) ) )
				)
			)
    	end
    end
end

function ArenaRankPop:scorllviewScriptScroll( table, cellNum )
	local cellHeight = 90
	if table:getContentOffset().y >= 0 then
    	self._topShadow:setVisible( true )
    	self._bottomShadow:setVisible( false )
    elseif table:getContentOffset().y <= self._tableviewSize.height - cellHeight*cellNum then
    	self._topShadow:setVisible( false )
    	self._bottomShadow:setVisible( true )
    else
    	self._topShadow:setVisible( true )
    	self._bottomShadow:setVisible( true )
    end
end

function ArenaRankPop:onEnter()
	--功能引导
	local _currentGID = getGuideManager():getCurrentGID()
	if inArray(_currentGID, GuideGroup.DUI_HUA) then
		local function _callBack()
            getGuideManager():setObjTarget(self._guide_btn)
            getGuideManager():startGuide()
        end
        getGuideManager():guideOver()
        getGuideManager():setHeroFuncCallBack(_callBack)
        getGuideManager():startGuide()
	end
end

return ArenaRankPop