--[[
	奖励预览
	唐实聪
	2016.3.24
]]
local ArenaRewardPop = class("ArenaRewardPop", function()
	return ZCPopLayer:create()
end)

function ArenaRewardPop:create( params )
	return ArenaRewardPop.new( params )
end

function ArenaRewardPop:ctor( params )
	self:show()
	self:initData( params )
	self:initUI()
end
function ArenaRewardPop:initData( params )
	local myRank = params.rank or 0
	local myType = params.type or 11

	local rewardList = gameData.getDataFromCache( "arena_award" )
	local _nums = table.nums(rewardList)
	local rewardData = {}
	for i=1, _nums do
		local v = rewardList[tostring(i)]
		if tonumber( v.type ) == myType then
			rewardData[#rewardData + 1] = v
		end
	end
	table.sort(rewardData, function( a, b )
		return a.id < b.id
	end)
	-- 找到自己所在档
	local myIndex = 0
	for i, v in ipairs( rewardData ) do
		if myRank <= tonumber( v.maxRank ) and myRank >= tonumber( v.minRank ) then
			myIndex = i
		end
	end
	-- 找到上下两档
	self._rewardData = {}
	self._index = 0
	if myIndex <= 3 then
		-- 开始五档
		for i = 1, 5 do
			self._rewardData[#self._rewardData + 1] = rewardData[i]
		end
		self._index = myIndex
	elseif myIndex >= #rewardData - 2 then
		-- 最后五档
		for i = #rewardData - 4, #rewardData do
			self._rewardData[#self._rewardData + 1] = rewardData[i]
		end
		self._index = 5 - ( #rewardData - myIndex )
	else
		-- 上下两档
		for i = myIndex - 2, myIndex + 2 do
			self._rewardData[#self._rewardData + 1] = rewardData[i]
		end
		self._index = 3
	end

	self._size = self:getContentSize()
end

function ArenaRewardPop:initUI()
	-- 背景
	local background = cc.Sprite:create( "res/Main/common/popBg/popBg_tip_blue.png" )
	background:setPosition( self._size.width/2, self._size.height/2 )
	self:addContent( background )
	self._bg = background
	self._bgSize = background:getContentSize()
	-- 关闭按钮
	local closeBtn = zc.createBackButton({
		pos = cc.p( self._bgSize.width, self._bgSize.height ),
		listener = function ()
			self:hide()
		end
	})
	self._bg:addChild( closeBtn )

	-- 奖励预览标题
	local title = cc.Sprite:create( "res/Main/arena/reward/title.png" )
	title:setPosition( self._bgSize.width/2, self._bgSize.height - 25 )
	self._bg:addChild( title )
	-- 背景
	self._mbgSize = cc.size( 605, 290 )
	-- 五个
	for i, v in ipairs( self._rewardData ) do
		-- cellBg
		local bgPath = ""
		local fontColor = display.COLOR_WHITE
		local fntPath = ""
		local noPath = ""
		if i == self._index then
			bgPath = "res/Main/arena/reward/cellBg_2.png"
			fontColor = display.COLOR_YELLOW
			fntPath = "res/Main/arena/reward/fnt/shuzi_2.fnt"
			noPath = "res/Main/arena/reward/no_2.png"
		else
			bgPath = "res/Main/arena/reward/cellBg_1.png"
			-- fontColor = display.COLOR_WHITE
			fntPath = "res/Main/arena/reward/fnt/shuzi_1.fnt"
			noPath = "res/Main/arena/reward/no_1.png"
		end
		local cellBg = cc.Sprite:create( bgPath )
		cellBg:setPosition( self._mbgSize.width/2 + 12, self._mbgSize.height/10*( 11 - i*2 ) + 13 )
		self._bg:addChild( cellBg )
		local middleY = cellBg:getContentSize().height/2
		-- split
		local split = ccui.Scale9Sprite:create( cc.rect( 0, 0, 2, 80 ), "res/Main/arena/rank/split.png" )
		split:setContentSize( 2, cellBg:getContentSize().height - 5 )
		split:setPosition( 230, middleY )
		cellBg:addChild( split )
		-- 排名
		local _text = ""
		if tonumber( v.minRank ) == tonumber( v.maxRank ) then
			_text = v.minRank
		else
			_text = v.minRank.."-"..v.maxRank
		end
		local rank = zc.createFntLabel({
	        fnt = fntPath,
	        text = _text,
	        pos = cc.p( split:getPositionX()/2 + 20, middleY - 3 ),
	        addwidth = -8,
	    })
		cellBg:addChild( rank )
		-- no
		local no = cc.Sprite:create( noPath )
		no:setPosition( rank:getPositionX() - rank:getContentSize().width/2 - no:getContentSize().width/2, middleY )
		cellBg:addChild( no )
		-- 奖励
		local itemTable = string.split( v.items, "," )
		for j, u in ipairs( itemTable ) do
			local rewardTable = string.split( u, "|" )
			local rewardInfo = zc._getIconInfo({data={mainType=rewardTable[1],type=rewardTable[2],id=rewardTable[2],num=rewardTable[3]}})
			local reward = cc.Sprite:create( rewardInfo.img )
			reward:setScale( 40/reward:getContentSize().width )
			cellBg:addChild( reward )
			local rewardNum = zc.createLabel({
		        text = "x"..zc.getWan( rewardInfo.num ),
		        fontSize = 18,
		        color = fontColor,
		        ap = cc.p( 0, 1 ),
		        outLine = true,
		    })
			cellBg:addChild( rewardNum )
			local _width = reward:getBoundingBox().width + rewardNum:getContentSize().width
			reward:setPosition( ( 2*j - 1 )*( cellBg:getContentSize().width - split:getPositionX() )/( #itemTable*2 ) + split:getPositionX() - _width/2 + 20, middleY )
			rewardNum:setPosition( reward:getPositionX() + reward:getBoundingBox().width/2, reward:getPositionY() )
		end
		
	end
end
return ArenaRewardPop