--[[
	BattleUILevelUpPop.lua
	主角升级
]]
local BattleUILevelUpPop = class("BattleUILevelUpPop",function ()
    return ZCDialog:create(255*0.1)
end)

function BattleUILevelUpPop:create( data )
    return BattleUILevelUpPop.new( data )
end

function BattleUILevelUpPop:ctor( data )
    self._isAdd = data.isAdd
    self:initUI(data)
    self:setTouchEndedCallback(function()
        self:setDoClose()
    end)
end

function BattleUILevelUpPop:initUI( data )
    local _spine = zc.getSpine("shengji")
    _spine:setPosition(zc.width*0.5, zc.height*0.5)
    self:addChild(_spine)
    _spine:playAction("1", false)
    _spine:registerSpineEventHandler( function ( event )
        if event.animation == "1" then
            _spine:playAction("2", true)
        end
    end, sp.EventType.ANIMATION_COMPLETE)

    local congratulateLabel = zc.createLabel({
        text = zc.lang("level_up_to") .. " ",
        fontSize = 28,
        ap = cc.p(0, 0.5),
        pos = cc.p(zc.width*0.5, zc.height*0.5),
        outLine = true,
        color = display.COLOR_YELLOW,
    })
    self:addChild(congratulateLabel)
    self._congratulateLabel = congratulateLabel
    local congratulateLevel = zc.createFntLabel({
        fnt = "res/Main/common/fnt/fight_change.fnt",
        text = PlayerData:getUserLevel(),
        addwidth = -12,
        ap = cc.p(0, 0.5),
        pos = cc.p(congratulateLabel:getContentSize().width, congratulateLabel:getContentSize().height*0.5 + 2),
    })
    congratulateLabel:addChild(congratulateLevel)
    local congratulateLabel2 = zc.createLabel({
        text = zc.lang("level_1"),
        fontSize = 28,
        ap = cc.p(0, 0.5),
        pos = cc.p(congratulateLabel:getContentSize().width + congratulateLevel:getContentSize().width, congratulateLabel:getContentSize().height*0.5),
        outLine = true,
        color = display.COLOR_YELLOW,
    })
    congratulateLabel:addChild(congratulateLabel2)
    local _width = congratulateLabel:getContentSize().width + congratulateLevel:getContentSize().width + congratulateLabel2:getContentSize().width
    congratulateLabel:setPositionX(zc.width*0.5 - _width*0.5)
    
    self._congratulateLabel:runAction(cc.Sequence:create(
        cc.MoveBy:create(0.1, cc.p(0, 55)),
        cc.MoveBy:create(0.05, cc.p(0, -10)),
        cc.MoveBy:create(0.02, cc.p(0, 5))
    ))

    local _tb = {
        {
            tip = zc.lang("now_tili") .. "：",
            now = PlayerData.resourcesInfo.energy,
            last = PlayerData._lastEnergy or PlayerData.resourcesInfo.energy,
        },
        {
            tip = zc.lang("max_tili") .. "：",
            now = PlayerData.resourcesInfo.energyLimit,
            last = PlayerData._lastEnergyLimit or PlayerData.resourcesInfo.energyLimit,
        },
        {
            tip = zc.lang("main_role_fight") .. "：",
            now = PlayerData:getPlayerHero().fighting,
            last = PlayerData._lastFighting or PlayerData:getPlayerHero().fighting,
        },
    }

    local _diPosY = zc.height*0.5 - 40

    for i=1, #_tb do
        local __data = _tb[i]
        local _node = zc.getSprite({
            src = "res/Main/battle/battleresult/levelip_openDi.png",
            pos = cc.p(zc.width*0.5, _diPosY - (i-2)*45)
        })
        _node:setOpacity(255*0.5)
        _node:setScaleY(0.5)
        self:addChild(_node)
        local energyTip = zc.createLabel({
            text = __data.tip,
            fontSize = 22,
            ap = cc.p(1, 0.5),
            pos = cc.p(_node:getContentSize().width*0.5 - 50, 35),
            color = cc.c3b(187, 182, 149),
            outLine = true,
        })
        energyTip:setScaleY(1/0.5)
        _node:addChild(energyTip)
        local lastNum = zc.createLabel({
            text = zc.getWan(__data.last),
            fontSize = 22,
            ap = cc.p(0.5, 0.5),
            pos = cc.p(_node:getContentSize().width*0.5, energyTip:getPositionY()),
            color = cc.c3b(226, 203, 90),
            outLine = true,
        })
        lastNum:setScaleY(1/0.5)
        _node:addChild(lastNum)

        local arrowImg = zc.getSprite({
            src = "res/Main/common/arrow_green.png",
            ap = cc.p(0.5, 0.5),
            pos = cc.p(_node:getContentSize().width*0.5 + 60, energyTip:getPositionY()),
        })
        arrowImg:setScaleY(1/0.5)
        _node:addChild(arrowImg)

        local nowNUM = zc.createLabel({
            text = zc.getWan(__data.now),
            fontSize = 22,
            ap = cc.p(0.5, 0.5),
            pos = cc.p(_node:getContentSize().width*0.5 + 140, energyTip:getPositionY()),
            color = display.COLOR_GREEN,
            outLine = true,
        })
        nowNUM:setScaleY(1/0.5)
        _node:addChild(nowNUM)

        _node:setVisible(false)
        _node:runAction(cc.Sequence:create(
            cc.DelayTime:create(0.05*i),
            cc.CallFunc:create(function()
                _node:setVisible(true)
                zc.doFuncForAllChild(_node, function( child )
                    child:setOpacity(0)
                    if child == _node then
                        child:runAction(cc.FadeTo:create(0.16, 255*0.5))
                    else
                        child:runAction(cc.FadeTo:create(0.16, 255))
                    end
                end)
            end)
        ))
    end


    -- 开启功能数据
    local _data = data or {}
    local myLevel = PlayerData:getUserLevel()
    myLevel = tonumber(_data.level) or myLevel
    local nextFunction = zc.getNextOpenFuncInfo(myLevel)
    -- dump( nextFunction, "nextFunction" )
    if not nextFunction then
        return
    end

    -- 有新功能
    local goFlag = false
    local nextLevel = tonumber(nextFunction.userLevel) or 1
    local _funcId = tonumber(nextFunction.functionId) or 0
    local _tmpGID = 10001 + _funcId*100
    if nextLevel ~= 1 and nextLevel == myLevel and GuideKey["KEY_" .. _tmpGID] then
        -- 开启
        goFlag = true
        getGuideManager():setCurrentGID(GuideKey["KEY_" .. _tmpGID], true) 
    end
 
    local tipIconBg = zc.getSprite({
        src = "res/Main/battle/battleresult/battle_levelup_quan.png",
        pos = cc.p(zc.width*0.5 - 140, zc.height*0.5 - 210),
    })
    self:addChild(tipIconBg, 1)
    local _iconBgSize = tipIconBg:getContentSize()
    local _descDiSp = zc.createSprite("res/Main/battle/battleresult/levelip_openDi.png")
    _descDiSp:setScaleY(1.1)

    local _descDi = cc.Node:create()
    _descDi:setContentSize(_descDiSp:getBoundingBox().width, _descDiSp:getBoundingBox().height)
    _descDi:setAnchorPoint(0, 0.5)
    _descDi:setPosition(0, _iconBgSize.height*0.5)
    tipIconBg:addChild(_descDi, -1)
    _descDi:addChild(_descDiSp)

    local tipIcon = zc.createPushButton({
        normalFile = "res/Main/common/common_icon_"..nextFunction.icon..".png",
        pos = cc.p(_iconBgSize.width*0.5, _iconBgSize.height*0.5),
        scale = 0.6,
        endCallback = function()
            --功能引导
            local _currentGID = getGuideManager():getCurrentGID()
            if inArray(_currentGID, GuideGroup.BTN_FUNC_ICON) then
                zc.setGuideFuncNextStep(_currentGID)
            end

            if goFlag then
                local _guideFuncId = tonumber(nextFunction.functionId)
                if _guideFuncId == 6
                  or _guideFuncId == 12
                  or _guideFuncId == 14
                  or _guideFuncId == 15
                  or _guideFuncId == 16
                  or _guideFuncId == 19
                  or _guideFuncId == 20
                  or _guideFuncId == 21
                  or _guideFuncId == 22
                  or _guideFuncId == 23
                  or _guideFuncId == 24
                  or _guideFuncId == 25
                  or _guideFuncId == 26
                  or _guideFuncId == 27
                  or _guideFuncId == 29
                  or _guideFuncId == 31
                  or _guideFuncId == 32
                  or _guideFuncId == 35
                  or _guideFuncId == 39
                  then
                    local _scene = cc.Director:getInstance():getRunningScene()
                    local _flag
                    if _scene:getName() == LAYER_NAME_TAG.LAYER_BATTLE then
                        _flag = false
                    end
                    if _guideFuncId == 15 then
                        LayerManager.backToMainCity(_flag)
                        zc.createChapter()
                    else
                        if _guideFuncId == 12
                          or _guideFuncId == 16
                          or _guideFuncId == 19
                          or _guideFuncId == 20
                          or _guideFuncId == 21
                          or _guideFuncId == 22
                          or _guideFuncId == 23
                          or _guideFuncId == 24
                          or _guideFuncId == 25
                          or _guideFuncId == 26
                          or _guideFuncId == 27
                          or _guideFuncId == 31
                          or _guideFuncId == 32
                          or _guideFuncId == 35
                          or _guideFuncId == 39
                          then
                            local _mainCity = zc.getScene("mainCity")
                            if _mainCity then
                                _mainCity:setFuncOpenFlag(nextFunction)
                            end
                        end
                        LayerManager.backToMainCity(_flag)
                    end
                    return
                end
                zc.gotoFunction( nextFunction.functionId, nil, true)
            else
                self:setDoClose()
            end
        end
    })
    tipIconBg:addChild(tipIcon)
    self._tipIcon = tipIcon

    local _descDiSize = _descDi:getContentSize()
    _descDiSp:setPosition(_descDiSize.width*0.5, _descDiSize.height*0.5)

    local tipName = zc.createLabel({
        text = nextFunction.content,
        ap = cc.p(0, 1),
        pos = cc.p(110, _descDiSize.height - 3),
        fontSize = 22,
        color = cc.c3b(227, 206, 169),
        outLine = true,
    })
    _descDi:addChild( tipName )

    local descText = zc.createLabel({
        text = nextFunction.desc,
        ap = cc.p(0, 0),
        pos = cc.p(110, 3),
        width = 290,
        fontSize = 20,
        color = cc.c3b(143, 125, 91),
    })
    _descDi:addChild( descText )

    if not goFlag then
        local openLevel = zc.createLabel({
            ap = cc.p(1, 1),
            pos = cc.p(_descDiSize.width - 10, _descDiSize.height - 3),
            fontSize = 22,
            color = display.COLOR_RED,
            outLine = true,
        })
        _descDi:addChild( openLevel )
        if nextLevel == 1 then
            local tollgateInfo = gameData.getDataFromCacheWithPrimaryKey("tollgate", nextFunction.tollgateId) or {}
            openLevel:setString(zc.lang("finish_tollgate") .. tostring(tollgateInfo.name) .. LANGUAGE_ROLE.unlock)
        else
            openLevel:setString(LANGUAGE_UNLOCK_FUNCTION_TIP(nil, nil, nextLevel))
        end
    end
end

function BattleUILevelUpPop:setDoClose( )
    if self._isAdd then
        self:removeFromParent()
    else
        LayerManager.removeLayout(true)
    end
end

function BattleUILevelUpPop:onEnter()
    local _mask = getGuideManager():getMaskLayer()
    self:addChild(_mask, 100)
    performWithDelay(_mask, function()
        _mask:removeFromParent()
        _mask = nil
    end, 0.5)
     --功能引导
    local _currentGID = getGuideManager():getCurrentGID()
    if inArray(_currentGID, GuideGroup.BTN_FUNC_ICON) then
        getGuideManager():setObjTarget(self._tipIcon)
        getGuideManager():startGuide()
        -- zc.addGuideBtn(self._tipIcon)
    end
end

function BattleUILevelUpPop:onCleanup()
    local _cache = cc.Director:getInstance():getTextureCache()
    _cache:removeTextureForKey("res/Main/battle/battleresult/levelup_bian.png")
end

return BattleUILevelUpPop