--[[
登录界面
wcc 2015-10-19
]]

local LoginScene = class("LoginScene", function ()
    return ZCDialog:create()
end);
function LoginScene:create()
    return LoginScene.new()
end

function LoginScene:onCleanup()
    cc.Director:getInstance():getTextureCache():removeTextureForKey("res/image/login/loginBg.jpg")
    cc.Director:getInstance():getTextureCache():removeTextureForKey("res/animation/beijing_renwu/beijing_renwu.png")
    cc.Director:getInstance():getTextureCache():removeTextureForKey("res/animation/beijing_renwu/beijing_renwu2.png")
    cc.Director:getInstance():getTextureCache():removeTextureForKey("res/animation/beijing_renwu/beijing_renwu3.png")
    cc.Director:getInstance():getTextureCache():removeTextureForKey("res/animation/beijing_renwu/beijing_renwu4.png")
    zc.removeEventListener(CUSTOM_EVENT.EVENT_SELECT_SERVER)
end

function LoginScene:ctor()
    self._allUpdateList = {} -- 本次要更新的列表
    self._updateIndex = 0
    gameData.firstInit()
    musicManager.setEffectVolume(1)
    musicManager.setVolume(0.4)
    musicManager.playMusic(zc.bgm.login, true)
    self._loginLayerSize = {w = 400, h = 250}

	-- 背景图
    local bgImg = zc.getSprite({
        src = "res/image/login/loginBg.jpg",
        pos = cc.p(zc.width*0.5, zc.height*0.5),
    })
    self:addChild(bgImg)
    self._bgImg = bgImg
    bgImg:setVisible(false)
    zc.setBgSize(bgImg)--放大适配背景

    local bgSpine = zc.getSpine("beijing_renwu")
    bgSpine:setPosition(zc.width*0.5, zc.height*0.5)
    self:addChild(bgSpine)
    self._bgSpine = bgSpine
    bgSpine:setScale(bgImg:getScale())

    local effNode = cc.Node:create()
    self:addChild(effNode)
    self._effNode = effNode

    local _loginNode = cc.Node:create()
    self:addChild(_loginNode)
    self._loginNode = _loginNode
    _loginNode:setVisible(false)

    -----登录界面
    self._loginLayer = self:getLoginLayer()
    _loginNode:addChild(self._loginLayer)
    
    local versionId = zc.createVersionBtn()
    versionId:setColor(display.COLOR_BLACK)
    _loginNode:addChild(versionId)


    --[[清除缓存 ]]
    local clear_btn = zc.createCleanBtn()
    _loginNode:addChild(clear_btn)


    zc.addEventListenerWithNode({name = CUSTOM_EVENT.EVENT_SELECT_SERVER , callback = function(event)
        local data      = event.data
        local server    = data.server
        lastLogin       = server
        if lastLogin then
            -- dump( lastLogin ,"----lastLogin---")
            GAMEAPI = lastLogin.host
            SOCKET_LOGIN_SERVER = lastLogin.socketIp
            CHAT_SERVER_PORT = lastLogin.socketUserPort
            if self._passwordBoxSize then
                local _tab = tonumber(lastLogin.tab) or 1
                self._passwordBoxSize:setText(_tab.."区"..tostring(lastLogin.name))
            end
        end
    end , node = self})
end

function LoginScene:onEnter()
    if self._bgSpine then
        self._bgSpine:playAction("start", false)
        self._bgSpine:registerSpineEventHandler(function(event)
            if event.animation ~= "start" then
                return
            end
            if not tolua.isnull(self._loadLay) then
                self._loadLay:removeFromParent()
                self._loadLay = nil
            end
            self._bgSpine:playAction("idle", true)
            helper.collectMemory(true, nil, true)
            zc.getFightScene(3)
            zc.preloadMainCity(self, 0)
        end, sp.EventType.ANIMATION_COMPLETE)
        self._bgSpine:registerSpineEventHandler( function ( event )
            local name = event.eventData.name
            if name == "start" then
                if self._bgImg then
                    self._bgImg:setVisible(true)
                    self._bgImg:setOpacity(0)
                    self._bgImg:runAction(cc.FadeIn:create(0.3))
                end
                if self._effNode then
                    local _eff = zc.createAnimationArmature({
                        fileName = "firemap",
                        actName = "idle",
                        loop = true,
                    })
                    _eff:setPosition(zc.width*0.5, zc.height*0.5)
                    self._effNode:addChild(_eff)
                end
            elseif name == "hit" then
                self._loadLay = LoadingCircleLayer:create()
                self:addChild(self._loadLay, 100)
                if self._loginNode then
                    self._loginNode:setVisible(true)
                    zc.doFuncForAllChild(self._loginNode, function( child )
                        child:setOpacity(0)
                        child:runAction(cc.FadeIn:create(0.3))
                    end)
                end
            elseif name == "flash" then
                local whiteLayer = cc.LayerColor:create(cc.c4b(255, 255, 255, 255), zc.width + 100, zc.height + 100)
                whiteLayer:setOpacity(255*0.8)
                whiteLayer:setPosition(-50, -50)
                self:addChild(whiteLayer, 1000)
                whiteLayer:runAction(cc.Sequence:create(
                    cc.FadeTo:create(0.3, 0),
                    cc.RemoveSelf:create()
                ))
            end
        end, sp.EventType.ANIMATION_EVENT)
    end
end

---------登录界面
function LoginScene:getLoginLayer()
    local userTable = zc.LoadTable("userTable")
    local curUserName = ""
    if userTable then
        for i=1,#userTable do 
            if userTable[i]["login"] == 1 then
                curUserName = userTable[i]["name"]
                break
            end
        end
    end
    
    --显示log按钮
    local touchCount = 0
    SHOW_LOG_BUTTON_TEST = false
    local showLogBtn = zc.createPushButton({
        normalNode = zc.getScaleNode("res/Main/common/img_empty.png", cc.size(50, 50)),
        needSwallow = false,
        endCallback = function()
            print("clicked ... ======================")
            touchCount = touchCount + 1
            if touchCount > 15 then
                SHOW_LOG_BUTTON_TEST = true
                touchCount = 0
            end
        end,
        anchor = cc.p(0.5, 0.5),
        pos = cc.p(zc.width/2+180, zc.height/2+180)
    })
    self:addChild(showLogBtn, 100)

    local loginLayer = zc.createLayout({
        size = cc.size(self._loginLayerSize.w, self._loginLayerSize.h),
        pos = cc.p((zc.width-self._loginLayerSize.w)/2, 0),
    })
    --用户名  
    local accountBg = zc.createImageView({
        src = "res/Main/login/login_acbg.png",
        pos = cc.p(self._loginLayerSize.w/2, 210),
    })
    loginLayer:addChild(accountBg)
    local accountImg = zc.createLabel({
        text = "账 号:",
        fontSize = 24,
        ttf = zc.getFont(1),
        ap = cc.p(0, 0.5),
        pos = cc.p(20, accountBg:getContentSize().height/2),
        shadow = true,
        color = display.COLOR_WHITE,
    })
    accountBg:addChild(accountImg)
    self._accountImg = accountImg
    --下拉选择用户
    local accountMore = zc.createImageView({
        src = "res/Main/login/login_acIcon.png",
        pos = cc.p(accountBg:getContentSize().width - 30, accountBg:getContentSize().height/2),
        listener = function () 
            local pop = requires("src/pop/moreUserLoginPop.lua"):create()
            self:addChild(pop)
        end
    })
    accountBg:addChild(accountMore)

    local accountBoxSize = cc.size(accountBg:getContentSize().width * 0.7, accountBg:getContentSize().height/4*3)
    local accountBox = ccui.EditBox:create(accountBoxSize, ("res/Main/common/img_empty.png"), 0)
    accountBox:setName("accountBox")
    --accountBox:setFontName("fonts/pw.ttf")
    accountBox:setFontSize(35)
    accountBox:setAnchorPoint(0, 0.5)
    accountBox:setText(curUserName)
    -- accountBox:setMaxLength(24)
    accountBox:setInputMode(cc.EDITBOX_INPUT_MODE_SINGLELINE)
    accountBox:setInputFlag(cc.EDITBOX_INPUT_FLAG_SENSITIVE)
    accountBox:setReturnType(cc.KEYBOARD_RETURNTYPE_DONE )
    accountBox:setPosition(90, accountBg:getContentSize().height/2+3)
    accountBg:addChild(accountBox)
    self._usernameBoxSize = accountBox
    --密码
    local passwordBg = zc.createImageView({
        src = "res/Main/login/login_acbg.png",
        pos = cc.p(self._loginLayerSize.w/2, 140),
        listener = function ()
            local _id = self._tag and 100 or 1
            local _channelId = GAMEAPI == GAMEAPI_EX and "5" or nil
            local _version = GAMEAPI == GAMEAPI_EX and "0.3.7" or nil

            local manifest      = zc.getManifest() or {}
            local manifest2      = zc.getManifest(1) or {}
            local _version = tostring(PACKAGE_VERSION) or "0.1"
            
            if manifest["version"] == nil then manifest["version"] = _version end

            zc.sendHttp({
                apiName = "Center_main",
                params = {
                    id = _id, 
                    passportId = accountBox:getText(),
                    channelId = _channelId,
                    version = _version,
                    codeVersion = manifest["version"],
                    tableVersion = manifest2["version"],
                    channelId = "5",
                },
                successCallback = function(data)
                    local pop = requires("src/layer/login/SelectServerPop.lua"):create(data)
                    if pop._exist then
                        self:addChild(pop)
                    end
                end,
            })
        end,
    })
    loginLayer:addChild(passwordBg)
    local serverLabel = zc.createLabel({
        text = "服务器:",
        fontSize = 20,
        ttf = zc.getFont(1),
        ap = cc.p(0, 0.5),
        pos = cc.p(15, passwordBg:getContentSize().height/2),
        shadow = true,
        color = display.COLOR_WHITE,
    })
    passwordBg:addChild(serverLabel)
    --密码输入框
    -- local passwordBoxSize = cc.size(passwordBg:getContentSize().width * 0.65, passwordBg:getContentSize().height/4*3)
    -- local passwordBox = ccui.EditBox:create(passwordBoxSize, ("res/Main/common/img_empty.png"), 0)
    -- passwordBox:setName("passwordBox")
    -- --passwordBox:setFontName("fonts/pw.ttf")
    -- passwordBox:setFontSize(35)
    -- passwordBox:setInputFlag(cc.EDITBOX_INPUT_FLAG_PASSWORD)
    -- passwordBox:setAnchorPoint(0, 0.5)
    -- passwordBox:setReturnType(cc.KEYBOARD_RETURNTYPE_DONE)
    -- passwordBox:setText("")
    -- passwordBox:setMaxLength(16)
    -- passwordBox:setInputMode(cc.EDITBOX_INPUT_MODE_SINGLELINE)
    -- passwordBox:setPosition(50, passwordBg:getContentSize().height/2+3)
    -- passwordBg:addChild(passwordBox)
    -- passwordBox:setTouchEnabled(false)
    local passwordBoxSize = zc.createLabel({
        text = "测试一服",
        fontSize = 25,
        ap = cc.p(0, 0.5),
        ttf = zc.getFont(1),
        pos = cc.p(100, passwordBg:getContentSize().height/2),
        --color = cc.c3b(0, 0, 0),
        --endCallback = function () zc.createPop("serverList") end,
    })
    self._passwordBoxSize = passwordBoxSize
    passwordBg:addChild(passwordBoxSize)
    --选择服务器
    local serverStatus = zc.createLabel({
        text = "火爆",
        fontSize = 25,
        stroke = true,
        ttf = zc.getFont(1),
        pos = cc.p(350, passwordBg:getContentSize().height/2),
        color = cc.c3b(230, 80, 0),
    })
    passwordBg:addChild(serverStatus)
    self.serverStatus = serverStatus

    --
    local urlBg = zc.createImageView({
        src = "res/Main/login/login_acbg.png",
        pos = cc.p(self._loginLayerSize.w/2, 210+72),
    })
    loginLayer:addChild(urlBg)


    local urlBoxSize = cc.size(urlBg:getContentSize().width, urlBg:getContentSize().height/4*3)
    local urlBox = ccui.EditBox:create(urlBoxSize, ("res/Main/common/img_empty.png"), 0)
    urlBox:setFontSize(35)
    urlBox:setAnchorPoint(0, 0.5)
    urlBox:setMaxLength(100)
    urlBox:setInputMode(cc.EDITBOX_INPUT_MODE_SINGLELINE)
    urlBox:setInputFlag(cc.EDITBOX_INPUT_FLAG_SENSITIVE)
    urlBox:setReturnType(cc.KEYBOARD_RETURNTYPE_DONE )
    urlBox:setPosition(20, urlBg:getContentSize().height/2+3)
    urlBg:addChild(urlBox)
    urlBox:setText("dev-trunk-rxsg.zhanchenggame.com")
    print("GAMEAPI ----------> ", GAMEAPI)

    --EditBox事件检测
    local handler = function(event,pSender)
        if event == "began" then
      
        end
        if event == "return" then
            local str = urlBox:getText()
            print("str return --> ", str)
        end
        if event == "ended" then
            local str = urlBox:getText()
            print("str ended--> ", str)
        end
        if event == "changed" then
            local str = urlBox:getText()
            print("str changed--> ", str)
            GAMEAPI = "https://"..str.."/index.php"

            print("GAMEAPI ---------> ", GAMEAPI)
        end
    end
    urlBox:registerScriptEditBoxHandler(handler)


    

    --登录按钮
    local loginButton = zc.createCommomBtn({
        tag = 1,
        pos = cc.p(self._loginLayerSize.w/2-120, 60), 
        text = "登 录",
        listener = function () 
            self:loginFunc(accountBox:getText(),"") 
        end,--passwordBox
    })
    loginLayer:addChild(loginButton)
    --跳到注册
    local regButton = zc.createCommomBtn({
        tag = 1,
        pos = cc.p(self._loginLayerSize.w/2+120, 60), 
        text = "注 册",
        listener = function () 
            loginLayer:removeFromParent()
            self._regLayer = self:getRegLayer()
            self:addChild(self._regLayer)
        end,
    })
    loginLayer:addChild(regButton)
    ---内网外网切换暂时使用---
    self._tag = false
    --账号切换默认
    local _text = "内网"
    if GAMEAPI == GAMEAPI_IN then
        _text = "内网"
        self._tag = false
    else
        _text = "外网"
        self._tag = true
    end
    self.changeButton = zc.createCommomBtn({
        tag = 1,
        pos = cc.p(self._loginLayerSize.w/2+300, 170), 
        text = _text,
        listener = function () 
            if self._tag then
                self._tag = false
                self.changeButton:setString("内 网")
                GAMEAPI        = GAMEAPI_IN                  --内网
                LOGIN_SERVER   = LOGIN_SERVER_IN             --内网
                CENTER_HOST_SERVER = LOGIN_SERVER_IN
                SOCKET_LOGIN_SERVER = SOCKET_LOGIN_SERVER_IN
                CHAT_SERVER_PORT = CHAT_SERVER_PORT_IN
            else
                self._tag = true
                self.changeButton:setString("外 网")              
                GAMEAPI        = GAMEAPI_EX                  --外网
                LOGIN_SERVER   = LOGIN_SERVER_EX             --外网
                CENTER_HOST_SERVER  = LOGIN_SERVER_EX
                SOCKET_LOGIN_SERVER = SOCKET_LOGIN_SERVER_EX
                CHAT_SERVER_PORT = CHAT_SERVER_PORT_EX
            end
        end,
    })
    loginLayer:addChild(self.changeButton)
    return loginLayer
end

---------注册界面
function LoginScene:getRegLayer()
    local regLayer = zc.createLayout({
        size = cc.size(self._loginLayerSize.w, self._loginLayerSize.h),
        pos = cc.p((zc.width-400)/2, 0),
    })
    --用户名  
    local accountBg = zc.createImageView({
        src = "res/Main/login/login_acbg.png",
        pos = cc.p(self._loginLayerSize.w/2, 210), 
    })
    regLayer:addChild(accountBg)
   local accountImg = zc.createLabel({
        text = "账 号:",
        fontSize = 24,
        ttf = zc.getFont(1),
        ap = cc.p(0, 0.5),
        pos = cc.p(20, accountBg:getContentSize().height/2),
        shadow = true,
        color = display.COLOR_WHITE,
    })
    accountBg:addChild(accountImg)
    local accountBoxSize = cc.size(accountBg:getContentSize().width * 0.7, accountBg:getContentSize().height/4*3)
    local accountBox = ccui.EditBox:create(accountBoxSize, ccui.Scale9Sprite:create("orange_edit.png"))
    accountBox:setName("accountBox")
    --accountBox:setFontName("fonts/pw.ttf")
    accountBox:setFontSize(35)
    accountBox:setAnchorPoint(0, 0.5)
    accountBox:setText("")
    -- accountBox:setMaxLength(16)
    accountBox:setInputMode(cc.EDITBOX_INPUT_MODE_SINGLELINE)
    accountBox:setInputFlag(cc.EDITBOX_INPUT_FLAG_SENSITIVE)
    accountBox:setReturnType(cc.KEYBOARD_RETURNTYPE_DONE )
    accountBox:setPosition(80, accountBg:getContentSize().height/2+3)
    accountBg:addChild(accountBox)
    self._accountBox = accountBox
    --密码
    -- local passwordBg = zc.createImageView({
    --     src = "res/Main/login/login_acbg.png",
    --     pos = cc.p(self._loginLayerSize.w/2, 150),
    -- })
    -- regLayer:addChild(passwordBg)
    -- local passImg = zc.createImageView({
    --     src = "res/Main/login/login_passwordIcon.png",
    --     pos = cc.p(30, passwordBg:getContentSize().height/2)
    -- })
    -- passwordBg:addChild(passImg)
    -- --密码输入框
    -- local passwordBoxSize = cc.size(passwordBg:getContentSize().width * 0.65, passwordBg:getContentSize().height/4*3)
    -- local passwordBox = ccui.EditBox:create(passwordBoxSize, ccui.Scale9Sprite:create("orange_edit.png"))
    -- passwordBox:setName("passwordBox")
    -- --passwordBox:setFontName("fonts/pw.ttf")
    -- passwordBox:setFontSize(35)
    -- passwordBox:setInputFlag(cc.EDITBOX_INPUT_FLAG_PASSWORD)
    -- passwordBox:setAnchorPoint(0, 0.5)
    -- passwordBox:setReturnType(cc.KEYBOARD_RETURNTYPE_DONE)
    -- passwordBox:setText("")
    -- passwordBox:setMaxLength(16)
    -- passwordBox:setInputMode(cc.EDITBOX_INPUT_MODE_SINGLELINE)
    -- passwordBox:setPosition(50, passwordBg:getContentSize().height/2+3)
    -- passwordBg:addChild(passwordBox)
    --登录按钮
    local loginButton = zc.createCommomBtn({
        tag = 1,
        pos = cc.p(self._loginLayerSize.w/2-120, 120), 
        text = "登 录",
        listener = function () 
            regLayer:removeFromParent()
            self._loginLayer = self:getLoginLayer()
            self:addChild(self._loginLayer)
                
        end,
    })
    regLayer:addChild(loginButton)
    --注册按钮
    local regButton = zc.createCommomBtn({
        tag = 1,
        pos = cc.p(self._loginLayerSize.w/2+120, 120), 
        text = "注 册",
        listener = function () self:regFunc(accountBox:getText()) end,
    })
    regLayer:addChild(regButton)

    return regLayer
end
function LoginScene:setServerName(serverTable)
    -- dump(serverTable, "---serverTable---")
    -- print("-----serverTable.ip------",serverTable.ip)
    GAMEAPI = "https://"..serverTable.host.."/index.php"
    self._passwordBoxSize:setString(serverTable.name)
    -- 当前服务器IP
    LOGIN_SERVER = serverTable.host
    --服务器列表ip
    CENTER_HOST_SERVER = serverTable.centerHost
    --登录端口
    -- LOGIN_SERVER_PORT = serverTable.port
    --SOCKET登录IP
    SOCKET_LOGIN_SERVER = serverTable.socketIp
    --聊天端口
    CHAT_SERVER_PORT = serverTable.socketUserPort
    --私钥
    PRIVATEKEY = serverTable.privateKey
end
function LoginScene:setUserName(username)
    self._usernameBoxSize:setText(username)
end
function LoginScene:loginFunc(name, password)
    if IS_TOKEN_LOGIN then
        GAMETOKEN = name
        self:goHome( _createTime )
        return
    end
	zc.sendHttp({
        apiName = "User_login",
        params = {id="",passportId=name},
        successCallback = function (data) 
            GAMETOKEN = data.key
            local _createTime = tonumber( data.createTime ) or 0
    		self:goHome( _createTime )
            --存储用户信息
            self:userDB({name = name,login = 1})
        end,
	})
end
function LoginScene:userDB(newUserTable)
    local userTableData = zc.LoadTable("userTable")
    local addNew = true
    for i=1,#userTableData do 
        if userTableData[i]["name"] == newUserTable["name"] then 
            addNew = false
            userTableData[i]["login"] = 1
        else
            userTableData[i]["login"] = 0
        end
    end
    if addNew then
        userTableData[#userTableData+1] = newUserTable
    end
    table.sort(userTableData,function(a,b)
        return a.login > b.login 
    end)
    zc.SaveTable("userTable", userTableData)
end

function LoginScene:regFunc(name)
    --name=test001&seigniorId=1&passportId=295012462&passportName=ios
    if name == "" or name == nil then 
        zc.tipInfo("通行证id为空")
        return 
    end
    local n = tonumber(name)
    if n then
        -- n就是得到数字
        self:userDB({name = self._accountBox:getText(),login = 1})
        -- zc.createSourceScene("CreateRole", "src/layer/createRole/CreateRoleLayer.lua", {portId=name})
        zc.doFirstGuide(self, name)
    else
        -- 转数字失败,不是数字, 这时n == nil
        zc.tipInfo("通行证id为数字，请输入数字")
        return
    end
end
function LoginScene:goHome( _createTime )
    zc.sendHttp({
        apiName = "User_main",
        successCallback =  function (data)
            PlayerData:setAllInfo(data)
            PlayerData:setCreateTime(_createTime)
            GAMEUID = PlayerData:getBaseInfo()["userId"]
            --http 平台验证后进入游戏（与socket是否连接成功没有关系）
            zc.createScene(LAYER_NAME_TAG.LAYER_MAINCITY ,{data = PlayerData:getHouseInfo(), newScene = true})
            --进行socket连接--
            initConnectNet(SOCKET_LOGIN_SERVER, CHAT_SERVER_PORT)
        end,
    })
end

return LoginScene