--[[
FileName 		MultiCopyTeamLayer.lua
ProjectName		HJ2

Created by wangming on 16-03-21

说明：后端socket消息不给提供专属方便查询的字段，而统一发送了全部队列队伍信息，前端做整体的for循环检查比对变化逻辑太为复杂，
所以处理简洁点直接做比对单例子对象做刷新
--
]]

local MultiCopyTeamLayer = class("MultiCopyTeamLayer", function ()
    return zc.createNormalDialog({showBack = false})
end)

function MultiCopyTeamLayer:create( params )
	return MultiCopyTeamLayer.new( params )
end

function MultiCopyTeamLayer:onEnter()
    if self._fight then
        -- 战斗回来
        self._fight = false
        if self._dirty then
            self:goBack( self._dirtyInfo )
        end
        local scene = cc.Director:getInstance():getRunningScene()
        if not scene then
            return
        end
        local tipLab = scene:getChildByTag( 31 )
        if not tipLab then
            return
        end
        cc.Director:getInstance():getActionManager():resumeTarget( tipLab )
        if tolua.isnull(self._autoBtn) then
            return
        end
        self._autoBtn:setSelected( false )
        self._autoBtn:setScale( 1 )
    end

    self:addGuide()
end

function MultiCopyTeamLayer:initEvent()
    zc.addEventListener({
        name = CUSTOM_EVENT.EVENT_MULTICOPYTALK,
        callback = function (data) 
            self:dealWithTalkShow(data.data)
        end
    })
end

function MultiCopyTeamLayer:doEvent( opType, data )
    if opType == "MultiCopy_dismiss" then
        -- dump(data, "MultiCopy_dismiss")
        if self._tollgateId ~= tonumber( data.data.tollgateId ) then
            return
        end
        local _data = data.data.teams
        local isMy = self:getMyTeamInfo(_data)
        if isMy == false then
            self:goBack(zc.lang("tips_dissolve_team_success"))
        end
    elseif opType == "MultiCopy_join" then
        local _data = data.data.teams
        local isMy = self:getMyTeamInfo(_data)
        if isMy == false then
            return
        end 
        self:freshTeamList()
    elseif opType == "MultiCopy_kickout" then
        -- dump(data,"MultiCopy_kickout")
        local _data = data.data.teams
        local isMy = self:getMyTeamInfo(_data)
        if isMy == false then
            return
        end 
        local _haveSelf = false
        for i=1,3 do
            local _id = tonumber(self._teamsInfo["user"..i].userId) or -1
            if _id == self._myUserId then
                _haveSelf = true
            end
        end
        if not _haveSelf then
            self:goBack(zc.lang("tips_kicked_out_team"))
            return
        end
        self:freshTeamList()
    elseif opType == "MultiCopy_quit" then
        -- dump(data, "MultiCopy_quit" .. data.op)
        local _data = data.data.teams
        local isMy = self:getMyTeamInfo(_data)
        if isMy == false then
            return
        end 
        local _haveSelf = false
        for i=1,3 do
            local _id = tonumber(self._teamsInfo["user"..i].userId) or -1
            if _id == self._myUserId then
                _haveSelf = true
            end
        end
        if not _haveSelf then
            self:goBack()
            return
        end
        self:freshTeamList()
    elseif opType == "MultiCopy_adjust" then
        local _data = data.data.teams
        local isMy = self:getMyTeamInfo(_data)
        if isMy == false then
            return
        end 
        self:freshTeamList()
    elseif opType == "MultiCopy_challenge" then
        self._fight = true
        -- dump(result,"MultiCopy_challenge")
        local _allItems = data.data.info.allItems or {}
        local _items = {}
        if #_allItems > 0 then
            for i,v in ipairs(_allItems) do
                if tonumber(v.userId) == self._myUserId then
                    _items = PlayerData:getShowIconDatas( v )
                    PlayerData:updataData({clientResources = v.clientResources, clientItems = v.clientItems, noAnimation = true})
                    break
                end
            end
        end
        self._challengeNum = self._challengeNum + 1
        self:refreshLeftTimes()

        for i=3, 1, -1 do
            local _id = tonumber(self._teamsInfo["user"..i].userId) or -1
            if _id ~= self._createrId then
                -- 不是队长
                self._teamsInfo["user"..i].prepare = 0
            end
        end
        self:freshTeamList()
        self._isRead = self._isCreater and 1 or 0
        if not self._isCreater then
            -- 队员
            self:refreshAutoBtnText()
        end

        local _data = data.data.info
        _data.items = _items
        local _info = {
            data = _data,
            battleType = BattleType.MultiCopy,
            isFirstFight = true
        }
        local battle = zc.createBattle({data = _info})
        battle:play()
    end
end


function MultiCopyTeamLayer:onCleanup()
    self:unregisterEvent()
end

function MultiCopyTeamLayer:unregisterEvent()
    if self._haveRemoved then
        return
    end
    self._haveRemoved = true
    zc.removeEventListener(CUSTOM_EVENT.EVENT_MULTICOPYTALK)
end

function MultiCopyTeamLayer:ctor( params )
    params = params or {}
    -- dump(params,"params")
    local _params = params[1]
    -- 挑战次数
    self._challengeNum = tonumber( params[2] ) or 0
    -- 挑战次数上限
    self._challengeNumLimit = tonumber( params[3] ) or 0
    -- 关卡id
    self._tollgateId = tonumber(_params.tollgateId) or 1
    -- 队伍id
    self._teamId = tonumber(_params.teamId) or 0
    -- 队长id
    self._createrId = tonumber(_params.creatorId) or 0
    -- 玩家id
    self._myUserId = tonumber(PlayerData:getBaseInfoByTag("userId")) or 0
    -- 是否是队长
    self._isCreater = self._myUserId == self._createrId
    -- 队伍信息
    self._teamsInfo = _params
    -- 聊天cd
    self._talkCd = 0
    -- 邀请cd
    self._inviteCd = 0
    -- 位置
    self._posTb = {}
    self._onePart = 190
    self._widthPart = 300 
    -- dump(self._teamsInfo, "self._teamsInfo : ")
    -- 玩家是否准备
    self._isRead = 1
    for i=1,3 do
        local _userData = _params["user"..i] or {}
        local _userId = tonumber( _userData.userId ) or 0
        if _userId == self._myUserId then
            self._isRead = tonumber( _userData.prepare ) or 1
            break
        end
    end
    
    -- 聊天文字
    self._chatText = LANGUAGE_grabmine_get_chat_tips()


    local _bg = zc.createImageView({
        src = "res/Main/multyCopy/backgroud.jpg",
        pos = cc.p(zc.width/2 ,zc.height/2),
    })
    zc.setBgSize(_bg)
    self:addChild(_bg, -1)

    --添加云动画
    zc.getYunDongTai(nil,1,function(armUp, armDown)
        if not tolua.isnull(self) then
            self:addChild( armUp )
            self:addChild( armDown )
        end
    end)

	-- 背景
    local __bgImg = zc.createNormalPop({
        width = zc.width - 70,
        height = 511,
        showSide = false,
    })
    __bgImg:setAnchorPoint( 1, 0.5 )
    __bgImg:setPosition( zc.width, zc.height/2 )
    self:addChild( __bgImg )
    local _bgImg = cc.Sprite:create()
    _bgImg:setContentSize( zc.width, 511 )
    _bgImg:setPosition( zc.width/2, zc.height/2 )
    self:addChild( _bgImg )

    -- 卷轴
    local _data = gameData.getDataFromCacheWithPrimaryKey("tollgateTeam", self._tollgateId)
    local _armature = zc.createArmature({
        parentPath = "res/Main/copy/juanzhou/",
        fileName = "juanzhou_".. _data.image,
        -- loop = true,
        actName = "a_stand_".._data.icon,
        eventCallback = function() end,
    })
    _armature:setPosition(70, 0)
    _bgImg:addChild(_armature, 4)
    -- 推荐战力
    local powerNum = zc.createFntLabel({
        fnt = "res/Main/common/fnt/fight.fnt",
        text = _data.fighting,
        ap = cc.p(0.5, 0.5),
        pos = cc.p(_armature:getPositionX(), 180),
        addwidth = -5,
    })
    _bgImg:addChild(powerNum, 4)
    -- 掉落
    local reward = string.split(_data["commonDrop"],",")
    local _normal = zc.createSprite("res/Main/common/button/common_btnSquare1_u.png")
    local _select = zc.createSprite("res/Main/common/button/common_btnSquare1_d.png")
    _normal:setScale(1.2)
    _select:setScale(1.2)
    local item = zc.createPushButton({
        normalNode = _normal,
        selectedNode = _select,
        pos = cc.p(_armature:getPositionX(), 120), 
        endCallback = function()
            if #reward <= 0 then
                return
            end
            zc.createSourcePop("MultiCopyReward","src/layer/multicopy/MultiCopyRewardPop.lua", {reward = reward})
        end
    })
    _bgImg:addChild(item , 4)
    local label = zc.createLabel({
        size = 22,
        width = 50,
        outLine = true,
        color = display.COLOR_YELLOW,
        color2 = display.COLOR_BROWN,
        text = LANGUAGE_ARENA.reward_preview,
        pos = cc.p(item:getContentSize().width/2+2,item:getContentSize().height/2),
    })
    item:addChild(label , 4) 
    
    local _tableSize = cc.size(_bgImg:getContentSize().width - 180, _bgImg:getContentSize().height - 30)
    local _node = cc.Node:create()
    _node:setContentSize(_tableSize)
    _node:setPosition(150, 15)
    _bgImg:addChild(_node)
    self._diNode = _node
    
    self._userDis = {}
    self:freshTeamList()

    --返回按钮
    local _backButton = zc.createTopBackButton({
        pos = cc.p(zc.width - 60, zc.height - 40), 
        listener = function () 
            self:leaveTeam()
        end,
    })
    self:addChild(_backButton, 6)

    local _autoBtn = zc.createCommomBtn({
        tag = 1,
        text = "",
        listener = function () 
            if self._isCreater then
                self:startFight()
            else
                self:doReady()
            end
        end,
        pos = cc.p(zc.width - 150, 40),
    })
    self:addChild(_autoBtn, 4)
    self._autoBtn = _autoBtn
    self:refreshAutoBtnText()

    _text = self._isCreater and zc.lang("dismissTeam") or zc.lang("outTeam")
    local _createBtn = zc.createCommomBtn({
        tag = 2,
        listener = function () 
            self:leaveTeam()
        end,
        pos = cc.p(zc.width - 350, 40),
        text = _text,
    })
    self:addChild(_createBtn, 4)

    -- 剩余次数
    local _sp = zc.createSprite("res/Main/copy/leftTimesBg.png")
    _sp:setPosition(20, 20)
    _sp:setAnchorPoint(0, 0)
    self:addChild(_sp, 4)

    local _label = zc.createLabel({
        text = LANGUAGE_ARENA.times_left,
        fontSize = 22,
        stroke = true,
        pixel = 3,
        ap = cc.p(0, 0.5),
        pos = cc.p(35, _sp:getBoundingBox().height * 0.5 - 1),
    })
    _sp:addChild(_label)

    local _label2 = zc.createLabel({
        fontSize = 22,
        color = display.COLOR_GREEN,
        outLine = true,
        ap = cc.p(0, 0.5),
        pos = cc.p(_label:getPositionX() + _label:getContentSize().width + 10, _label:getPositionY()),
    })
    _sp:addChild(_label2)
    self._leftTimes = _label2
    self:refreshLeftTimes()

    self:initEvent()
end

--获取自己队伍的信息
function MultiCopyTeamLayer:getMyTeamInfo( data )
    for k,v in pairs(data) do
        local _teamId = tonumber(v.teamId) or 0
        if _teamId == self._teamId then
            self._teamsInfo = v
            return true
        end
    end
    return false
end

--创建一个玩家信息体
function MultiCopyTeamLayer:getOnePlayer( index )
    local _userData = self._teamsInfo["user"..index]
    local _di = nil
    -- dump(_userData, "wm----MultiCopyTeamLayer_userData : ")
    if _userData and next(_userData) ~= nil then
        local _userId = tonumber(_userData.userId) or 0
        if _userId == self._myUserId then
            -- 自己
            _di = ccui.ImageView:create("res/Main/multyCopy/selfDi.png")
            for j=1,4 do
                local _bar = zc.createPushButton({
                    normalFile = "res/Main/multyCopy/wordsDi.png",
                    touchScale = 0.95,
                    endCallback = function() 
                        self:sendWorldsToTeam(self._chatText[j])
                    end,
                    pos = cc.p(_di:getContentSize().width*0.5 - 45 + j*13, 75 + (j - 1)*55) 
                })
                _di:addChild(_bar, 1)
                local _ttf = zc.createLabel({
                    text = self._chatText[j],
                    fontSize = 22,
                    color2 = cc.c3b( 51, 0, 0 ),
                    pos = cc.p(_bar:getContentSize().width*0.5, _bar:getContentSize().height*0.5),
                    outLine = true,
                    pixel = 1,
                })
                _bar:addChild(_ttf)
            end
        else
            -- 队友
            _di = ccui.ImageView:create("res/Main/multyCopy/teamMateDi.png")
            local _posX = _di:getContentSize().width/2
            local _posY = _di:getContentSize().height/2 + 10
            -- 称号
            local _role_title = zc.createLabel({
                text = zc.getGodNameById( _userData.godName ),
                fontSize = 20,
                color = zc.COLOR_PURPLE,
                outLine = true,
                pixel = 1,
            })
            _role_title:setPosition( _posX, _posY )
            _di:addChild( _role_title )
            local _laceLeft = cc.Sprite:create( "res/Main/common/lace.png" )
            _laceLeft:setAnchorPoint( 1, 0.5 )
            _laceLeft:setPosition( _role_title:getPositionX() - _role_title:getContentSize().width/2, _role_title:getPositionY() )
            _di:addChild( _laceLeft )
            local _laceRight = cc.Sprite:create( "res/Main/common/lace.png" )
            _laceRight:setAnchorPoint( 0, 0.5 )
            _laceRight:setPosition( _role_title:getPositionX() + _role_title:getContentSize().width/2, _role_title:getPositionY() )
            _laceRight:setFlippedX( true )
            _di:addChild( _laceRight )
            --名称
            local _role_name_bg = ccui.Scale9Sprite:create( cc.rect( 12, 11, 14, 10 ), "res/Main/role/common/name_bg.png" )
            _role_name_bg:setContentSize( 220, 30 )
            _role_name_bg:setPosition( _posX - 5, _posY - 35 )
            _di:addChild( _role_name_bg )
            local _role_name = zc.createLabel({
                text = zc.getLevelTipShow()..tostring( _userData.level ).." ".._userData.userName,
                fontSize = 20,
                pos = cc.p(_posX - 5, _role_name_bg:getPositionY()),
                outLine = true,
                pixel = 1,
            })
            _di:addChild(_role_name)

            local _value = {
                {LANGUAGE_getAttributeTextById(1), 0},
                {LANGUAGE_getAttributeTextById(2), 0},
                {LANGUAGE_getAttributeTextById(3), 0},
            }
            if _userData.hero and _userData.hero.attribute then
                _value[1][2] = _userData.hero.attribute.force or 0
                _value[2][2] = _userData.hero.attribute.wisdom or 0
                _value[3][2] = _userData.hero.attribute.soma or 0
            end
            for i = 1, 3 do
                local _attrBg = ccui.Scale9Sprite:create( cc.rect( 12, 11, 14, 10 ), "res/Main/role/common/name_bg.png" )
                _attrBg:setContentSize( 220, 27 )
                _attrBg:setPosition( _posX - 5 - i*10, _posY - 35 - 35*i )
                _di:addChild( _attrBg )
                local _attrText = zc.createRichLabel({
                    text = _value[i][1],
                    fontSize = 18,
                    anchor = cc.p( 0, 0.5 ),
                    pos = cc.p( 10, _attrBg:getContentSize().height/2 ),
                })
                _attrText:setGradientColor( cc.c4b(244, 159, 8, 255), cc.c4b(255, 255, 124, 255), cc.p(0, -1) )
                _attrBg:addChild( _attrText )
                local _attrNum = zc.createLabel({
                    text = _value[i][2],
                    fontSize = 18,
                    ap = cc.p(  1, 0.5 ),
                    pos = cc.p( 210, _attrText:getPositionY() ),
                    outLine = true,
                })
                _attrBg:addChild( _attrNum )
            end

            if self._isCreater then
                -- 踢人
                local _btn = zc.createPushButton({
                    normalFile = "res/Main/multyCopy/kickOut.png",
                    touchScale = 0.9,
                    endCallback = function() 
                        self:kickOutOne(index, _userData.userName)
                    end,
                    anchor = cc.p(0.5, 0.5),
                    pos = cc.p(_di:getContentSize().width - 20, _di:getContentSize().height - 15),
                })
                _di:addChild(_btn, 1)
            end
        end

        _di.userId = _userId
        local num1 = self._createrId
        local num2 = _userId
        if num1 == num2 then
            -- 队长
            local _teamLeader = zc.createImageView({
                src = "res/Main/multyCopy/teamMaster.png",
                ap = cc.p(0, 1),
                pos = cc.p(80, _di:getContentSize().height +5) 
            })
            _di:addChild(_teamLeader, 1)
        end
        local _heroDatas = zc.getHeroInfo(_userData.seigniorId)
        local modelId = _heroDatas.modelId
        local _weaponId = tonumber(_userData.hero.weaponId) or 0
        local armature = zc.battle.createHeroArmatureWithResId(modelId, _weaponId)
        armature:setPosition(290, 310)
        armature:getAnimation():play("idle")
        _di:addChild(armature, 1)
        zc.soldierChangeWeapon(armature, _weaponId)

        --战斗力img
        local power = zc.createSprite("res/Main/role/common/fight_bg.png")
        power:setAnchorPoint(0, 0.5)
        power:setPosition(_di:getContentSize().width*0.5 - 15, _di:getContentSize().height*0.5 + 60)
        power:setScale( 0.7 )
        _di:addChild(power, 1)
        --战力
        local powerNum = zc.createFntLabel({
            fnt = "res/Main/common/fnt/fight_change.fnt",
            text = _userData.fighting,
            ap = cc.p(0, 0.5),
            pos = cc.p(90, power:getContentSize().height/2 ),
            addwidth = -14,
        })
        powerNum:setScale( 0.7 )
        power:addChild(powerNum)

        if self._isCreater then
            local _tagDi = zc.createImageView({
                src = "res/Main/multyCopy/changeStand.png",
                ap = cc.p(0.5, 0),
                pos = cc.p(_di:getContentSize().width*0.5 - 30, 5)
            })
            _di:addChild(_tagDi, 1)

            local _tagSp = zc.createImageView({
                src = "res/Main/multyCopy/hand_" .. index .. ".png",
                pos = cc.p(5, _tagDi:getContentSize().height*0.5)
            })
            _tagDi:addChild(_tagSp)
        end
    else
        _di = ccui.ImageView:create("res/Main/multyCopy/selfDi.png")
        _di.userId = 0

        local _emptyDi = zc.createSprite("res/Main/multyCopy/emptyBg.png")
        _emptyDi:setPosition(_di:getContentSize().width*0.5 + 40, _di:getContentSize().height*0.5 + 20)
        _di:addChild(_emptyDi)

        local _plus = zc.createImageView({
            src = "res/Main/multyCopy/plus.png",
            pos = cc.p(_di:getContentSize().width*0.5 + 8, _di:getContentSize().height*0.5 + 40),
            listener = function()
                self:showInvitePop()
            end,
            touchScale = 0.9,
        })
        _di:addChild(_plus)
        if index == 2 then
            self._guidePlusBtn = _plus
        end
    end
    return _di
end

--刷新队伍每个人的信息
function MultiCopyTeamLayer:freshTeamList()
    local bg = self._diNode
    local _size = bg:getContentSize()

    local _onePart = self._onePart
    local _widthPart = self._widthPart 
    local _posY = _size.height*0.5
    for i=3, 1, -1 do
        local _userData = self._teamsInfo["user"..i]
        local _di = self._userDis[i]
        local _userId = tonumber(_userData.userId) or 0
        if not _di or tonumber(_di.userId) ~= _userId then
            if _di then
                _di:removeFromParent()
            end
            _di = self:getOnePlayer(i)
            if _userId ~= 0 and self._isCreater then
                _di:setTouchEnabled(true)
                _di:addTouchEventListener(function ( sender, eventType )
                    if eventType == ccui.TouchEventType.began then
                        self._partPoint = sender:getTouchBeganPosition()
                        _di:setLocalZOrder(4)
                        return true
                    elseif eventType == ccui.TouchEventType.moved then 
                        local _movePoint = sender:getTouchMovePosition()
                        local _pos = cc.pSub(_movePoint, self._partPoint)
                        _pos = cc.pAdd(_pos, cc.p(_di:getPosition()))
                        self._partPoint = _movePoint
                        _di:setPosition(_pos.x, _di:getPositionY())
                        self:checkMove(_di)
                    else
                        -- self:resetUserDiPos()
                        if not self:checkHaveChange() then
                            self:resetUserDiPos()
                        end
                    end 
                end)
            end
            bg:addChild(_di, 3 - i)
        end

        local _posX = _onePart + (i-1)*_widthPart
        _di:setPosition(_posX, _posY)
        self._posTb[i] = i
        _di.index = i
        self._userDis[i] = _di
        if tonumber(_userData.prepare) == 1 then
            if not _di.readyImg then
                local _ready = zc.createSprite("res/Main/multyCopy/ready.png")
                _ready:setPosition(180, 360)
                _di:addChild(_ready)
                _di.readyImg = _ready
            end
        else
            if _di.readyImg then
                _di.readyImg:removeFromParent()
                _di.readyImg = nil
            end
        end
    end
end

function MultiCopyTeamLayer:resetUserDiPos()
    local _onePart = self._onePart
    local _widthPart = self._widthPart
    for j=1, 3 do
        self._posTb[j] = self._userDis[j].index
        self._userDis[j]:stopAllActions()
        self._userDis[j]:setLocalZOrder(3 - self._userDis[j].index)
        self._userDis[j]:setPosition(_onePart + (self._userDis[j].index-1)*_widthPart, self._userDis[j]:getPositionY())
    end
end

function MultiCopyTeamLayer:checkMove( _di )
    local _onePart = self._onePart
    local _widthPart = self._widthPart
    local _idx1 = _di.index == 1 and 2 or 1
    local _idx2 = _di.index == 3 and 2 or 3
    local _posX = _di:getPositionX()

    local function _go( target, x )
        target.isMoveing = true
        target:runAction(cc.Sequence:create(
            cc.MoveBy:create(0.1, cc.p(x, 0)),
            cc.CallFunc:create(function()
                target.isMoveing = false
            end)
        ))
    end

    local _part = (_posX - _onePart)/_widthPart
    if _part > 1.5 then
        self._posTb[3] = _di.index
        self._posTb[2] = _idx2
        self._posTb[1] = _idx1
    elseif _part > 0.5 then
        self._posTb[3] = _idx2
        self._posTb[2] = _di.index
        self._posTb[1] = _idx1
    else
        self._posTb[3] = _idx2
        self._posTb[2] = _idx1
        self._posTb[1] = _di.index
    end

    for i=1,3 do
        local _nIdx = self._posTb[i]
        local _tar = self._userDis[_nIdx]
        if _nIdx ~= _di.index and not _tar.isMoveing then
            local _tarX = _tar:getPositionX()
            local _pNum = _onePart + (i-1)*_widthPart - _tarX
            if _pNum ~= 0 then
                _go(_tar, _pNum)
            end
        end
    end
end

--监测调整的状态
function MultiCopyTeamLayer:checkHaveChange( index )
    for i=1,3 do
        if self._posTb[i] ~= i then
            self:adjustTeam()
            return true
        end
    end
    return false
end

--发送邀请
function MultiCopyTeamLayer:showInvitePop()
    if not self._isCreater then
        zc.tipInfo(zc.lang("tips_leader_can_use_func"))
        return 
    end
    if self._inviteCd ~= 0 then
        zc.tipInfo(zc.lang("tips_invite_cding") .. self._inviteCd)
        return
    end
    -- teamId = self._teamId
    -- tollgateId = self._tollgateId
    local toData = gameData.getDataFromCacheWithPrimaryKey("tollgateTeam", self._tollgateId)
    local _name = toData.name or ""
    local _info = string.format(zc.lang("tips_wait_to_jion"), tostring(_name))

    local _text = {
        teamId = self._teamId,
        tollgateId = self._tollgateId,
        info = _info,
    }
    local function _sendSuccess()
        zc.tipInfo(zc.lang("tips_send_success"))
        self._inviteCd = 30
        self._inviteCdAction = schedule(self, function()
            self._inviteCd = self._inviteCd - 1
            if self._inviteCd == 0 then
                self:stopAction(self._inviteCdAction)
            end
        end, 1)
    end

    local inputBoxSize = cc.size(400, 100) 
    local inputBox = ccui.EditBox:create(inputBoxSize, "res/Main/common/img_empty.png") 
    inputBox:setContentSize(inputBoxSize)
    inputBox:setFontSize(22)
    inputBox:setFontName(zc.getFont(1))
    inputBox:setAnchorPoint(0, 0.5)
    inputBox:setFontColor(cc.c3b(108,85,64))
    inputBox:setPlaceHolder(_info)
    inputBox:setPlaceholderFontColor(cc.c3b(108,85,64))
    inputBox:setPlaceholderFont(zc.getFont(1), 22)
    inputBox:setMaxLength(40) 
    inputBox:setInputMode(cc.EDITBOX_INPUT_MODE_ANY)
    inputBox:setInputFlag(cc.EDITBOX_INPUT_FLAG_SENSITIVE)
    inputBox:setReturnType(cc.KEYBOARD_RETURNTYPE_DONE )
    inputBox:setText(_info)
    inputBox:setPosition(cc.p(zc.width/2-200,zc.height/2))

    local function _leftCallback()
        local _str = inputBox:getText()
        if _str ~= "" and string.len(_str) > 0 then
            _text.info = _str
        end
        local _currentGID = getGuideManager():getCurrentGID()
        if _currentGID ~= GuideKey.KEY_11909 then
            getNetManager():getRequestCenter():sendWords(1, _text)
        end
        _sendSuccess()
    end

    local _isHide = true
    local _currentGID = getGuideManager():getCurrentGID()
    if _currentGID == GuideKey.KEY_11907 then
        _isHide = false
    end
    local _comfirmPop = nil
    _comfirmPop = ZCConfirmDialog:createWithParams({
        title = zc.lang("SEND_INVITE"),
        -- contentNode = inputBox,
        showDi = true,
        doshow = false,
        isHide = _isHide,
        leftText = zc.lang("SENDTOWORLD"),
        rightText = zc.lang("SENDTOCOUNTRY"),
        leftCallback = function()
            local _currentGID = getGuideManager():getCurrentGID()
            if _currentGID == GuideKey.KEY_11908 then
                zc.sendHttp({
                    apiName = "MultiCopy_getRobot",
                    params = {teamId = self._teamId},
                    successCallback = function (result)
                        zc.setGuideFuncNextStep(_currentGID)
                        _leftCallback()
                        _comfirmPop:hide()
                        local _data = result.data.teams
                        local isMy = self:getMyTeamInfo(_data)
                        if isMy == false then
                            return
                        end 
                        self:freshTeamList()
                        local _currentGID = getGuideManager():getCurrentGID()
                        if _currentGID == GuideKey.KEY_11909 then
                            getGuideManager():removeMaskLayer()
                            getGuideManager():setObjTarget(self._autoBtn)
                            getGuideManager():startGuide()
                        end
                    end,
                })
            else
                _leftCallback()
            end
        end,
        rightCallback = function()
            local _str = inputBox:getText()
            if _str ~= "" and string.len(_str) > 0 then
                _text.info = _str
            end
            local _countryId = tonumber(PlayerData:getBaseInfoByTag("countryId")) or 0
            if _countryId == 0 and PlayerData:getBaseInfoByTag("countryName") == ""  then 
                zc.tipInfo(zc.lang("tips_unjoin_country"))
                return
            end
            local _data = {countryId = _countryId}
            getNetManager():getRequestCenter():sendWords(2, _text, _data)
            _sendSuccess()
        end
    }) 
    _comfirmPop:addChild(inputBox)
    self:addChild(_comfirmPop, 10)

    local _currentGID = getGuideManager():getCurrentGID()
    if _currentGID == GuideKey.KEY_11907 then
        zc.setGuideFuncNextStep(_currentGID)
        getGuideManager():setObjTarget(_comfirmPop:getLeftButton())
        getGuideManager():startGuide()
    end
end

--返回
function MultiCopyTeamLayer:goBack( showInfo )
    -- 正在战斗，不处理
    self._dirty = nil
    self._dirtyInfo = nil
    if self._fight then
        self._dirty = true
        self._dirtyInfo = showInfo
        return
    end
    if self._haveGo then
        return
    end
    self._haveGo = true
    if showInfo then
        zc.tipInfo(tostring(showInfo))
    end
    self:unregisterEvent()
    local _id = self._tollgateId
    local function _callBack()
        local _pop = zc.getPop("MultiCopyReward")
        if _pop then
            _pop:hide()
        end
        LayerManager.removeLayout()
    end
    if self._guideOver then
        LayerManager.addShieldLayout()
        performWithDelay(self, _callBack, 0.01)
        return
    end
    zc.createMultiCopyScene(_id, _callBack)
end

--准备
function MultiCopyTeamLayer:doReady()
    local _readyStatus = self._isRead == 1 and 2 or 1
    zc.sendHttp({
        apiName = "MultiCopy_readyStatus",
        params = {tollgateId = self._tollgateId, type = _readyStatus},
        successCallback = function (result)
            self._isRead = _readyStatus
            self:refreshAutoBtnText()
        end,
    }) 
end

--调整
function MultiCopyTeamLayer:adjustTeam( idx1, idx2)
    local _info = ""
    local _target, idx
    for i=1,3 do
        idx = self._posTb[i]
        _target = self._userDis[idx]
        if _target then
            _info = _info .. _target.userId
        else
            _info = _info .. "0"
        end
        if i ~= 3 then
            _info = _info .. "|"
        end
    end
    zc.sendHttp({
        apiName = "MultiCopy_adjust",
        params = {teamId = self._teamId, info = _info},
    }) 
end

--开始战斗
function MultiCopyTeamLayer:startFight()
    local _heroCount = 0
    local haveUnReady = false
    for i=1, 3 do
        local _userData = self._teamsInfo["user"..i]
        if next(_userData) ~= nil then
            _heroCount = _heroCount + 1
            if tonumber(_userData.prepare) ~= 1 then
                haveUnReady = true
            end
        end
    end
    if _heroCount < 2 then
        zc.tipInfo(zc.lang("tips_fight_less_2_player"))
        return
    end
    if haveUnReady then
        zc.tipInfo(zc.lang("tips_fight_player_not_ready"))
        return
    end
    zc.sendHttp({
        apiName = "MultiCopy_challenge",
        params = {teamId = self._teamId},
        successCallback = function()
            local _currentGID = getGuideManager():getCurrentGID()
            if _currentGID == GuideKey.KEY_11909 then
                getGuideManager():setFuncGuideOver()
                self._guideOver = true
            end
        end
    }) 
end

--离开队伍
function MultiCopyTeamLayer:leaveTeam()
    local _config = "MultiCopy_quit"
    local _title = zc.lang("prompt")
    local _text = zc.lang("tips_is_to_quit_team")
    if self._isCreater then
        _config = "MultiCopy_dismiss"
        _title = zc.lang("prompt")
        _text = zc.lang("tips_is_to_dissolve_team")
    end
    local _comfirmPop = ZCConfirmDialog:createWithParams({
        title = _title,
        msg = _text,
        showDi = true,
        rightCallback = function()
            zc.sendHttp({
                -- 接口
                apiName = _config,
                -- 参数
                params = {tollgateId = self._tollgateId},
                -- 成功回调
                successCallback = function(result) 
                    if tolua.isnull(self) then
                        return
                    end
                    self:goBack()
                end,
                -- 错误回调
                errorCallback = function( errorCode )
                    if tolua.isnull(self) then
                        return
                    end
                    local _errorCode = ( tonumber( errorCode ) or 0 )
                    if _errorCode == 296 then
                        -- 队伍已解散
                        self:goBack()
                    elseif _errorCode == 98 then
                        -- 队伍不存在
                        self:goBack()
                    end
                end,
            }) 
        end
    }) 
    self:addChild(_comfirmPop, 10) 
end

--踢出队伍
function MultiCopyTeamLayer:kickOutOne( index, name )
    local _comfirmPop = ZCConfirmDialog:createWithParams({
        title = zc.lang("prompt"),
        showDi = true,
        msg = string.format(zc.lang("tips_is_kick_out_team"), tostring(name)),
        rightCallback = function()
            zc.sendHttp({
                apiName = "MultiCopy_kickout",
                params = {teamId = self._teamId, index = index},
            })
        end
    }) 
    self:addChild(_comfirmPop, 10)
end

--获取一个其他玩家的信息体封装
function MultiCopyTeamLayer:getOneHeroInfoImg( data, tag )
    local src_bg = "res/Main/role/role_attr_prograss_bg.png"
    local _progress = {
        red = "res/Main/role/role_attr_prograss_red.png",
        blue = "res/Main/role/role_attr_prograss_blue.png",
        green = "res/Main/role/role_attr_prograss_green.png",
    }
    local _tip = {
        red = "res/Main/role/role_wu_tip.png",
        blue = "res/Main/role/role_zhi_tip.png",
        green = "res/Main/role/role_ti_tip.png",
    }
    local _num1 = 0
    local _num2 = 1
    if data.hero then
        if tag == "red" then
            _num1 = data.hero.attribute.force
            _num2 = _num1 + data.hero.culture.forceLimit - data.hero.culture.force
        elseif tag == "blue" then
            _num1 = data.hero.attribute.wisdom
            _num2 = _num1 + data.hero.culture.wisdomLimit - data.hero.culture.force
        else
            _num1 = data.hero.attribute.soma
            _num2 = _num1 + data.hero.culture.somaLimit - data.hero.culture.soma
        end
    end

    local _di, _press = self:createProgress({
        src1 = src_bg,
        src2 = _progress[tag],
        percent = (_num1/_num2)*100,
    })
    _press:setPosition(0, _di:getContentSize().height*0.5 - 1)
    _di:addChild(_press)
    
    local _img = zc.createImageView({
        src = _tip[tag],
        ap = cc.p(0.5, 0.5),
        pos = cc.p(0, _di:getContentSize().height*0.5) 
    })
    _di:addChild(_img)
    local _text = zc.createLabel({
        text = _num1,
        fontSize = 26,
        ap = cc.p(0, 0.5),
        pos = cc.p(_di:getContentSize().width + 5, _di:getContentSize().height*0.5), 
        stroke = true,
    })
    _di:addChild(_text)
    return _di
end

--创建进度条
function MultiCopyTeamLayer:createProgress(params)
    local expImgBg = cc.Sprite:create(params.src1)
    expImgBg:setAnchorPoint(cc.p(0, 0.5))
    local expGressImg = cc.Sprite:create(params.src2)
    expGressImg:setAnchorPoint(cc.p(0, 0.5))
    local expPrgress = cc.ProgressTimer:create(expGressImg)
    expPrgress:setAnchorPoint(cc.p(0, 0.5))
    expPrgress:setType(cc.PROGRESS_TIMER_TYPE_BAR)
    expPrgress:setBarChangeRate(cc.p(1, 0))
    expPrgress:setMidpoint(cc.p(0, 0))
    local _percent = tonumber(params.percent) or 0
    _percent = _percent > 100 and 100 or _percent
    expPrgress:setPercentage(_percent)
    if params.pos then
        expImgBg:setPosition(params.pos)
        expPrgress:setPosition(params.pos)
    end
    if params.par then
        params.par:addChild(expImgBg)
        params.par:addChild(expPrgress)
    end
    return expImgBg, expPrgress
end

function MultiCopyTeamLayer:sendWorldsToTeam( text )
    if self._talkCd ~= 0 then
        zc.tipInfo("发送消息cd中，请稍候..." .. self._talkCd)
        return
    end
    self._talkCd = 5
    self._cdAction = schedule(self, function()
        self._talkCd = self._talkCd - 1
        if self._talkCd == 0 then
            self:stopAction(self._cdAction)
        end
    end, 1)
    for i=1, 3 do
        local _userData = self._teamsInfo["user"..i]
        local _userId = tonumber(_userData.userId) or 0
        if _userId ~= 0 and _userId ~= self._myUserId then
            local _data = {toUserId = _userId}
            getNetManager():getRequestCenter():sendWords(100, text, _data)
        end
    end
    --可以处理下自己显示下自己说的话
    self:dealWithTalkShow({userId = self._myUserId, text = text}, true)
end

function MultiCopyTeamLayer:dealWithTalkShow( data, isSelfShow )
    local _isSelfShow = isSelfShow == nil and false or isSelfShow
    if _isSelfShow then
    elseif data.userId == self._myUserId then
        return
    end
    for k,v in pairs(self._userDis) do
        local _userId = v.userId
        if _userId == data.userId then
            --可以处理显示了
            local _text = data.text
            local _chatDi = zc.createSprite("res/Main/multyCopy/talkBar.png")
            _chatDi:setAnchorPoint(1, 0.5)
            local _pos = v:convertToWorldSpace(cc.p(280, 370))
            _chatDi:setPosition(_pos)
            self:addChild(_chatDi, 2)
            _chatDi:setScale(0.1)
            _chatDi:runAction(cc.Sequence:create(
                cc.ScaleTo:create(0.1, 1),
                cc.DelayTime:create(2),
                cc.ScaleTo:create(0.1, 0.1),
                cc.RemoveSelf:create(true)
            ))
            local _label = zc.createLabel({
                text = _text,
                width = 150,
                fontSize = 20,
                color = cc.c3b( 51, 0, 0 ),
                ap = cc.p(0.5, 0.5),
                pos = cc.pMul(cc.pFromSize(_chatDi:getContentSize()), 0.5),
                -- stroke = true,
            })
            _chatDi:addChild(_label)
            break
        end
    end 
end

-- 刷新右侧的按钮文字
function MultiCopyTeamLayer:refreshAutoBtnText()
    if self._isCreater then
        -- 队长，只有开始挑战
        self._autoBtn:setText( zc.lang("start_fight") )
    elseif self._isRead == 1 then
        -- 队员，取消准备
        self._autoBtn:setText( zc.lang("unReady") )
    else
        -- 队员，准备
        self._autoBtn:setText( zc.lang("doReady") )
    end
end

-- 刷新剩余次数
function MultiCopyTeamLayer:refreshLeftTimes()
    local _n1 = self._challengeNum
    local _n2 = self._challengeNumLimit
    local _n3 = _n2-_n1
    _n3 = _n3 < 0 and 0 or _n3
    local _txt = _n3 .. "/" .. _n2
    self._leftTimes:setText( _txt )
end

function MultiCopyTeamLayer:addGuide()
    local _currentGID = getGuideManager():getCurrentGID()
    if _currentGID == GuideKey.KEY_11906 then
        zc.setGuideFuncNextStep(_currentGID)
        self:addGuide()
    elseif _currentGID == GuideKey.KEY_11907 then
        getGuideManager():setObjTarget(self._guidePlusBtn)
        getGuideManager():startGuide()
    end
end

return MultiCopyTeamLayer