--[[
    FileName: GeneralManualLayer.lua
    Author: andong
    Date: 2017-01-05
    Purpose: 武将图鉴界面
]]
local GeneralManualLayer = class( "GeneralManualLayer", function ()
    return zc.createNormalDialog()
end)
function GeneralManualLayer:create(parmas)
    return GeneralManualLayer.new(parmas)
end
function GeneralManualLayer:ctor(parmas)
    self:initData(parmas)
    self:initUI()
end

function GeneralManualLayer:initData(parmas)
    if parmas and parmas.callback then
        self._callback = parmas.callback
    end
    --
    self._playerLevel = PlayerData:getUserLevel()
    local _houseInfo = PlayerData:getHouseInfoByHouseId(9) or {}
    self._houseLevel = _houseInfo["level"] or 1

    self._scrollAnimation = {
        [1] = "qunxiongbingqitujian",
        [2] = "zhanshenmolu",
        [3] = "guanduzhizhan",
        [4] = "moudingqiankuan",
    }

    --需要跳转
    self._scrollIndex = 1
    self._gotoHero = false
    if parmas.gotoHeroId then
        local gotoHeroInfo = gameData.getDataFromCache("hero_handbook_info", {generalid=parmas.gotoHeroId} )
        self._scrollIndex = tonumber(gotoHeroInfo.handbook) or 1
        self._gotoHeroId = tonumber(parmas.gotoHeroId)
        self._gotoHero = true
    end

    --
    self._flower_l_u = self:getChildByName("flower_l_u")
    self._flower_r_u = self:getChildByName("flower_r_u")
    self._flower_r_d = self:getChildByName("flower_r_d")

    --
    local hero_handbook_card = gameData.getDataFromCache("hero_handbook_card") or {}
    local _hero_handbook = {}
    for k,v in pairs(hero_handbook_card) do
        if tonumber(v.id) ~= 5 then
            local _isOpen = true
            local handBook = v
            if handBook then
                if handBook["level"] then --酒馆等级
                    if tonumber(self._houseLevel) < tonumber(handBook["level"]) then
                        _isOpen = false
                    end
                end
                if _isOpen then
                    if handBook["plevel"] then --酒馆等级
                        if tonumber(self._playerLevel) < tonumber(handBook["plevel"]) then
                            _isOpen = false
                        end
                    end
                end
            end

            local _index = #_hero_handbook+1
            _hero_handbook[_index] = v
            _hero_handbook[_index]._isOpen = _isOpen
        end
    end

    table.sort( _hero_handbook, function(a, b)
        return tonumber(a.id) < tonumber(b.id)
    end)

    --整理显示卷轴，未开启的卷轴只显示一个
    self._hero_handbook = {}
    self._bookIdTo_hero_handbook = {}
    for i = 1, #_hero_handbook do
        local v = _hero_handbook[i]
        if v then
            self._hero_handbook[#self._hero_handbook+1] = v
            self._bookIdTo_hero_handbook[tostring(v.id)] = v
            if not v._isOpen then
                break
            end
        end
    end

end
function GeneralManualLayer:initUI()
    local manual_title = zc.createSprite("res/Main/pub/manual/manual_title.png")
    manual_title:setAnchorPoint(0, 1)
    manual_title:setPosition(0, zc.height)
    self:addChild(manual_title, 10)
    self._title = manual_title

    self:initScollPage()
end
function GeneralManualLayer:initScollPage()

    local scrollPage = zc.createSprite()
    scrollPage:setContentSize( zc.width, zc.height )
    scrollPage:setPosition(zc.width/2, zc.height/2)
    self:addChild(scrollPage, 1)
    self._scrollPage = scrollPage

  
    local setNum = table.nums(self._hero_handbook) + 4
    self._seatNum = setNum

    local tableSize = cc.size(1136, zc.height-60)
    
    local myTable = zc.ZCTableView:create(tableSize)
    myTable:setPosition((zc.width-tableSize.width)/2, (zc.height-tableSize.height)/2)
    myTable:setInertia(true) --设置惯性
    myTable:setBounceable(false)
    myTable:setAutoAlign(true)
    myTable:setDirection(zctech.SCROLLVIEW_DIRECTION_HORIZONTAL) --设置横向纵向
    myTable:setDelegate()
    myTable:setVerticalFillOrder(zctech.TABLEVIEW_FILL_TOPDOWN)
    
    scrollPage:addChild(myTable)
    self._myTable = myTable

    local _min = 0.55

    local cellSize = cc.size(tableSize.width*0.2, tableSize.height-4)

    local baseOffset = cellSize.width/2
    if zc.width < 1000 then
        baseOffset = 22.6
    end
    local function scorllviewScriptScroll(table)
        local currentPage = table:getCurrentPage() + 2
        local _offSet = table:getContentOffset().x --table偏移量
        for i = 2, setNum - 3 do
            local _cell = table:cellAtIndex(i)
            if _cell and _cell._sprite then
                local _posX = _cell:getPositionX() + _offSet + baseOffset
                local _scale = 1
                if _posX <= 0 or _posX > zc.width then
                    _scale = _min
                else
                    _scale = 1 - math.abs(zc.width*0.5 - _posX)/(zc.width*0.5)*_min
                end
                _cell._sprite:setScale(_scale)
            end
        end
    end

    local function tablePageAfterSliding( table, cell )
        self._scrolling = false
        local currentPage = table:getCurrentPage() + 2
        if self._curSeatId == currentPage then
            return
        end
        self._curSeatId = currentPage

        for i = 2, self._seatNum - 3 do
            local _tmpCell = table:cellAtIndex(i)

            if i ~= currentPage then
                if _tmpCell and _tmpCell._sprite then
                    local _arm = _tmpCell._sprite:getChildByName("armature")
                    _arm:playAction("stop", true)
                end
            else
                local _arm = _tmpCell._sprite:getChildByName("armature")
                if _arm then
                    _arm:playAction("start", false)
                    _arm:registerSpineEventHandler( function ( event )
                        if event.animation == "start" then
                            _arm:playAction("idle", true)
                        end
                    end, sp.EventType.ANIMATION_COMPLETE)
                end
            end
        end
    end

    local function cellSizeForTable(table,idx)
        return cellSize.height, cellSize.width
    end
    local function numberOfCellsInTableView(table)
        return setNum
    end
    local function tableTouchEnded(table, cell)
        scorllviewScriptScroll(table)
    end
    local function tableCellAtIndex(table,idx)
        local nowIdx = idx + 1
        local cell = table:dequeueCell()
        if cell == nil then
            cell = zc.ZCTableViewCell:new()
            cell:setContentSize(cc.size(cellSize.width, cellSize.height))
        else
            cell:removeAllChildren()
        end

        if idx < 2 or idx > setNum - 3 then
            return cell
        end

        nowIdx = idx - 1
    
        self:updateCell(cell, nowIdx)
        return cell
    end

    myTable:registerScriptHandler(numberOfCellsInTableView,cc.NUMBER_OF_CELLS_IN_TABLEVIEW)
    myTable:registerScriptHandler(cellSizeForTable,cc.TABLECELL_SIZE_FOR_INDEX)
    myTable:registerScriptHandler(scorllviewScriptScroll,zctech.SCROLLVIEW_SCRIPT_SCROLL)
    myTable:registerScriptHandler(tableCellAtIndex,cc.TABLECELL_SIZE_AT_INDEX)
    myTable:registerScriptHandler(tableTouchEnded,zctech.TABLEVIEW_TOUCH_ENDED)
    myTable:registerScriptHandler(tablePageAfterSliding,zctech.TABLECELL_AFTER_SLIDING)
    
    myTable:reloadData()

    --跳转的指定位置
    self._myTable:scrollToCell(self._scrollIndex - 1, true)
end
function GeneralManualLayer:updateCell(cell, index)
    local cellSize = cell:getContentSize()
    local cellSprite = zc.getScaleNode("res/Main/common/img_empty.png", cellSize)
    cellSprite:setAnchorPoint(cc.p(0.5, 0.5))
    cellSprite:setPosition(cc.p(cellSize.width/2, cellSize.height/2))
    cell:addChild(cellSprite)
    cellSprite:setName("cellSprite"..index)
    cellSprite:setCascadeOpacityEnabled(true)
    cell._sprite = cellSprite

    local fileName = self._scrollAnimation[index]
    if not fileName then
        fileName = "qunxiongbingqitujian"
    end
    local _boxSpine = zc.createSpine({
        fileName = fileName,
        actName = "stop",
        loop = true,
        needInit = false,
    })
    _boxSpine:setAnchorPoint( 0.5, 0.5 )
    _boxSpine:setPosition(cellSize.width/2,cellSize.height/2)
    _boxSpine:setName("armature")
    cellSprite:addChild(_boxSpine)

    local handBook = self._hero_handbook[index] or {}
    local _isOpen = handBook._isOpen

    local scaleSize = cc.size(cellSize.width-10, cellSize.height/3*2)
    local myScale9 = zc.getScaleNode("res/Main/common/img_empty.png", scaleSize)
    -- local myScale9 = zc.getScaleNode("res/Main/setting/scale9_1.png", scaleSize)
    local touchBtn = zc.createPushButton({
        normalNode = myScale9,
        needSwallow = false,
        needEnableWhenMoving = true,
        endCallback = function()
            if self._curSeatId ~= index + 1 then
                self._myTable:stopAllActions()
                self._myTable:scrollToCell(index - 1, true)
                return
            end

            if not _isOpen then
                zc.tipInfo(LANGUAGE_TOLLGATEHERO.not_open .. "！")
                return
            end
            self:openManualList(index)
        end,
        anchor = cc.p(0.5, 0.5),
        pos = cc.p(cellSize.width/2, cellSize.height/2)
    })
    cellSprite:addChild(touchBtn)
    if index == 1 then
        self._guideBtn = touchBtn
    end

    if not _isOpen then
        local tipLab1 = zc.createLabel({
            text = zc.lang("mainhero_achieve_space_level"),
            fontSize = 18,
            outLine = true,
            pos = cc.p(cellSize.width/2, 80),
        })
        cellSprite:addChild(tipLab1)
        local levelLab1 = zc.createLabel({
            text = handBook["plevel"],
            fontSize = 20,
            outLine = true,
            pos = cc.p(123, tipLab1:getContentSize().height/2),
            color = display.COLOR_RED,
        })
        tipLab1:addChild(levelLab1)
        local tipLab1 = zc.createLabel({
            text = zc.lang("and_house_level_achieve_space_level_open") .. "！",
            fontSize = 18,
            outLine = true,
            pos = cc.p(cellSize.width/2, 50),
        })
        cellSprite:addChild(tipLab1)
        local levelLab1 = zc.createLabel({
            text = handBook["level"],
            fontSize = 20,
            outLine = true,
            pos = cc.p(140, tipLab1:getContentSize().height/2),
            color = display.COLOR_RED,
        })
        tipLab1:addChild(levelLab1)
    end

end
function GeneralManualLayer:openManualList(index)
    local handBook = self._hero_handbook[index] or {}
    local _isOpen = handBook._isOpen
    if not _isOpen then
        zc.tipInfo(LANGUAGE_TOLLGATEHERO.not_open .. "！")
        return
    end
    self._scrollPage:setVisible(false)
    self._title:setVisible(false)
    self._flower_l_u:setVisible(false)
    self._flower_r_u:setVisible(false)
    self._flower_r_d:setVisible(false)

    local function _layerCleanCallback()
        self._closeBtn:setTouchEndedCall(function()
            LayerManager.removeLayout()
        end)
        self._scrollPage:setVisible(true)
        self._title:setVisible(true)
        self._flower_l_u:setVisible(true)
        self._flower_r_u:setVisible(true)
        self._flower_r_d:setVisible(true)
    end

    local _currentGID = getGuideManager():getCurrentGID()
    if _currentGID == GuideKey.KEY_14405 then
        zc.setGuideFuncNextStep(_currentGID, false)
    end
    local _layer = requires("src/layer/pub/GeneralManualListLayer.lua"):create( {
        data=index,
        callback=_layerCleanCallback,
        closeBtn = self._closeBtn,
        gotoHeroId = self._gotoHeroId,
        parent = self,
    })
    _layer:setAnchorPoint(0.5, 0.5)
    _layer:setPosition(zc.width/2,zc.height/2)
    self:addChild(_layer, 2)


    self._listLayer = _layer
end
function GeneralManualLayer:onCleanup()
    if self._callback and type(self._callback) == "function" then
        self._callback()
    end
    helper.collectMemory()
end

function GeneralManualLayer:onEnter( ... )
    if self._gotoHero then
        self:openManualList(self._scrollIndex)
        
        self._gotoHero = false
        self._gotoHeroId = nil
    end
    if self._init then
        if self._listLayer then
            self._listLayer:onEnter()
        end
    end
    self._init = true
    self:addGuide()
end
function GeneralManualLayer:addGuide()
    local _currentGID = getGuideManager():getCurrentGID()
    if _currentGID == GuideKey.KEY_14403 then
        zc.setGuideFuncNextStep(_currentGID, false)
        getGuideManager():setHeroFuncCallBack(function()
            getGuideManager():setObjTarget(self._guideBtn, 0.8)
            getGuideManager():startGuide()
        end)
        getGuideManager():startGuide()
    end
end
return GeneralManualLayer