--[[
    FileName: ResearchBingFaLayer.lua
    Author: andong
    Date: 2016-04-07
    Purpose: 研究院界面
]]
local ResearchBingFaLayer = class( "ResearchBingFaLayer", function ()
    return ZCDialog:create()
end)
function ResearchBingFaLayer:create(parmas)
    return ResearchBingFaLayer.new(parmas)
end
function ResearchBingFaLayer:ctor(parmas)
    self:initData(parmas)
    self:initUI()
end

function ResearchBingFaLayer:initData(parmas)
    if parmas and parmas.callback then
        self._callback = parmas.callback
    end

    local toll = PlayerData:getEnemyInfoByEnemyId(1001001)
    local ch = gameData.getDataFromCacheWithPrimaryKey("chapter", toll.chapterId)
    
    self._activeMainId = tonumber(parmas.activeId)
    self._openInfo = {} --已经激活的兵书列表,bookId做为key
    for i=1, #parmas.books do
        self._openInfo[tonumber(parmas.books[i].bookId)] = parmas.books[i]
    end
    
    self._tag = 1 --开启兵书的最大id
    for k,v in pairs(self._openInfo) do
        if tonumber(v.id) == self._activeMainId then
            self._tag = tonumber(v.bookId)
            break
        end
        if tonumber(v.bookId) > self._tag then
            self._tag = tonumber(v.bookId)
        end
    end
end
function ResearchBingFaLayer:initUI()
    local background = cc.Sprite:create("res/Main/research/bingfa/bingfa_bg.jpg")
    background:setPosition(cc.p(zc.width/2, zc.height/2))
    self:addChild(background)
    zc.setBgSize(background)
    
    local backButton = zc.createTopBackButton({
        pos = cc.p(zc.width - 60, zc.height - 40), 
        listener = function () 
            LayerManager.removeLayout()
        end,
    })
    self:addChild(backButton, 10)
    
    self:createBaseInfo()
    self:initPage()
end

function ResearchBingFaLayer:createBaseInfo()
    local title = zc.createSprite("res/Main/research/bingfa/title.png")
    title:setAnchorPoint(0, 1)
    title:setPosition(0, zc.height)
    self:addChild(title, 10)
    
    --添加云动画
    zc.getYunDongTai(nil,2,function(armUp, armDown)
        if not tolua.isnull(self) then
            self:addChild( armUp )
            self:addChild( armDown )
        end
    end)
end
function ResearchBingFaLayer:initPage()
    local _datas = gameData.getAllDataWithNoClone("book")
    local setNum = table.nums(_datas) + 4

    local tableSize = cc.size(zc.width, zc.height-80)
    local tableNode = self
    
    local myTableBg = zc.getScaleNode("res/Main/setting/scale9_1.png", tableSize)
    myTableBg:setAnchorPoint(cc.p(0.5, 0.5))
    myTableBg:setPosition(cc.p(zc.width/2, zc.height/2))
    local myTable = zc.ZCTableView:create(tableSize)
    myTable:setPosition((zc.width-tableSize.width)/2, (zc.height-tableSize.height)/2)
    
    myTable:setInertia(true) --设置惯性
    myTable:setBounceable(false)
    myTable:setAutoAlign(true)
    myTable:setDirection(zctech.SCROLLVIEW_DIRECTION_HORIZONTAL) --设置横向纵向
    myTable:setDelegate()
    myTable:setVerticalFillOrder(zctech.TABLEVIEW_FILL_TOPDOWN)
    
    self:addChild(myTable, 10)
    self._myTable = myTable
    local _min = 0.55
    local cellSize = cc.size(tableSize.width*0.2, tableSize.height-4)
    local baseOffset = cellSize.width/2
    if zc.width < 1000 then
        baseOffset = 22.6
    end
    local function scorllviewScriptScroll(table)
        local currentPage = table:getCurrentPage() + 2
        local _offSet = table:getContentOffset().x --table偏移量
        for i = 2, setNum - 3 do
            local _cell = table:cellAtIndex(i)
            if _cell and _cell._sprite then
                local _posX = _cell:getPositionX() + _offSet + baseOffset
                local _scale = 1
                if _posX <= 0 or _posX > zc.width then
                    _scale = _min
                else
                    _scale = 1 - math.abs(zc.width*0.5 - _posX)/(zc.width*0.5)*_min
                end
                _cell._sprite:setScale(_scale)
                if _cell.openTarget then
                    if i ~= currentPage then
                        _cell.openTarget:setVisible(false)
                    else
                        _cell.openTarget:setVisible(true)
                    end
                end
            end
        end
    end
    local function tablePageAfterSliding(table, cell)
        local currentPage = table:getCurrentPage() + 2

        if self._nowPage and self._nowPage == currentPage then
            return 
        end
        self._nowPage = currentPage
        
        for i = 2, setNum - 3 do
            local _cell = table:cellAtIndex(i)
            if _cell then
                if _cell:getChildByName("bookBg") then
                    local bookBg = _cell:getChildByName("bookBg")
                    bookBg:removeFromParent()
                end
                local bookBg
                if i == currentPage then
                    bookBg = zc.createSprite("res/Main/research/bingfa/shu_bg2.png")
                    bookBg:setPosition(_cell:getContentSize().width/2, _cell:getContentSize().height/2)
                    bookBg:setOpacity(0)
                    bookBg:runAction(cc.Sequence:create(cc.FadeIn:create(0.2)))
                elseif i < currentPage then
                    bookBg = zc.createSprite("res/Main/research/bingfa/shu_bg1.png")
                    bookBg:setPosition(_cell:getContentSize().width/2, _cell:getContentSize().height/2)
                    bookBg:setFlippedX(true)

                elseif i > currentPage then
                    bookBg = zc.createSprite("res/Main/research/bingfa/shu_bg1.png")
                    bookBg:setPosition(_cell:getContentSize().width/2, _cell:getContentSize().height/2)
                end
                _cell:addChild(bookBg, -1)
                bookBg:setName("bookBg")

                if _cell._armature then
                    if i == currentPage then
                        local arm = _cell._armature
                        local am = "open_"..(i-1)
                        arm:playAction(am, false)
                        arm:registerSpineEventHandler( function ( event )
                            if event.animation == am then
                            end
                        end, sp.EventType.ANIMATION_COMPLETE)
                    else
                        local arm = _cell._armature
                        local _currentMovementID = arm:getNowAniName()
                        -- ""刚播放 open 动画
                        if _currentMovementID == "" then
                            local am = "close_"..(i-1)
                            arm:playAction(am, false)
                            arm:registerSpineEventHandler( function ( event )
                                if event.animation == am then
                                    arm:playAction("idle_"..(i-1), true)
                                end
                            end, sp.EventType.ANIMATION_COMPLETE)
                        else
                            arm:playAction("idle_"..(i-1), true)
                        end
                        if cell._openTar then
                            cell._openTar:setVisible(false)
                        end
                    end
                end
                if i == currentPage then
                    if _cell._openTar then
                        _cell._openTar:setVisible(true)
                    end
                else
                    if _cell._openTar then
                        _cell._openTar:setVisible(false)
                    end
                end
            end
        end
    end

    local function cellSizeForTable(table,idx)
        return cellSize.height, cellSize.width
    end
    local function numberOfCellsInTableView(table)
        return setNum
    end
    local function tableTouchEnded(table, cell)
        scorllviewScriptScroll(table)
    end
    local function tableCellAtIndex(table,idx)
        local nowIdx = idx + 1
        local cell = table:dequeueCell()
        if cell == nil then
            cell = zc.ZCTableViewCell:new()
            cell:setContentSize(cc.size(cellSize.width, cellSize.height))
        else
            cell:removeAllChildren()
            cell._armature = nil
            cell._openTar = nil
        end

        if idx < 2 or idx > setNum - 3 then
            return cell
        end

        nowIdx = idx - 1
    
        self:updateCell(cell, nowIdx)
        return cell
    end

    myTable:registerScriptHandler(numberOfCellsInTableView,cc.NUMBER_OF_CELLS_IN_TABLEVIEW)
    myTable:registerScriptHandler(cellSizeForTable,cc.TABLECELL_SIZE_FOR_INDEX)
    myTable:registerScriptHandler(scorllviewScriptScroll,zctech.SCROLLVIEW_SCRIPT_SCROLL)
    myTable:registerScriptHandler(tableCellAtIndex,cc.TABLECELL_SIZE_AT_INDEX)
    myTable:registerScriptHandler(tableTouchEnded,zctech.TABLEVIEW_TOUCH_ENDED)
    myTable:registerScriptHandler(tablePageAfterSliding,zctech.TABLECELL_AFTER_SLIDING)
    
    myTable:reloadData()

    --跳转的指定位置
    self._myTable:scrollToCell(self._tag-1, true)
end

function ResearchBingFaLayer:updateCell(cell, indx)
    local curBook = gameData.getDataFromCacheWithPrimaryKey("book", indx)
    local cellSize = cell:getContentSize()
    local am = "idle_"..indx
    local armature = zc.getSpine("bingfajuanzhou")
    armature:playAction(am, true)
    local bookSize = cc.size(cellSize.width-10, cellSize.height-20)
    local bookSprite = zc.createSprite()
    bookSprite:setContentSize( bookSize )
    armature:setPosition(bookSize.width/2, bookSize.height/2)
    bookSprite:addChild(armature)
    cell._armature = armature

    local clickBtn = zc.createPushButton({
        normalNode = bookSprite,
        needSwallow = false,
        needEnableWhenMoving = true,
        noScaleAction = true,
        touchScale = 0.9,
        endCallback = function()
            --功能引导
            local currentPage = self._myTable:getCurrentPage() + 2
            if currentPage ~= indx + 1 then
                getGuideManager():setFuncGuideOver()
                self._myTable:stopAllActions()
                self._myTable:scrollToCell(indx - 1, true)
                return
            end

            if self._openInfo[tonumber(curBook.id)] then
                zc.sendHttp({
                    apiName = "Captain_bookInfo",
                    successCallback = function (net_data)
                        self:guideFunc_1()
                        net_data["tag"] = indx
                        net_data.callback = function(backData)
                            self._nowPage = nil
                            if backData then
                                self._activeMainId = backData.id
                                self._openInfo = backData.openInfo
                                self._tag = 1
                                for k,v in pairs(self._openInfo) do
                                    if tonumber(v.id) == self._activeMainId then
                                        self._tag = tonumber(v.bookId)
                                        break
                                    end
                                    if tonumber(v.bookId) > self._tag then
                                        self._tag = tonumber(v.bookId)
                                    end
                                end
                            end
                            self._myTable:reloadData()
                            self._myTable:scrollToCell(self._tag-1, true)
                        end
                        self._tag = indx
                        zc.createSourceScene("BingFaMain", "src/layer/research/BingFaMainLayer.lua", net_data)
                    end,
                })
            else
                zc.tipInfo(zc.lang("tips_bingfa_un_active"))
                --功能引导结束
                getGuideManager():setFuncGuideOver()
            end
        end,
        anchor = cc.p(0.5, 0.5),
        pos = cc.p(cellSize.width/2, cellSize.height/2)
    })
    cell:addChild(clickBtn)
    cell._sprite = clickBtn
    if indx == 1 then
        self._guide_clickBtn = clickBtn
    end

    local nameBg = zc.createSprite("res/Main/research/bingfa/name_bg.png")
    nameBg:setAnchorPoint(0.5, 1)
    nameBg:setPosition(bookSize.width-50, bookSize.height/2 + 250)
    bookSprite:addChild(nameBg, 1)
    local name = zc.createLabel({
        text = curBook.name,
        outLine = true,
        fontSize = 30,
        ap = cc.p(0.5, 1),
        pos = cc.p(nameBg:getContentSize().width/2, nameBg:getContentSize().height-20),
        pixel = 1.3,
        width = 30,
    })
    nameBg:addChild(name)

    if self._openInfo[tonumber(curBook.id)] then --如果已经激活
        local openInfo = self._openInfo[tonumber(curBook.id)]
        local advanceImgId = openInfo.advancedLevel or 1
        local advanceNum = zc.createSprite("res/Main/research/bingfa/advance_"..advanceImgId..".png")
        advanceNum:setAnchorPoint(0.5, 0.5)
        advanceNum:setPosition(nameBg:getPositionX(), bookSize.height/2 - 150)
        bookSprite:addChild(advanceNum, 1)

        local levelBg = zc.createSprite("res/Main/research/bingfa/level_bg.png")
        levelBg:setAnchorPoint(0.5, 0.5)
        levelBg:setPosition(nameBg:getPositionX()-10, advanceNum:getPositionY() - 48 )
        bookSprite:addChild(levelBg, 1)

        local levelNum = zc.createLabel({
            text = openInfo.level,
            outLine = true,
            fontSize = 26,
            pos = cc.p(levelBg:getContentSize().width/2, levelBg:getContentSize().height/2),
            pixel = 1.3
        })
        levelBg:addChild(levelNum)
        local levelJi = zc.createLabel({
            text = zc.lang("level_1"),
            outLine = true,
            fontSize = 24,
            ap = cc.p(0, 0.5),
            pos = cc.p(levelNum:getPositionX()+levelNum:getContentSize().width/2-1, levelNum:getPositionY()),
            color = cc.c3b(247,188,12),
            pixel = 1.3
        })
        levelBg:addChild(levelJi)

        if openInfo.id == self._activeMainId then
            local qiyongzhong  = zc.createSprite("res/Main/research/bingfa/qiyong.png")
            qiyongzhong:setAnchorPoint(0.5, 0)
            qiyongzhong:setPosition(bookSize.width/2, 0)
            bookSprite:addChild(qiyongzhong, 1)
        end
    else
        zc.setGray(bookSprite)
        zc.setGray(nameBg)
        name:setColor(display.COLOR_DEEPGRAY)
        --设置提示开启条件
        local captainName = LANGUAGE_KEY_GETQUNXIONGNAME_BY_ID(curBook.captainId)
        local captainOpen = curBook.combineNum
        local scaleSize = cc.size(254, 35)
        local myScale9 = zc.getScaleNode("res/Main/copy/openTargetBg.png", scaleSize)
        myScale9:setAnchorPoint(cc.p(0.5, 0))
        myScale9:setPosition(cc.p(bookSize.width/2, 0))
        bookSprite:addChild(myScale9)

        local openInfoLab = zc.createLabel({
            text = captainName.. zc.lang("activated") ..captainOpen.. zc.lang("individual") .. zc.lang("kaiqi"),
            outLine = true,
            fontSize = 22,
            pos = cc.p(50, 18),
            pixel = 1.3
        })
        myScale9:addChild(openInfoLab, 1)
        myScale9:setContentSize(cc.size(330, 35))
        openInfoLab:setPosition(myScale9:getContentSize().width/2, myScale9:getContentSize().height/2)
        myScale9:setVisible(false)

        cell._openTar = myScale9
    end
end

function ResearchBingFaLayer:onCleanup()
    if self._callback and type(self._callback) == "function" then
        self._callback()
    end
    local textureCache = cc.Director:getInstance():getTextureCache()
    textureCache:removeTextureForKey("res/animation/bingfajuanzhou/bingfajuanzhou.png")
end
function ResearchBingFaLayer:onEnter( ... )
    --新手引导
    local _currentGID = getGuideManager():getCurrentGID()
    if _currentGID == GuideKey.KEY_12004 then 
        getGuideManager():setObjTarget(self._guide_clickBtn)
        getGuideManager():startGuide()
    end
end
--功能引导
function ResearchBingFaLayer:guideFunc_1()
    --新手引导
    local _currentGID = getGuideManager():getCurrentGID()
    if _currentGID == GuideKey.KEY_12004 then
        zc.setGuideFuncNextStep(_currentGID)
    end
end

return ResearchBingFaLayer