--[[
    角色界面
    唐实聪
    2016.4.20
    2016.8.19重做
]]
local RoleLayer = class("RoleLayer", function()
    return ZCDialog:create()
end)

function RoleLayer:create( data )
	return RoleLayer.new( data )
end

function RoleLayer:ctor( data )
	self:initData( data )
	self:initUI()

    self:selectHeroAndTab( data.tab or 1 )
    if data.goBreak then
        self:goBreakthroughLayer(true)
    end

    zc.addEventListener({
        name = CUSTOM_EVENT.EVENT_ROLE_SELECT_HERO_TAB,
        callback = function(event)
            local _heroId = event.data.heroId or 0
            local _index = 0
            for i, v in ipairs( self._heroList ) do
                if tonumber( v.id ) == _heroId then
                    _index = i
                    break
                end
            end
            if _index == 0 then
                return
            end
            self._heroIndex = _index

            local _tabId = event.data.tabId
            self:selectHeroAndTab( _tabId )
        end
    })

    -- 刷新红点监听
    zc.addEventListener({
        name = CUSTOM_EVENT.EVENT_ROLELAYER_CHANGE,
        callback = function(event)
            local _id = tonumber( event.data.id ) or 0
            self:refreshRedDot( _id )
        end
    })
    -- 主角换装监听
    zc.addEventListener({
        name = CUSTOM_EVENT.EVENT_ROLELAYER_MAINROLE_CHANGE,
        callback = function(event)
            -- 刷新tableview
            self._heroTableview:updateCellAtIndex( self._heroIndex - 1 )
            local _texture = cc.Director:getInstance():getTextureCache()
            _texture:removeTextureForKey( self._cache[( self._cacheNum + 1 )/2] )
            local _cellIndex = self._heroTableview:getCurrentPage()
            local _cell = self._heroTableview:cellAtIndex( _cellIndex )
            self._cache[( self._cacheNum + 1 )/2] = _cell._heroPath
        end
    })
    -- 升星刷新技能
    zc.addEventListener({
        name = CUSTOM_EVENT.EVENT_ROLE_REFRESHSKILL,
        callback = function(event)
            self:refreshSkill()
        end
    })
end
function RoleLayer:onCleanup()
    self:removeAllChildren()
    zc.removeEventListener( CUSTOM_EVENT.EVENT_ROLE_SELECT_HERO_TAB )
    zc.removeEventListener( CUSTOM_EVENT.EVENT_ROLELAYER_CHANGE )
    zc.removeEventListener( CUSTOM_EVENT.EVENT_ROLE_REFRESHSKILL )
    zc.removeEventListener( CUSTOM_EVENT.EVENT_ROLELAYER_MAINROLE_CHANGE )
    local adm = ccs.ArmatureDataManager:getInstance()
    adm:removeArmatureFileInfo( "res/animation/shengxing/shengxing.csb", true )
    adm:removeArmatureFileInfo( "res/animation/shengxing_star/shengxing_star.csb", true )
    adm:removeArmatureFileInfo( "res/image/battle/hero/change.csb", true )
    -- dump( self._record, "self._record" )
    local _forms = PlayerData:getNowFormationFiles()
    -- dump( _forms, "_forms" )
    local _remove = {}
    local adm = ccs.ArmatureDataManager:getInstance()
    for i, v in ipairs( self._record ) do
        -- 如果已上阵，不删除
        local _path, _resId = zc.battle.getHeroArmaturePathByResId( v[1], v[2] )
        if not _forms[_resId] and not _remove[_path] then
            -- print("~~~~~~~~~~~~~~~~~~~删除人物资源=",_path)
            adm:removeArmatureFileInfo( _path )
            _remove[_path] = true
        end
    end
    helper.collectMemory()
end
function RoleLayer:onEnter()
    if self._init then
        -- 刷新红点
        self:refreshRedDots()
    else
        self._init = true
    end

    -- 刷新界面
    if self._infoLayer then
        if self._infoLayer.onEnter then
            self._infoLayer:onEnter()
        end
    end
    
    --新手引导
    local _currentGID = getGuideManager():getCurrentGID()
    if inArray(_currentGID, GuideGroup.BTN_BACK_MAINCITY) then
        zc.addGuideBtn(self._back)
    elseif inArray(_currentGID, GuideGroup.DUI_HUA) then
        if _currentGID == GuideKey.KEY_11802 then
            performWithDelay(self, function()
                local function  _callBack()
                    getGuideManager():setFuncGuideOver()
                end
                getGuideManager():setHeroFuncCallBack(_callBack)
                getGuideManager():startGuide()
            end, 0.01)
        else
            local function _callBack()
                local _currentGID = getGuideManager():getCurrentGID()
                if inArray(_currentGID, GuideGroup.BTN_SELECT_EQUIP) then
                    self._infoLayer:guideFunc()
                end
            end
            getGuideManager():setHeroFuncCallBack(_callBack)
            getGuideManager():startGuide()
        end
    elseif inArray(_currentGID, GuideGroup.BTN_SELECT_EQUIP) then
        self._infoLayer:guideFunc()
    elseif _currentGID == GuideKey.KEY_11403
      or _currentGID == GuideKey.KEY_11510 
      or _currentGID == GuideKey.KEY_12310 
      then
        getGuideManager():setObjTarget(self._guideTagBtn)
        getGuideManager():startGuide()
    elseif _currentGID == GuideKey.KEY_14304 then
        getGuideManager():setObjTarget(self._breakthroughBtn)
        getGuideManager():startGuide()
    elseif _currentGID == GuideKey.KEY_12314 then
        getGuideManager():setObjTarget(self._back)
        getGuideManager():startGuide()
    end
end
-- 数据
function RoleLayer:initData( data )
    -- 主角武将id
    self._mainHeroId = tonumber( PlayerData.info.heroId ) or 0

    --武将列表
    self._heroList = {}
    local _heroList = PlayerData:getGeneralInfo()
    local _mainHeroData = {}
    for i, v in ipairs( _heroList ) do
        if tonumber( v.id ) ~= self._mainHeroId then
            self._heroList[#self._heroList + 1] = v
        else
            _mainHeroData = v
        end
    end
    table.sort(self._heroList, function( a, b )
        -- local _qlft_a = tonumber( a.qualifications ) or 0
        -- local _qlft_b = tonumber( b.qualifications ) or 0
        -- if _qlft_a ~= _qlft_b then
        --     return _qlft_a > _qlft_b
        -- else
            return tonumber( a.fighting ) > tonumber( b.fighting )
        -- end
    end)
    table.insert(self._heroList, 1, _mainHeroData)

    -- 查找heroId对应index
    -- index默认是1
    self._heroIndex = 1
    local _heroId = tonumber( data.id ) or 0
    if _heroId ~= 0 then
        -- 传入id，查找index
        for i, v in ipairs( self._heroList ) do
            if _heroId == v.id then
                self._heroIndex = i
                break
            end
        end
    end

    -- 特殊需求，无限滑动，呵呵
    if #self._heroList > 1 then
        table.insert( self._heroList, 1, self._heroList[#self._heroList] )
        table.insert( self._heroList, #self._heroList + 1, self._heroList[2] )
        self._heroIndex = self._heroIndex + 1
    end

    -- 标签配置
    --[[
        file = 界面文件
        title = 标题路径
        openTag = 功能开启id
    ]]
    self._tabInfo = {
        [1] = {
            -- 总览
            file = "src/layer/role/RoleAttributeLayer.lua",
            name = zc.lang("country_overview"),
            index = 1,
        },
        [2] = {
            -- 培养
            file = "src/layer/role/RoleCultureLayer.lua",
            name = LANGUAGE_ROLE.culture,
            openTag = 13,
            index = 2,
        },
        [3] = {
            -- 羁绊
            file = "src/layer/role/RoleFetterLayer.lua",
            name = LANGUAGE_ROLE.fetter,
            openTag = 18,
            index = 3,
        },
        [4] = {
            -- 升星
            file = "src/layer/role/RoleStarupLayer.lua",
            name = LANGUAGE_ROLE.starup,
            openTag = 14,
            index = 4,
        },
        [5] = {
            -- 宝石
            file = "src/layer/role/RoleGemLayer.lua",
            name = LANGUAGE_SHOP.shop_gem,
            openTag = 37,
            index = 5,
        },
    }

    local _size = self:getContentSize()
    self._width = _size.width
    self._height = _size.height

    -- 记录创建的武将动画
    self._record = {}

    -- 记录武将大图
    self._cache = {}
    -- 最多缓存武将大图数量
    self._cacheNum = 5
    for i = 1, self._cacheNum do
        self._cache[i] = ""
    end
end
-- ui
function RoleLayer:initUI()
    -- 通用背景
    local _commonBg = cc.Sprite:create( "res/Main/common/common_bg.jpg" )
    _commonBg:setPosition( self._width/2, self._height/2 )
    self:addChild( _commonBg )
    zc.setBgSize( _commonBg )

    zc.getYunDongTai(2, 2, function(armUp, armDown)
        if not tolua.isnull(self) then
            armDown:setPositionY( 40 )
            self:addChild( armDown, 4 )
        end
    end)

    -- 底墨
    local _inkBottom = cc.Sprite:create( "res/Main/role/common/ink_bottom.png" )
    _inkBottom:setAnchorPoint( 0.5, 1 )
    _inkBottom:setPosition( self._width/4 - 49, self._height )
    self:addChild( _inkBottom, 1 )
    self._inkBottom = _inkBottom
    -- tableview
    local _tableviewWidth = self._width*0.5
    -- local _tableviewWidth = self._width*0.5 - 98
    local _tableviewHeight = self._height
    local _heroTableview = zc.ZCTableView:create( cc.size( _tableviewWidth, _tableviewHeight ) )
    self:addChild( _heroTableview, 1 )
    self._heroTableview = _heroTableview
    -- 底墨
    local _inkTop = cc.Sprite:create( "res/Main/role/common/ink_top.png" )
    _inkTop:setAnchorPoint( 0, 0 )
    _inkTop:setPosition( 0, 0 )
    self:addChild( _inkTop, 1 )
    self._inkTop = _inkTop
    --突破入口
    local _breakthroughBtn = zc.createPushButton({
        normalFile = "res/Main/role/breakthrough/breakthroughBtn.png",
        pos = cc.p(120, 100),
        endCallback = function()
            self:goBreakthroughLayer(true)
        end,
    })
    self:addChild( _breakthroughBtn, 5)
    self._breakthroughBtn = _breakthroughBtn
    local _btBtnEff = zc.getSpine("speedsudu")
    _btBtnEff:playAction("animation", true)
    _breakthroughBtn:addChild(_btBtnEff, -1)
    _btBtnEff:setPosition(_breakthroughBtn:getContentSize().width*0.5, _breakthroughBtn:getContentSize().height*0.5)
    -- 技能
    self._skill = {}
    for i = 1, 4 do
        local _skillIconBg = cc.Sprite:create( "res/Main/role/fetter/fetterItemBg.png" )
        _skillIconBg:setPosition( 97*( i - 0.5), 70 )
        _skillIconBg:setScale( 0.8 )
        self:addChild( _skillIconBg, 5 )
        self._skill[i] = _skillIconBg
        local _skillIcon = zc.createIconByData({
            data = {
                mainType=zc.mainTypeSkill,
                type=1, 
                id=1,
            },
            pos = cc.p( _skillIconBg:getContentSize().width/2, _skillIconBg:getContentSize().height/2 ),
        })
        _skillIconBg:addChild( _skillIcon )
        _skillIconBg._skillIcon = _skillIcon
        local _tag = cc.Sprite:create()
        _tag:setPosition( _skillIcon:getPositionX() - 5, 5 )
        _skillIconBg:addChild( _tag )
        local _num = cc.Sprite:create("res/Main/role/1.png")
        _num:setPosition(_tag:getPositionX() + 10,5)
        _skillIconBg:addChild( _num )
        _num:setVisible(false)
        _skillIconBg._num = _num
        _skillIconBg._tag = _tag
    end

    _heroTableview:setAutoAlign( true )
    _heroTableview:setInertia( true )
    _heroTableview:setDirection( cc.SCROLLVIEW_DIRECTION_HORIZONTAL )
    _heroTableview:setPosition( 0, 0 )
    _heroTableview:setDelegate()
    _heroTableview:setVerticalFillOrder( cc.TABLEVIEW_FILL_TOPDOWN )
    local function cellSizeForTable(table, cellIndex)
        return _tableviewHeight, _tableviewWidth
    end
    local function tableCellAtIndex(table, cellIndex)
        local cell = table:dequeueCell()
        -- 创建模板
        if cell == nil then
            cell = zc.ZCTableViewCell:new()
            local _heroSp = cc.Sprite:create()
            -- _heroSp:setPosition( _tableviewWidth/2, _tableviewHeight/2 )
            _heroSp:setPosition( _tableviewWidth/2 - 98*0.5, _tableviewHeight/2 )
            cell:addChild( _heroSp, 1 )
            cell._heroSp = _heroSp
            local _light = cc.Sprite:create()
            _light:setPosition( _heroSp:getPosition() )
            cell:addChild( _light )
            cell._light = _light
        end
        local _index = cellIndex + 1
        local _data = self._heroList[_index]
        local _heroName = _data.picture
        local _heroPath = string.format("res/Main/hero/body/%s.png",_heroName)
        if self._mainHeroId == tonumber( _data.id ) then
            -- 主角
            local _head = nil
            _head, _heroPath = PlayerData:getMainRoleHeadImg()
        end
        if not zc.checkFileExist( _heroPath ) then
            _heroPath = "res/Main/hero/body/1.png"
        end
        cell._heroPath = _heroPath
        cell._heroSp:setTexture( _heroPath )
        cell._light:setTexture( "res/Main/role/common/light_"..tostring( _data.light )..".png" )

        return cell
    end
    local function numberOfCellsInTableView()
        return #self._heroList
    end
    local function tablePageAfterSliding( table, cell )
        local _curIndex = table:getCurrentPage() + 1
        if self._heroIndex ~= _curIndex then
            if _curIndex == 1 then
                -- 滑到最左边，移动到倒数第二个
                table:scrollToCell( #self._heroList - 2 )
            elseif _curIndex == #self._heroList then
                -- 滑到最右边，移动到第二个
                table:scrollToCell( 1 )
            else
                -- 其他正常刷新
                self._heroIndex = _curIndex
                self:refreshHero()
            end
            self:checkCache( cell._heroPath )
        end
    end
    local function scorllviewScriptScroll( table )
        local _offSetX = table:getContentOffset().x
        _offSetX = _offSetX%_tableviewWidth
        local _pct = _offSetX/_tableviewWidth - 0.5
        _pct = math.abs( _pct )*2
        _pct = 1 - _pct
        local _width = 230
        local _height = 290
        self._inkBottom:setPosition( self._width/4 - 49 + _width*_pct, self._height + _height*_pct )
        self._inkTop:setPosition( -_width*_pct, -_height*_pct )
    end
    _heroTableview:registerScriptHandler( cellSizeForTable, cc.TABLECELL_SIZE_FOR_INDEX )
    _heroTableview:registerScriptHandler( tableCellAtIndex, cc.TABLECELL_SIZE_AT_INDEX )
    _heroTableview:registerScriptHandler( numberOfCellsInTableView, cc.NUMBER_OF_CELLS_IN_TABLEVIEW )
    _heroTableview:registerScriptHandler( tablePageAfterSliding,zctech.TABLECELL_AFTER_SLIDING )
    _heroTableview:registerScriptHandler( scorllviewScriptScroll, zctech.SCROLLVIEW_SCRIPT_SCROLL )
    _heroTableview:reloadData()

    -- 花
    local _flower_l_u = cc.Sprite:create( "res/Main/common/flower_l_u.png" )
    _flower_l_u:setAnchorPoint( 0, 1 )
    _flower_l_u:setPosition( 0, self._height )
    self:addChild( _flower_l_u, 3 )
    local _flower_r_u = cc.Sprite:create( "res/Main/common/flower_r_u.png" )
    _flower_r_u:setAnchorPoint( 1, 1 )
    _flower_r_u:setPosition( self._width, self._height )
    self:addChild( _flower_r_u, 3 )
    local _flower_r_d = cc.Sprite:create( "res/Main/common/flower_r_d.png" )
    _flower_r_d:setAnchorPoint( 1, 0 )
    _flower_r_d:setPosition( self._width, 0 )
    self:addChild( _flower_r_d, 3 )
    -- 标签页
    self:initTabs()
    -- 关闭
    local _back = zc.createTopBackButton({
        listener = function() 
            --新手引导
            local __breakthrough = self._breakthroughLay
            local _currentGID = getGuideManager():getCurrentGID()
            if inArray(_currentGID, GuideGroup.BTN_BACK_MAINCITY) then
                zc.setGuideFuncNextStep(_currentGID)
                if _currentGID == GuideKey.KEY_5038 then
                    local _tollLay = zc.getScene("tollgate")
                    if _tollLay then
                        getGuideManager():setCurrentGID(GuideKey.KEY_6001)
                    end
                    LayerManager.removeLayout()
                end
            elseif _currentGID == GuideKey.KEY_12314 then
                LayerManager.removeLayout()
                getGuideManager():setFuncGuideOver()
            end
            if __breakthrough then
                self:goBreakthroughLayer(false)
            else
                LayerManager.removeLayout()
            end
        end,
    })
    self:addChild( _back, 3 )
    self._back = _back

    for i = 1, 4 do
        performWithDelay(self, function()
            local _flower = cc.ParticleSnow:createWithTotalParticles( 1 )
            _flower:setPosition( 0, self._height )
            _flower:setPosVar( cc.p( 30, self._height/2 ) )
            _flower:setTexture( cc.Director:getInstance():getTextureCache():addImage("res/Main/role/common/huaban"..i..".png") )
            self:addChild( _flower, 5 )
            _flower:setEmissionRate( 0.7 )
            _flower:setStartSize( 20 )
            _flower:setStartSizeVar( 5 )
            local _time = 1.5
            _flower:setLife( _time )
            _flower:setEndColor( cc.c4b( 242, 168, 230, 0 ) )
            _flower:setEndColorVar( cc.c4b( 242, 168, 230, 0 ) )
            local _radians = math.atan( self._height/self._width )
            local _degrees = CC_RADIANS_TO_DEGREES( _radians )
            _flower:setAngle( -_degrees )
            _flower:setSpeed( 50 )
            _flower:setSpeedVar( 10 )
            _flower:setAutoRemoveOnFinish(true)
        end, 0.2*( i - 1 ))
    end
end
-- 创建标签按钮
function RoleLayer:initTabs()
    -- 创建标签
    self._tabBtn = {}
    local _tabPosX = self._width/2 + 538
    local _tabPosY = self._height/2 + 280
    for i, v in ipairs( self._tabInfo ) do
        local _tabInfo = v

        local _normal = cc.Sprite:create( "res/Main/common/button/tab_small_u.png" )
        local _normal_label = zc.createLabel({
            color = cc.c3b( 122, 42, 0 ),
            color2 = cc.c3b( 255, 232, 128 ),
            outLine = true,
            pixel = 0.5,
            fontSize = 24,
            width = 30,
            pos = cc.p( _normal:getContentSize().width/2 - 8, _normal:getContentSize().height/2 + 3 ),
        })
        _normal:addChild( _normal_label )
        local _selected = cc.Sprite:create( "res/Main/common/button/tab_small_d.png" )
        local _selected_label = zc.createLabel({
            color = display.COLOR_WHITE,
            color2 = cc.c3b( 72, 37, 0 ),
            outLine = true,
            pixel = 0.5,
            fontSize = 24,
            width = 30,
            pos = cc.p( _selected:getContentSize().width/2 - 8, _selected:getContentSize().height/2 + 3 ),
        })
        _selected:addChild( _selected_label )
        local _tabBtn = zc.createPushButton({
            normalNode = _normal,
            selectedNode = _selected,
            needSwallow = true,
            pos = cc.p( _tabPosX, _tabPosY - i*98), 
            endCallback = function()
                if i ~= self._tabIndex then
                    local _currentGID = getGuideManager():getCurrentGID()
                    if _currentGID == GuideKey.KEY_11403
                      or _currentGID == GuideKey.KEY_11510
                      or _currentGID == GuideKey.KEY_12310
                      then
                        zc.setGuideFuncNextStep(_currentGID)
                    end
                    self:clickTab( _tabInfo.index )
                end
            end,
        })
        self:addChild( _tabBtn, 3 )
        self._tabBtn[i] = _tabBtn
        _tabBtn._normal_label = _normal_label
        _tabBtn._selected_label = _selected_label

        --添加红点
        local _redPoint = cc.Sprite:create( "res/Main/common/common_red_point.png" )
        _redPoint:setAnchorPoint( 1, 1 )
        _redPoint:setPosition( _tabBtn:getBoundingBox().width + 5, _tabBtn:getBoundingBox().height - 5 )
        _redPoint:setVisible( false )
        _tabBtn:addChild( _redPoint )
        _tabBtn._redPoint = _redPoint
    end
end
-- 切换钮标签处理
function RoleLayer:clickTab( index )
    -- 开启条件
    local _tabId = self._tabConfig[index]
    local _tabInfo = self._tabInfo[_tabId]
    if _tabInfo.openTag then
        local _flag = zc.getFunctionStatus( _tabInfo.openTag, true )
        if not _flag then
            return
        end
    end

    if self._tabIndex then
        self._tabBtn[self._tabIndex]:setSelected( false )
    end
    self._tabIndex = index
    self._tabBtn[self._tabIndex]:setSelected( true )

    self:refreshInfo()
end
-- 刷新内容
function RoleLayer:refreshInfo()
    -- 刷新技能
    self:refreshSkill()

    local _heroData = self._heroList[self._heroIndex]
    local _tabId = self._tabConfig[self._tabIndex]
    if self._infoLayer then
        if self._infoLayer.tabId == _tabId then
            -- 直接刷新
            if self._infoLayer.onExit then
                self._infoLayer:onExit()
            end
            self._infoLayer:refreshHero({
                id = _heroData.id,
                record = self._record,
            })
            if self._infoLayer.onEnter then
                self._infoLayer:onEnter()
            end
            return
        else
            -- 重新创建
            self._infoLayer:removeFromParent()
        end
    end
    local _tabInfo = self._tabInfo[_tabId]
    local _layer = requires( _tabInfo.file ):create({
        id = _heroData.id,
        record = self._record
    })
    _layer:setPosition( self._width/2 + 217, self._height/2 - 25 )
    self:addChild( _layer, 2 )
    _layer.tabId = _tabId
    self._infoLayer = _layer
end
-- 换武将刷新
function RoleLayer:refreshHero( tabId, freshHero )
    local _heroData = self._heroList[self._heroIndex]
    -- 当前界面标签数据
    local _tabConfig = {}
    if self._mainHeroId == tonumber( _heroData.id ) then
        -- 主角
        _tabConfig = { 1, 5, 4, 2 }
    else
        -- 英雄
        _tabConfig = { 1, 5, 4, 2, 3 }
    end
    -- 查找当前的tabid
    local _tabId = 1
    if tabId then
        -- 初始化
        _tabId = tabId
    else
        -- 滑动
        _tabId = self._tabConfig[self._tabIndex]
    end
    local _tabIndex = 1
    for i, v in ipairs( _tabConfig ) do
        if _tabId == v then
            _tabIndex = i
            break
        end
    end
    self._tabConfig = _tabConfig

    -- tab按钮文字
    for i, v in ipairs( self._tabBtn ) do
        local _tabId = _tabConfig[i]
        if _tabId then
            local _tabInfo = self._tabInfo[_tabId]
            v._normal_label:setText( _tabInfo.name )
            v._selected_label:setText( _tabInfo.name )
            --判断开启状态
            local openState = true
            if _tabInfo.openTag then
                openState = zc.getFunctionStatus( _tabInfo.openTag, false )
            end
            if self._breakthroughLay then
                openState = false
            end
            if openState then
                v:setVisible( true )
            else
                v:setVisible( false )
            end
            local _currentGID = getGuideManager():getCurrentGID()
            if (_currentGID == GuideKey.KEY_11403 and tonumber(_tabId) == 4)
            or (_currentGID == GuideKey.KEY_12310 and tonumber(_tabId) == 5) then
                self._guideTagBtn = v
            end
        else
            v:setVisible( false )
        end
    end

    --刷新按钮位置
    local _tabPosY = self._height/2 + 280
    local _visIndex = 1
    for i, v in ipairs( self._tabBtn ) do
        local _btn = v
        local _isVis = _btn:isVisible() or false
        if _isVis then
            _btn:setPositionY(_tabPosY - _visIndex*98)
            _visIndex = _visIndex + 1
        end
    end

    local _freshHero = freshHero == nil and true or freshHero
    if _freshHero then
        self:clickTab( _tabIndex )
    end

    -- 刷新红点
    self:refreshRedDots()
    if self._breakthroughLay then
        self._breakthroughLay:selfFresh(_heroData.id)
    end
end
-- 刷新技能图标
function RoleLayer:refreshSkill()
    local _tabId = self._tabConfig[self._tabIndex]
    local _flag = not self._breakthroughLay and _tabId == 1
    self._breakthroughBtn:setVisible(_flag)

    if _tabId ~= 4 then
        for i, v in ipairs( self._skill ) do
            v:setVisible( false )
        end
        return
    end
    -- 技能数据
    -- dump(self._heroList[self._heroIndex],"self._heroList:")
    local _newAdvance = tonumber(self._heroList[self._heroIndex].newAdvance) or 0
    local combineSkillLevelList_
    local jienng_ = false
    if tonumber(self._heroList[self._heroIndex].combineSkillId2) ~= 0 then
        combineSkillLevelList_ =  {self._heroList[self._heroIndex].combineSkillLevel,self._heroList[self._heroIndex].combineSkillLevel2}
        jienng_ = true
    else
        combineSkillLevelList_ = nil
        jienng_ = false
    end
    local _params = {
        heroId = self._heroList[self._heroIndex].heroId,
        star = self._heroList[self._heroIndex].star, 
        newAdvance = newAdvance,
        combineSkillLevelList = combineSkillLevelList_,
        jineng = jienng_,
    }
    if self._heroList[self._heroIndex].id == tonumber(PlayerData.info.heroId) then
        _params.heroData = self._heroList[self._heroIndex]
    end
    local _skillData = zc.getHeroSkillData(_params)
    local num = 1
    local list = {}
    for k,v in pairs(_skillData) do
        if v.skillType == 4 then
            if v.id == v.combineSkillId then
                table.insert(list,1,v)
            else
                table.insert(list,v)
            end
        else
            table.insert(list,v)
        end
    end

    for k,v in pairs(_skillData) do
        if v.skillType == 4 then
            num = num + 1
        end
    end

    for i, v in ipairs( self._skill ) do
        local _data = list[i]
        -- dump(_data,"_data**********_data")
        if _data then
            local _flag
            -- print("tonumber(self._heroList[self._heroIndex].combineSkillId2) ",tonumber(self._heroList[self._heroIndex].combineSkillId2))
            if tonumber(self._heroList[self._heroIndex].combineSkillId2) ~= 0 then
                if _data.skillType == 4 and _data.id == tonumber(self._heroList[self._heroIndex].combineSkillId2) then
                    _flag = not zc.getFunctionStatus(48,false)
                else
                    _flag = _data.level == 0
                end
            else
                _flag = _data.level == 0
            end
            
            if _data.stageSkillActive then
                _flag = _newAdvance < _data.stageSkillActive 
            end
            -- print("_flag:",_flag,"_newAdvance:",_newAdvance,"_data.stageSkillActive ",_data.stageSkillActive )
            zc.updateIconByData(v._skillIcon, {
                data = {
                mainType=zc.mainTypeSkill,
                type=_data.id, 
                id=_data.id,
            },
            showGray = _flag,
            listener = function()
                local _level = _data.level
                zc.createSkillInfoPop({
                    id = _data.id, 
                    level = _level,
                    heroData = self._heroList[self._heroIndex],
                    type = _data.skillType,
                    combineSkillId = _data.combineSkillId
                    })
            end})
            if _data.skillType == 3 then
                -- 必杀
                v._tag:setTexture( "res/Main/pub/critical.png" )
                v._tag:setVisible( true )
                v._num:setVisible(false)
            elseif _data.skillType == 4 then
                -- 奥义
                if num > 1 then
                    v._tag:setTexture( "res/Main/pub/aoyi.png" )
                    v._tag:setVisible( true )
                    v._num:setTexture("res/Main/role/"..i..".png")
                    v._num:setPosition(v._tag:getPositionX() + v._tag:getContentSize().width/2 - 15,5)
                    v._num:setVisible(true)
                else
                    v._tag:setTexture( "res/Main/pub/aoyi.png" )
                    v._tag:setVisible( true )
                    v._num:setPosition(v._tag:getPositionX() + v._tag:getContentSize().width/2 - 15,5)
                    v._num:setVisible(true)
                end
            else
                v._num:setVisible(false)
                v._tag:setVisible( false )
            end
            
            v:setVisible( true )
        else
            v:setVisible( false )
        end
    end
end

function RoleLayer:checkCache( path )
    local _max = self._cacheNum
    local _middle = ( _max + 1 )/2
    local _cache = self._cache
    local _texture = cc.Director:getInstance():getTextureCache()
    if _cache[_middle] == "" then
        -- 刚开始，没有中间
        _cache[_middle] = path
    else
        if _cache[_middle] == path then
            -- 中间就是新的，位置没有变化
        elseif _cache[_middle - 1] == path then
            -- 左边等于新的
            print("~~~~~~~~~~~~~~remove=", _cache[_max])
            _texture:removeTextureForKey( _cache[_max] )
            _cache[_max] = nil
            table.insert( _cache, 1, "" )
        elseif _cache[_middle + 1] == path then
            -- 右边等于新的
            print("~~~~~~~~~~~~~~remove=", _cache[1])
            _texture:removeTextureForKey( _cache[1] )
            table.remove( _cache, 1 )
            _cache[_max] = ""
        elseif _cache[_middle - 1] == "" then
            -- 左边不存在，插入左边
            print("~~~~~~~~~~~~~~remove=", _cache[_max])
            _texture:removeTextureForKey( _cache[_max] )
            _cache[_max] = nil
            table.insert( _cache, 1, "" )
            _cache[_middle] = path
        elseif _cache[_middle + 1] == "" then
            -- 右边不存在，插入右边
            print("~~~~~~~~~~~~~~remove=", _cache[1])
            _texture:removeTextureForKey( _cache[1] )
            table.remove( _cache, 1 )
            _cache[_max] = ""
            _cache[_middle] = path
        end
    end
    self._cache = _cache
    -- dump(_cache,"_cache")
end

function RoleLayer:selectHeroAndTab( tabId )
    self:refreshHero( tabId )

    -- 滑动到武将
    self._heroTableview:scrollToCell( self._heroIndex - 1 )
    do
        local _cellIndex = self._heroTableview:getCurrentPage()
        local _cell = self._heroTableview:cellAtIndex( _cellIndex )
        self:checkCache( _cell._heroPath )
    end
end

-------------------------红点-------------------------
-- 刷新所有红点
function RoleLayer:refreshRedDots()
    for i, v in ipairs( self._tabConfig ) do
        self:refreshRedDot( v )
    end
end
-- 刷新单个红点，传id方便功能内部刷新红点
function RoleLayer:refreshRedDot( _id )
    local _index
    for i, v in ipairs( self._tabConfig ) do
        if v == _id then
            _index = i
        end
    end
    if not _index then
        return
    end
    local _flag = false
    if _id == 1 then
        -- 总览
        -- 装备
        _flag = zc.getEquipRedDotStatus({ self._heroList[self._heroIndex] })
        if not _flag then
            local _heroId = tonumber( self._heroList[self._heroIndex].id ) or 0
            if self._mainHeroId == _heroId then
                -- 主角神兵
                _flag = PlayerData:getGodWeaponRedDotState()
            else
                -- 武将换武器 洗练
                _flag = zc.getChangeWeaponRedDotStatus({ self._heroList[self._heroIndex] })
                if _flag == false then
                    _flag = PlayerData:checkCanClearWeaponsByHeroId( self._heroList[self._heroIndex].id )
                end
            end
        end
    elseif _id == 2 then
        -- 培养
    elseif _id == 3 then
        -- 羁绊
        _flag = zc.getFetterRedDotStatus({ self._heroList[self._heroIndex] })
    elseif _id == 4 then
        -- 升星
        _flag = zc.getStarupRedDotStatus({ self._heroList[self._heroIndex] })
    elseif _id == 5 then
        -- 宝石
        _flag = zc.getGemSetRedDotStatusByHero({ self._heroList[self._heroIndex] })
    end
    print("红点 _id=",_id," _flag=",_flag)
    self._tabBtn[_index]._redPoint:setVisible( _flag )
end

function RoleLayer:goBreakthroughLayer( isShow )
    if isShow then
        local _flag = zc.getFunctionStatus(43, true)
        if not _flag then
            return
        end
        self._breakthroughBtn:setVisible(false)
        if self._infoLayer then
            self._infoLayer:setVisible(false)
        end
        for i, v in ipairs( self._tabBtn ) do
            v:setVisible(false)
        end
        if self._breakthroughLay then
            return
        end
        local _heroData = self._heroList[self._heroIndex]
        local _breakthroughLay = requires("src/layer/role/RoleBreakthroughLayer.lua"):create({
            roleLayer = self,
            id = _heroData.id,
        })
        self:addChild(_breakthroughLay, 5)
        local _top = LayerManager.getTopInfoLayer()
        _top:setPopInfo("RoleBreakthrough")
        self._breakthroughLay = _breakthroughLay
    else
        local _top = LayerManager.getTopInfoLayer()
        _top:setPopInfo("RoleAttributeLayer")
        if self._breakthroughLay then
            self._breakthroughLay:removeFromParent()
            self._breakthroughLay = nil
        end
        self._breakthroughBtn:setVisible(true)
        if self._infoLayer then
            self._infoLayer:setVisible(true)
        end
        self:refreshHero(nil, false)
    end
end

return RoleLayer