--[[
	兑换
	唐实聪
	2016.3.29
	1. 	建筑商店
	2.	军功商店
	3. 	突飞卡商店
]]
local ShopLayer = class("ShopLayer", function()
	return ZCDialog:create()
end)

function ShopLayer:create( params )
	return ShopLayer.new( params )
end

function ShopLayer:onCleanup()
	self:stopTimer()
	self:removeAllChildren()
	helper.collectMemory()
end

function ShopLayer:ctor( params )
	self:initData( params )
	self:initUI()

	self:refreshShop( params.data )
end

-- 创建数据
function ShopLayer:initData( params )
	-- 初始化标签数据
	self:initTabData( params.index )
	-- 商品数据
	self._goodsData = {}
	-- 刷新使用的数据
	self._flushData = {}
	-- 倒计时
	self._timer = nil

	self._size = self:getContentSize()
	self.Total_refresh = gameData.getDataFromCacheWithPrimaryKey("vip",7)
	self.free_total_refresh = gameData.getDataFromCacheWithPrimaryKey("vip",11)
end
-- 创建分页的数据
function ShopLayer:initTabData( index )
	-- 所有建筑的主键id
	local _houseData = {}
	for i, v in ipairs( PlayerData:getHouseInfo().houses ) do
		_houseData[tonumber( v.houseId )] = v.id
	end
	--[[
		name      = 商城名称
		initOp    = 初始化接口
		refreshOp = 刷新接口
		buyOp     = 购买接口
		params    = 后端参数，各接口公用
		db        = 商品静态表
		limit     = 商品购买是否有次数限制
		popHide   = 弹窗购买完是否关闭
		topInfo   = 顶部信息配置
		openState = 开启状态
	]]
	local tabList = {
		[1] = {
			name   = LANGUAGE_SHOP.goods_shop,
			initOp = "Timelimit_shop_main",
			buyOp  = "Timelimit_shop_buy",
			params = {},
			db       = "limit_mall",
			limit    = true,
			popHide  = false,
			openState= true,
			topInfo  = "TimeLimitShopLayer",
			activityId = 13, --活动id(根据活动数据检查开启状态)
		},
		[2] = {
			name      = LANGUAGE_SHOP.shop_arena,
			initOp    = "HouseShop_goods",
			refreshOp = "HouseShop_goodsFlush",
			buyOp     = "HouseShop_goodsBuy",
			params    = {
				type="2",
				houseId=_houseData[12],
				arenaHouseId=_houseData[13],
			},
			db      = "arena_goods",
			limit   = false,
			popHide = true,
			openState= true,
			topInfo = "ArenaShopLayer",
		},
		[3] = {
			name   = LANGUAGE_SHOP.shop_gem,
			initOp = "HouseShop_goods",
			buyOp  = "HouseShop_goodsBuy",
			params = {
				type="4",
				houseId=_houseData[12],
			},
			db       = "shop_goods",
			limit    = false,
			popHide  = false,
			openState= true,
			topInfo  = "GemShopLayer",
		},
		[4] = {
			name   = LANGUAGE_SHOP.shengwang..LANGUAGE_SHOP.shop,
			initOp    = "ShengWang_goods",
			refreshOp = "ShengWang_goodsFlush",
			buyOp     = "ShengWang_goodsBuy",
			params = {
				type = 5
			},
			db       = "prestigeShop",
			limit    = true,
			popHide  = false,
			openState= true,
			topInfo  = "PrestigeShopLayer",
		}
	}
	self._tabData = {}
	for i = 1, #tabList do
		if tabList[i] and tabList[i].openState then
			self._tabData[#self._tabData+1] = tabList[i]
		end
	end


	if not self._tabData[2].params.houseId or not self._tabData[2].params.arenaHouseId then
		self._tabData[2].params = nil
	end
	if not self._tabData[3].params.houseId then
		self._tabData[3].params = nil
	end


	self._tabIndex = index or 1
end


-- 创建界面
function ShopLayer:initUI()
	-- 模糊背景
	local _bg = cc.Sprite:create( "res/Main/common/common_bg.jpg" )
	_bg:setPosition( self._size.width/2, self._size.height/2 )
	self:addChild( _bg )
	zc.setBgSize(_bg)
	-- 背景
	local background = cc.Sprite:create()
	background:setContentSize( 885, 527 )
	background:setPosition( self._size.width/2, self._size.height/2 - 20 )
	self:addChild( background )
	self._bg = background
	self._bgSize = background:getContentSize()
	
	-- 商店背景
	local shopBg = cc.Sprite:create( "res/Main/shop/shopBg.jpg" )
	shopBg:setAnchorPoint( 0.5, 1 )
	shopBg:setPosition( self._bgSize.width/2 + 2, self._bgSize.height - 50 )
	self._bg:addChild( shopBg )

	-- 标题
	local topTitle = cc.Sprite:create( "res/Main/common/popBg/titleBg2.jpg" )
	topTitle:setAnchorPoint( 1, 1 )
	topTitle:setPosition( self._bgSize.width, self._bgSize.height )
	background:addChild( topTitle )

	-- 框
    local frame = ccui.Scale9Sprite:create( cc.rect( 13, 60, 9, 62 ), "res/Main/common/frame_s.png" )
    frame:setContentSize( self._bgSize.width, self._bgSize.height )
    frame:setPosition( self._bgSize.width/2, self._bgSize.height/2 )
    background:addChild( frame, 3 )

    -- side
    local side_b = zc.getScaleNode( "res/Main/common/popBg/shadow_b.png", cc.size( self._bgSize.width + 6, 31 ) )
    side_b:setAnchorPoint( 0, 1 )
    side_b:setPosition( -6, 10 )
    background:addChild( side_b )
    local side_r = zc.getScaleNode( "res/Main/common/popBg/shadow_r.png", cc.size( 32, self._bgSize.height + 21 ) )
    side_r:setAnchorPoint( 0, 1 )
    side_r:setPosition( self._bgSize.width - 10, self._bgSize.height )
    background:addChild( side_r )

	-- 标题
	local title = cc.Sprite:create( "res/Main/shop/title.png" )
	title:setPosition( 60, self._bgSize.height - 18 )
	self._bg:addChild( title, 3 )

	-- 关闭
	local close = zc.createBackButton({
		pos = cc.p( self._bgSize.width - 5, self._bgSize.height ),
		listener = function()
			LayerManager.removeLayout()
		end
	})
	self._bg:addChild( close, 3 )

	self:initLeft()

	self:initRight()

	self:initBottom()
end
-- 创建左边分页
function ShopLayer:initLeft()
	-- 分页背景
	local tabBg = zc.createTabButtonBg({tag=3})
	tabBg:setAnchorPoint( 0, 0 )
	tabBg:setPosition( 7, 7 )
	self._bg:addChild( tabBg, 1 )
    local splitY = zc.createSprite( "res/Main/common/tab_split_y.png" )
	splitY:setPosition( tabBg:getContentSize().width + 3, tabBg:getContentSize().height/2 + 3)
	splitY:setScaleY( 473/493 )
	tabBg:addChild( splitY, 2 )
	self._tabBtn = {}
    -- 分隔线
    local split1 = zc.createSprite("res/Main/common/tab_split_x_new.png")
    split1:setPosition(tabBg:getContentSize().width/2, 332 + 62 )
    tabBg:addChild(split1)
	-- tab
	for i, v in ipairs( self._tabData ) do
		local pos = cc.p( tabBg:getContentSize().width/2, 440 - i*62 )
		-- btn
        local tabBtn = zc.createComTabButton({
            tag = 4,
            btnText = v.name,
            touchSize = cc.size( 153, 62),
			pos = pos,
            needSwallow = false,
            needEnableWhenOut = true,
            needEnableWhenMoving = true,
			endCallback = function()
				if self._tabIndex ~= i then
					-- 判断是否需要请求后端
					self._tabBtn[i]:setSelected( true )
					self._tabBtn[self._tabIndex]:setSelected( false )
	            	if v.initOp and #v.initOp > 0 then
	            		-- 需要
	            		-- 判断参数
	            		-- dump(v, "v-----------dump--------ad-------------" )
	            		if v.params then
	            			local function gotoTab()
		            			-- 可以请求
			            		zc.sendHttp({
			            			apiName = v.initOp,
			            			params = v.params,
			            			successCallback = function( data )
			            				-- 请求成功
			            				self._tabIndex = i
			            				-- dump(data)
			            				self.shop_data = data
			            				self:refreshShop( data )
			            			end,
			            			failCallback = function()
										self._tabBtn[i]:setSelected( false )
										self._tabBtn[self._tabIndex]:setSelected( true )
			            			end,
			            			errorCallback = function()
										self._tabBtn[i]:setSelected( false )
										self._tabBtn[self._tabIndex]:setSelected( true )
			            			end,
			        			})
			        		end
			      --   		if v.activityId then
			      --   			local function outOfDate()
			      --   				self._tabBtn[i]:setVisible(false)
			      --   				self._tabBtn[i]._split:setVisible(false)
									-- self._tabBtn[i]:setSelected( false )
									-- self._tabBtn[self._tabIndex]:setSelected( true )
			      --   			end
			      --   			self:checkOpenstate(v.activityId, gotoTab, outOfDate)
			      --   		else
			        			gotoTab()
			        		-- end
		            	else
							self._tabBtn[i]:setSelected( false )
							self._tabBtn[self._tabIndex]:setSelected( true )
		            		-- 不可以请求，提示
		            		zc.tipInfo(zc.lang("tips_shop_unopen"))
		            	end
	            	else
	            		-- 不需要
	            		self._tabIndex = i
	            		self:refreshShop()
	            	end
				end
			end
        })
        tabBg:addChild( tabBtn, 1 )
        self._tabBtn[i] = tabBtn
	    -- 分隔线
        local split = zc.createSprite("res/Main/common/tab_split_x_new.png")
        split:setPosition(tabBtn:getPositionX(), tabBtn:getPositionY() - tabBtn:getContentSize().height*0.5 - 9 )
        tabBg:addChild(split)
	end
	self._tabBtn[self._tabIndex]:setSelected( true )

end
function ShopLayer:refreshTabBtnList()


end
-- 创建右边tableview
function ShopLayer:initRight()
	local goodsSize = cc.size( 713, 460 )
	local goodsTableview = zc.ZCTableView:create( goodsSize )
    goodsTableview:setInertia( true )
    goodsTableview:setDirection( cc.SCROLLVIEW_DIRECTION_VERTICAL )
    goodsTableview:setPosition( 162, 15 )
    goodsTableview:setDelegate()
    goodsTableview:setVerticalFillOrder( cc.TABLEVIEW_FILL_TOPDOWN )
    local cellWidth = goodsSize.width
    local cellHeight = 124
    local function cellSizeForTable( table, idx )
        return cellHeight, cellWidth
    end
    local function tableCellAtIndex( table, cellIndex )
    	-- print("cellIndex+ 1",cellIndex + 1)
        local cell = table:dequeueCell()
        -- 创建模板
        if cell == nil then
            cell = zc.ZCTableViewCell:new()
            self:buildCell( cell, cellIndex, cellWidth, cellHeight )
        end
        -- 更新界面
        self:updateCell( cell, cellIndex )
        return cell
    end
    local function numberOfCellsInTableView()
    	return math.ceil( #self._goodsData/2 )
    end
    goodsTableview:registerScriptHandler( cellSizeForTable, cc.TABLECELL_SIZE_FOR_INDEX )
    goodsTableview:registerScriptHandler( tableCellAtIndex, cc.TABLECELL_SIZE_AT_INDEX )
    goodsTableview:registerScriptHandler( numberOfCellsInTableView, cc.NUMBER_OF_CELLS_IN_TABLEVIEW )
    self._bg:addChild( goodsTableview )
    self._goodsTableview = goodsTableview
end
-- 创建cell
function ShopLayer:buildCell( cell, index, cellWidth, cellHeight )
	cell._index = index
	cell._item = {}
	-- print("buildCell( cell, index, cellWidth, cellHeight ). idnex ",index)
	for i = 1, 2 do
		cell._item[i] = {}
		-- cellBg
		local cellBg = cc.Sprite:create( "res/Main/shop/cellBg.png" )
		cellBg:setAnchorPoint( 0.5, 0.5 )
		cellBg:setPosition( cellWidth/4*( i*2 - 1 ), cellBg:getContentSize().height/2 )
		cell:addChild( cellBg )
		cell._item[i]._cellBg = cellBg
		local cellBgSize = cellBg:getContentSize()
		-- icon
		local icon = zc.createIconByData({
			data = {
				mainType = zc.mainTypeWeapon,
				type = 1,
				id = 1,
				quality = 1,
			},
			pos = cc.p( 50, cellBgSize.height/2 ),
		})
		icon:setScale( 0.7 )
		cellBg:addChild( icon )
		cell._item[i]._icon = icon
		-- noteBg
		local noteBg = cc.Sprite:create( "res/Main/shop/shop_zk.png" )
		noteBg:setAnchorPoint( 0, 1 )
		noteBg:setPosition( 0, cellBg:getContentSize().height + 3 )
		cellBg:addChild( noteBg, 1 )
		cell._item[i]._noteBg = noteBg
		-- note
		local note = zc.createLabel({
            fontSize = 20,
            outLine = true,
            pixel = 2,
            color = display.COLOR_WHITE,
            color2 = cc.c3b( 233, 64, 28 ),
            ap = cc.p( 0, 0.5 ),
            pos = cc.p( noteBg:getContentSize().width/2 - 27, noteBg:getContentSize().height/2 - 3 ),
        })
        note:setRotation( -45 )
        noteBg:addChild( note )
		cell._item[i]._note = note
		-- nameBg
		local nameBg = ccui.Scale9Sprite:create( cc.rect( 12, 11, 14, 10 ), "res/Main/role/common/name_bg.png" )
		nameBg:setContentSize( 116, 27 )
		nameBg:setPosition( 156, 83 )
		cellBg:addChild( nameBg )
		-- name
		local name = zc.createLabel({
			fontSize = 18,
            outLine = true,
            pixel = 1.5,
            pos = cc.p( nameBg:getContentSize().width/2, nameBg:getContentSize().height/2 ),
		})
		nameBg:addChild( name )
		cell._item[i]._name = name
		-- 拥有
		local own = zc.createLabel({
			fontSize = 20,
            color = display.COLOR_BROWN,
            stroke = false,
            pos = cc.p( 153, 30 ),
		})
		cellBg:addChild( own )
		cell._item[i]._own = own
		-- 消耗
		cell._item[i]._consume = {}
		for j = 1, 2 do
			cell._item[i]._consume[j] = {}
			-- bg
			local consumeBg = ccui.Scale9Sprite:create( cc.rect( 12, 11, 14, 10 ), "res/Main/role/common/name_bg.png" )
			consumeBg:setContentSize( 106, 30 )
			consumeBg:setPosition( 285, 120 - 30*j )
			cellBg:addChild( consumeBg )
			cell._item[i]._consume[j]._bg = consumeBg
			-- icon
			local consumeIcon = cc.Sprite:create()
			consumeIcon:setContentSize( 40, 40 )
			consumeIcon:setPosition( 260, consumeBg:getPositionY() )
			consumeIcon:setScale( 0.8 )
			cellBg:addChild( consumeIcon )
			cell._item[i]._consume[j]._icon = consumeIcon
			-- num
			local consumeNum = zc.createLabel({
				fontSize = 20,
	            outLine = true,
	            pixel = 1.5,
	            ap = cc.p( 0, 0.5 ),
	            pos = cc.p( 280, consumeBg:getPositionY() ),
			})
			cellBg:addChild( consumeNum )
			cell._item[i]._consume[j]._num = consumeNum
		end
		-- 购买按钮
		local buyBtn = zc.createCommomBtn({
        	tag = 3,
        	scale = 0.6,
        	labScale = true,
	        text = LANGUAGE_SHOP.buy_space,
	        pos = cc.p( 285, 40 ),
	        endCallback = function()
    			local _index = cell._index
    			local _tabData = self._tabData[self._tabIndex]
	        	if _tabData.db == "limit_mall" or
	        		_tabData.db == "arena_goods" then
	        		local _info = self._goodsData[2*_index + i] 
	        		if _info.buyTimes and tonumber(_info.buyTimes) > 0 then
	        			_info.buyNum = tonumber(_info.buyTimes) - _info._buyNum
	        		end
			        local params = {
			            info = _info,
			            hideAfterBuy = true,
			            addTenTimes = true,
			            notMinMax = true,
			            rightFunc = function(_num, callback)
			            	self:buyLimitGoods(_info, _num, _index)
			            end,
			        }
			        zc.createCommonBuyPop(params)
				elseif _tabData.db == "prestigeShop" then
					local info = self._goodsData[2*cell._index + i]
					-- print("test---index",cell._index)
					-- dump(info,"-----test----")

					zc.sendHttp({
						apiName = _tabData.buyOp,
						params = {
							goodsId = info.id,
							nums = 1
						},
						successCallback = function (data)
							data.noAnimation = true
				            PlayerData:updataData(data)
				            musicManager.playEffect( zc.effect.shopBuy )
				            local id = tonumber(info.id)
				            for k,v in pairs(self.shop_data.info.goods) do
				            	if tonumber(k) == id then	
				            		-- print("k" ,k)
				            		self.shop_data.info.goods[k] = 1
				            	end
				            end
				            zc.tipInfo( zc.lang("tips_buy_success") )
				            -- 刷新tableview
				            self._goodsTableview:reloadDataAndScrollToCurrentCell()
				            
				            local id = tonumber(info.id)
				            -- print("id:",id)
				            -- dump(self.shop_data)
				            for k,v in pairs(self.shop_data.info.goods) do
				            	if tonumber(k) == id then	
				            		-- print("k" ,k)
				            		self.shop_data.info.goods[k] = 1
				            	end
				            end
				            self:refreshShop(self.shop_data)
						end
					})
	        	else
		        	zc.createShopBuyPop({
		        		limit = self._tabData[self._tabIndex].limit,
		        		info = self._goodsData[2*cell._index + i],
		        		clickHide = self._tabData[self._tabIndex].popHide,
		        		callback = function(_callback)
			            	self:clickBuyCallback( _index, i, _callback )
		            	end
	            	})
	            end
	    	end
    	})
		cellBg:addChild( buyBtn )
		cell._item[i]._buyBtn = buyBtn
		local buyed_icon = zc.createSprite("res/Main/shop/buyed.png")
		buyed_icon:setScale(0.8)
		buyed_icon:setPosition(buyBtn:getPosition())
		cellBg:addChild(buyed_icon)
		cell._item[i]._buyed_icon = buyed_icon
		buyed_icon:setVisible(false)
	end 
end
-- 刷新cell
function ShopLayer:updateCell( cell, index )
	cell._index = index
	-- print("index:",index)
	for i = 1, 2 do
		local data = self._goodsData[2*index + i]
		-- dump( data, "data" )
		if data then
			if self._tabIndex == 4 then
				-- cellBg
				cell._item[i]._cellBg:setVisible( true )
				-- icon
				local params = {
					data = {
	                    mainType = data.itemType,
	                    type = data.itemId or 0,
	                    id = data.itemId,
	                    num = data.itemNum,
					},
				}
				local _info = zc.updateIconByData(cell._item[i]._icon, params)
				cell._item[i]._icon:setTouchEndedShowTip()
				data.info = _info
				cell._item[i]._name:setText( _info.name or "" )
				cell._item[i]._own:setVisible(false)
				for j = 1,2 do
					local state = true
					if j == 1 then
						state = true
					else
						state = false
					end
					cell._item[i]._consume[j]._bg:setVisible(state)
					cell._item[i]._consume[j]._num:setVisible(state)
					cell._item[i]._consume[j]._icon:setVisible(state)

					cell._item[i]._consume[j]._bg:setPositionY( 120 - 30*j )
					cell._item[i]._consume[j]._icon:setPositionY( 120 - 30*j )
					cell._item[i]._consume[j]._num:setPositionY( 120 - 30*j )

					cell._item[i]._consume[j]._num:setString(data.price)
					cell._item[i]._consume[j]._icon:setTexture( zc.getImgByType( data.priceType ) )
				end
				cell._item[i]._noteBg:setVisible(false)
				-- 已购买
				if data.buyed then
					-- for j = 1,2 do
						if self._tabData[self._tabIndex].limit and tonumber( data.buyed ) > 0 then
							-- cell._item[i]._buyBtn:setEnable(false)
							cell._item[i]._buyBtn:setVisible(false)
							cell._item[i]._buyed_icon:setVisible(true)
						else
							cell._item[i]._buyed_icon:setVisible(false)
							cell._item[i]._buyBtn:setVisible(true)
							-- cell._item[i]._buyBtn:setEnable(true)
						end
					-- end
				end
			else
				cell._item[i]._buyBtn:setVisible(true)
				cell._item[i]._buyed_icon:setVisible(false)
				-- cellBg
				cell._item[i]._cellBg:setVisible( true )
				-- icon
				local params = {
					data = {
	                    mainType = data.itemType,
	                    type = data.itemId or 0,
	                    id = data.itemId,
	                    num = data.itemNum,
	                    color = data.color,
					},
				}
				local _info = zc.updateIconByData(cell._item[i]._icon, params)
				cell._item[i]._icon:setTouchEndedShowTip()
				data.info = _info
				-- note
				if data.rebatePrice1 and tonumber( data.rebatePrice1 ) > 0 or data.rebatePrice2 and tonumber( data.rebatePrice2 ) > 0 then
					-- noteBg
					cell._item[i]._noteBg:setVisible( true )
					-- note
					cell._item[i]._note:setText( LANGUAGE_SHOP.discount )
				else
					-- noteBg
					cell._item[i]._noteBg:setVisible( false )
				end
				-- name
				cell._item[i]._name:setText( data.name or "" )
				-- own
				-- dump(data, "datadata-----------dump--------ad-------------" )
				cell._item[i]._own:setVisible(true)
				if self._tabData[self._tabIndex].db == "limit_mall" or
					self._tabData[self._tabIndex].db == "arena_goods" then
					if data.buyTimes and tonumber(data.buyTimes) > 0 then
						cell._item[i]._own:setText( zc.lang("today_can_buy")..(tonumber(data.buyTimes)-data._buyNum)..zc.lang("ci"))
					else
						-- cell._item[i]._own:setText("")
						local ownNum = zc.getNumByTypeId( data.itemType, data.itemId )
						if ownNum > 0 then
							cell._item[i]._own:setText( "【"..LANGUAGE_SHOP.own..":"..zc.getShopWan( ownNum ).."】" )
						else
							cell._item[i]._own:setText( "【"..LANGUAGE_SHOP.own..":0】" )
						end
					end
				else
					local ownNum = zc.getNumByTypeId( data.itemType, data.itemId )
					if ownNum > 0 then
						cell._item[i]._own:setText( "【"..LANGUAGE_SHOP.own..":"..zc.getShopWan( ownNum ).."】" )
					else
						cell._item[i]._own:setText( "【"..LANGUAGE_SHOP.own..":0】" )
					end
				end
				-- 消耗
				local visibleIndex = {}
				for j = 1, 2 do
					if data["priceType"..j] and data["price"..j] and tonumber( data["price"..j] ) > 0 then
						visibleIndex[#visibleIndex + 1] = j
						cell._item[i]._consume[j]._bg:setVisible( true )
						cell._item[i]._consume[j]._icon:setVisible( true )
						cell._item[i]._consume[j]._num:setVisible( true )
						cell._item[i]._consume[j]._icon:setTexture( zc.getImgByType( data["priceType"..j] ) )
						local _num = tonumber( data["rebatePrice"..j] ) or 0
						if _num == 0 then
							_num = tonumber( data["price"..j] ) or 0
						end
						cell._item[i]._consume[j]._num:setText( _num )
					else
						cell._item[i]._consume[j]._bg:setVisible( false )
						cell._item[i]._consume[j]._icon:setVisible( false )
						cell._item[i]._consume[j]._num:setVisible( false )
					end
				end
				if #visibleIndex == 1 then
					cell._item[i]._consume[visibleIndex[1]]._bg:setPositionY( 83 )
					cell._item[i]._consume[visibleIndex[1]]._icon:setPositionY( 83 )
					cell._item[i]._consume[visibleIndex[1]]._num:setPositionY( 83 )
				else
					for j = 1, 2 do
						cell._item[i]._consume[j]._bg:setPositionY( 120 - 30*j )
						cell._item[i]._consume[j]._icon:setPositionY( 120 - 30*j )
						cell._item[i]._consume[j]._num:setPositionY( 120 - 30*j )
					end
				end
				-- 已购买
				if self._tabData[self._tabIndex].limit and tonumber( data.buyed ) > 0 then
					cell._item[i]._buyBtn:setEnable(false)
				else
					cell._item[i]._buyBtn:setEnable(true)
				end
			end
		else
			-- cellBg
			cell._item[i]._cellBg:setVisible( false )
		end
	end
end
-- 创建底部
function ShopLayer:initBottom()
	-- 背景
	local bottomImg = ccui.Scale9Sprite:create( cc.rect( 20, 10, 549, 71 ), "res/Main/role/trinket/sellDi.png" )
    bottomImg:setContentSize( 720, 90 )
	local bottom = zc.createPushButton({
		normalNode = bottomImg,
		needSwallow = true,
		needEnableWhenMoving = true,
		noScaleAction = true,
		anchor = cc.p( 0.5, 0),
	})
	bottom:setPosition( self._goodsTableview:getPositionX() + self._goodsTableview:getBoundingBox().width/2 + 0, 7 )
	self._bg:addChild( bottom, 2 )
	self._bottom = bottom

	-- 总刷新次数背景
	local totalRefreshBg = ccui.Scale9Sprite:create( cc.rect( 12, 11, 14, 10 ), "res/Main/role/common/name_bg.png" )
	totalRefreshBg:setContentSize( 240, 30 )
	totalRefreshBg:setAnchorPoint(0,0.5)
	totalRefreshBg:setPosition( 10, 45 )
	bottom:addChild( totalRefreshBg )
	self._totalRefreshBg = totalRefreshBg
	-- 今日总刷新次数文字
	local totalRefreshText = zc.createLabel({
		text = LANGUAGE_SHOP.total_refresh_times..":",
        fontSize = 20,
        outLine = true,
        pixel = 1.5,
        ap = cc.p( 0, 0.5 ),
        pos = cc.p( 10, totalRefreshBg:getContentSize().height/2 ),
	})
	totalRefreshBg:addChild( totalRefreshText )
	self._totalRefreshText = totalRefreshText
	-- 今日总刷新次数数字
	local totalRefreshNum = zc.createLabel({
        fontSize = 22,
        outLine = true,
        pixel = 1.5,
        color = display.COLOR_GREEN,
        ap = cc.p( 0, 0.5 ),
        pos = cc.p( 155, totalRefreshBg:getContentSize().height/2 ),
	})
	totalRefreshBg:addChild( totalRefreshNum )
	self._totalRefreshNum = totalRefreshNum
	-- 免费刷新，含倒计时
	local freeRefresh = zc.createRichLabel({
        fontSize = 22,
        pixel = 1.5,
        color = display.COLOR_GREEN,
        pos = cc.p( 550, 45 ),
	})
	freeRefresh:setAnchorPoint( 1, 0.5 )
    freeRefresh:enableStroke(display.COLOR_BLACK, 0.2)
	bottom:addChild( freeRefresh )
	self._freeRefresh = freeRefresh
	-- 刷新按钮
	local refreshBtn = zc.createCommomBtn({
        tag = 2,
        size = cc.size( 110, 44 ),
        pos = cc.p( 630, 45 ),
        text = "",
        listener = function()
        	if self._tabData[self._tabIndex].db  == "house_goods" then
        		if self._flushData.flushNumLimit > self._flushData.goodsFlushNum and self._flushData.flushNumLimit > self._flushData.goodsFlushNum then
	        		-- 还有刷新次数
	        		self:clickRefreshCallback()
	        	else
	        		zc.tipInfo( LANGUAGE_SHOP.no_refresh_times )
	        	end
	        elseif self._tabData[self._tabIndex].db  == "arena_goods" then
	        	if self._flushData.flushNumLimit > self._flushData.goodsFlushNum then
	        		self:clickRefreshCallback()
	        	else
	        		zc.tipInfo( LANGUAGE_SHOP.no_refresh_times )
	        	end
	        elseif self._tabData[self._tabIndex].db  == "prestigeShop" then
	        	print("flushNumLimit ?",self._flushData.flushNumLimit,"?",self._flushData.prestigeShopRefresh)
	        	local function sendRefresh( )
	        		zc.sendHttp({
						apiName = self._tabData[self._tabIndex].refreshOp,
						successCallback = function(data)
							zc.tipInfo( zc.lang("refresh_success") )
				        	-- dump(data)
				        	self.shop_data = data
				        	if self._flushData.free > 0 then
								PlayerData.limitInfo.prestigeShopRefresh = PlayerData.limitInfo.prestigeShopRefresh + 1
			        			PlayerData:updataData(data)
			        			self:refreshShop(data)
			        		else
			        			PlayerData.limitInfo.prestigeShopRefresh = PlayerData.limitInfo.prestigeShopRefresh + 1
			        			PlayerData:updataData(data)
			        			self:refreshShop( data )
			        		end
					    end
				    })
	        	end
				if self._flushData.flushNumLimit > self._flushData.prestigeShopRefresh then
	        		sendRefresh()
	        	else
	        		-- if self._flushData.free > 0 then
	        		-- 	sendRefresh()
	        		-- else
		        		local _tip_ = zc.lang("tishengVip")
		        		local comfirm = ZCConfirmDialog:createWithParams({
	                        title = zc.lang("vip_max"),
	                        msg = _tip_,
	                        showDi = true,
	                        rightText = zc.lang("tisheng_"),
	                        rightCallback = function()
	                        	LayerManager.backToMainCity(false)
	                        	zc.createRechargePop()
	                        end
	                    })
	                    self:addChild(comfirm, 11)
                	-- end
	        	end
	        end
    	end
	})
	
	bottom:addChild( refreshBtn )
	self._refreshBtn = refreshBtn

	-- 刷新消耗
	-- icon
	local consumeIcon = cc.Sprite:create()
	consumeIcon:setContentSize( 40, 40 )
	consumeIcon:setAnchorPoint(0,0.5)
	consumeIcon:setPosition( 25, refreshBtn:getContentSize().height/2 )
	-- consumeIcon:setScale( 0.8 )
	refreshBtn:addChild( consumeIcon )
	self._consumeIcon = consumeIcon
	-- num
	local consumeNum = zc.createLabel({
		fontSize = 20,
        outLine = true,
        pixel = 2,
        ap = cc.p( 0.5, 1),
        pos = cc.p( consumeIcon:getContentSize().width/2, 20 ),
	})
	consumeIcon:addChild( consumeNum )
	self._consumeNum = consumeNum
end
-- 点击刷新按钮
function ShopLayer:clickRefreshCallback()
	-- 需要tss
	zc.sendHttp({
		apiName = self._tabData[self._tabIndex].refreshOp,
		sendType = 1,
		params = self._tabData[self._tabIndex].params,
		successCallback = function(data)
			if data.status and tonumber(data.status) == 0 then
	        	data.data.noAnimation = true
	            PlayerData:updataData( data.data )
				self._tss = tonumber( data.tss ) or 0
	            self:refreshShop( data.data )
	            self:refreshBubble( data.data )
	        end
	    end
    })
end
-- 点击购买按钮
function ShopLayer:clickBuyCallback( index, i, _callback )
	local params = table.copy( self._tabData[self._tabIndex].params )
	params.id = self._goodsData[2*index + i].id
	zc.sendHttp({
		apiName = self._tabData[self._tabIndex].buyOp,
		params = params,
		successCallback = function(data)
			data.noAnimation = true
            PlayerData:updataData(data)
            musicManager.playEffect( zc.effect.shopBuy )
            if self._tabData[self._tabIndex].limit then
            	self._goodsData[2*index + i].buyed = 1
            end
            -- 刷新tableview
            self._goodsTableview:reloadDataAndScrollToCurrentCell()
            zc.tipInfo( zc.lang("tips_buy_success") )
            if _callback then
            	_callback()
            end
        end,
	})
end
function ShopLayer:buyLimitGoods( _info, num )
	local _tabData = self._tabData[self._tabIndex]
	local params = table.copy( _tabData.params )
	params.id = _info.id
	params.num = num or 1
	zc.sendHttp({
	    apiName = _tabData.buyOp,
	    params = params,
	    successCallback = function (net_data)
			net_data.noAnimation = true
            PlayerData:updataData(net_data)
            zc.tipInfo( zc.lang("tips_buy_success") )
            if not _info._buyNum then _info._buyNum = 0 end
            _info._buyNum = _info._buyNum + num
            if _info.buyTimes and tonumber(_info.buyTimes) > 0 and _info._buyNum >= tonumber(_info.buyTimes) then
            	_info.buyed = 1
            end
            self._goodsTableview:reloadDataAndScrollToCurrentCell()
	    end
	})
end

-- 根据self._tabIndex刷新当前界面
function ShopLayer:refreshShop( data )
	-- print("self._tabIndex:",self._tabIndex)
	-- dump(data,"----test---")
	self.shop_data = data
	local _tabData = self._tabData[self._tabIndex]
	-- dump(_tabData,"_tabData")
	self._goodsData = {}
    LayerManager.getTopInfoLayer():setPopInfo( _tabData.topInfo )
	if _tabData.db == "limit_mall" then
		for k,v in pairs(data.buyInfo) do
			local _goodsId = tonumber(k)
			if _goodsId and _goodsId > 0 then
				local _goodsStatic = gameData.getDataFromCacheWithPrimaryKey("limit_mall", _goodsId )
				local _index = #self._goodsData + 1
				self._goodsData[_index] = _goodsStatic
				self._goodsData[_index]._buyNum = tonumber(v)
				if tonumber(_goodsStatic.buyTimes) > 0 and self._goodsData[_index]._buyNum >= tonumber(_goodsStatic.buyTimes) then
					self._goodsData[_index].buyed = 1
				else
					self._goodsData[_index].buyed = 0
				end
				self:getGoodsName( _goodsStatic )
			end
		end
		table.sort( self._goodsData, function(a,b)
			return tonumber(a.id) < tonumber(b.id)
		end)
		self:refreshBottom()
	else
		local _tabType = _tabData.params.type
		-- 顶部信息栏
		-- 查找数据
		if data then
			if _tabType == 5 then
				-- dump(data) 
				-- local goods = cjson.encode(data.goods)
				local goods = data.info.goods
				for i,v in pairs(goods) do
					local goodsData = gameData.getDataFromCacheWithPrimaryKey( _tabData.db, tonumber(i))
					-- dump(goodsData)
					local hourseInfo = PlayerData:getHouseInfoByHouseId(16)
					local level = hourseInfo["level"]

					if goodsData.itemType == 1 and goodsData.itemId == 2 then
						goodsData.itemNum = goodsData.itemNum * level
					end
					goodsData.buyed = v
					self:getGoodsName( goodsData )
					self._goodsData[#self._goodsData + 1] = goodsData
				end
			else
				for i, v in ipairs( data.goods[_tabType] ) do
					local goodsData = gameData.getDataFromCacheWithPrimaryKey( _tabData.db, v.id)
					for k, u in pairs( v ) do
						goodsData[k] = u
					end
					self:getGoodsName( goodsData )
					self._goodsData[#self._goodsData + 1] = goodsData
				end
			end
		else
			local allGoodsData = gameData.getDataFromCache( _tabData.db )
			for k, goodsData in pairs( allGoodsData ) do
				self:getGoodsName( goodsData )
				self._goodsData[#self._goodsData + 1] = goodsData
			end
		end
		table.sort(self._goodsData, function( a, b )
			return ( tonumber( a.id ) or 0 ) < ( tonumber( b.id ) or 0 )
		end)
		-- dump(data,"data--------")
		if _tabType ~= 5 then
			self:refreshBottom( data and data.info[_tabType] )
		else
			self:refreshBottom( data)
		end
	end
	-- 设置是否可以滑动
	if #self._goodsData > 6 then
		self._goodsTableview:setBounceable( true )
	else
		self._goodsTableview:setBounceable( false )
	end
end
-- 刷新底部
function ShopLayer:refreshBottom( data )
	-- dump( data, "data ============ " )
	-- 根据tabIndex刷新底部
	self._consumeIcon:setPosition(25,self._refreshBtn:getContentSize().height/2)
	self:stopTimer()
	if self._tabData[self._tabIndex].db == "house_goods" then
		-- 刷新恢复倒计时
		self._flushData.recoverInterval = tonumber( data.recoverInterval or self._flushData.recoverInterval )
		-- 总刷新次数上限
		self._flushData.flushNumLimit = tonumber( data.flushNumLimit or self._flushData.flushNumLimit )
		-- 已刷新次数
		self._flushData.goodsFlushNum = tonumber( data.goodsFlushNum or self._flushData.goodsFlushNum )
		-- 免费刷新次数
		self._flushData.freeFlushNum = tonumber( data.freeFlushNum or self._flushData.freeFlushNum )
		-- 免费刷新次数上限
		self._flushData.freeFlushNumLimit = tonumber( data.freeFlushNumLimit or self._flushData.freeFlushNumLimit )
		-- 已购买刷新次数
		self._flushData.goodsFlushPayNum = tonumber( data.goodsFlushPayNum or self._flushData.goodsFlushPayNum )
		-- 上次刷新时间
		self._flushData.flushNumTime = tonumber( data.flushNumTime or self._flushData.flushNumTime )

		-- 商店
		self._bottom:setVisible( true )
		self._goodsTableview:setViewSize( cc.size( 713, 390 ) )
		self._goodsTableview:setPosition( 162, 85 )
		self._goodsTableview:reloadData()
		-- 今日总刷新次数
		self._totalRefreshText:setString(LANGUAGE_SHOP.total_refresh_times..":")
		self._totalRefreshNum:setText( ( self._flushData.flushNumLimit - self._flushData.goodsFlushNum ).."/"..self._flushData.flushNumLimit )
        self._totalRefreshNum:setPositionX(self._totalRefreshText:getPositionX()+self._totalRefreshText:getContentSize().width)
		-- 免费刷新次数
		self:startTimer( self._flushData.flushNumTime + self._flushData.recoverInterval - PlayerData:getTime() )
		self._refreshBtn:setVisible(true)
		if self._flushData.freeFlushNum > 0 then
			-- 刷新消耗
			self._consumeIcon:setVisible( false )
			self._consumeNum:setText( "" )
			-- 刷新按钮
			self._refreshBtn:setString( LANGUAGE_SHOP.free_refresh )
			-- 倒计时
			self._freeRefresh:setPositionY( 45 )
		else
			-- 刷新消耗上限次数
			local consumeList = gameData.getDataFromCache("consume_price", {type = 29})
			local timesMax = 0
			for i, v in ipairs( consumeList ) do
				local _times = tonumber( v.subtype )
				if timesMax < _times then
					timesMax = _times
				end
			end
			-- 表里查询的刷新次数
			local times = tonumber( self._flushData.goodsFlushPayNum ) + 1
			times = times > timesMax and timesMax or times
			local consumeData = {}
			for i, v in ipairs( consumeList ) do
				if tonumber( v.subtype ) == times then
					consumeData = v
					break
				end
			end
			-- 刷新消耗
			self._consumeIcon:setTexture( zc.getImgByType( zc.dataTypeGold ) )
			self._consumeIcon:setVisible( true )
			self._consumeNum:setText( consumeData.price or "" )
			-- 刷新按钮
			self._refreshBtn:setString( LANGUAGE_SHOP.refresh_space )
			-- 倒计时
			self._freeRefresh:setPositionY( 25 )
		end
	elseif self._tabData[self._tabIndex].db == "arena_goods" then
		-- 隐藏底部
		self._bottom:setVisible( false )
		self._goodsTableview:setViewSize( cc.size( 713, 460 ) )
		self._goodsTableview:setPosition( 162, 15 )
		self._goodsTableview:reloadData()
	elseif self._tabData[self._tabIndex].db == "prestigeShop" then
		self._bottom:setVisible( true )

		self._goodsTableview:setViewSize( cc.size( 713, 390 ) )
		self._goodsTableview:setPosition( 162, 85 )
		self._goodsTableview:reloadData()

		local vip_ = PlayerData.info.vip
		-- 花元宝次数
		self._flushData.prestigeShopPayRefresh = tonumber(PlayerData.limitInfo.prestigeShopPayRefresh or self._flushData.prestigeShopPayRefresh)
		-- 已刷新次数
		self._flushData.prestigeShopRefresh = tonumber(PlayerData.limitInfo.prestigeShopRefresh or self._flushData.prestigeShopRefresh)
		-- 总刷新次数上限
		if self.Total_refresh["vip"..vip_] then
			self._flushData.flushNumLimit = tonumber( self.Total_refresh["vip"..vip_] or self._flushData.flushNumLimit )
		end
		-- print("vip_:",vip_)
		-- dump(self.free_total_refresßh)
		if self.free_total_refresh["vip"..vip_] then
			self._flushData.free_total_refresh_num = tonumber( self.free_total_refresh["vip"..vip_] or self._flushData.free_total_refresh_num )
		end
		-- 今日总刷新次数
		self._totalRefreshText:setString(LANGUAGE_SHOP.total_refresh_times..":")
		-- 已经刷新了几次
		self._totalRefreshNum:setText( (self._flushData.flushNumLimit - self._flushData.prestigeShopRefresh).."/"..self._flushData.flushNumLimit )
		self._totalRefreshNum:setColor(self._flushData.flushNumLimit > self._flushData.prestigeShopRefresh and display.COLOR_GREEN or display.COLOR_RED)
        self._totalRefreshNum:setPositionX(self._totalRefreshText:getPositionX()+self._totalRefreshText:getContentSize().width)
		-- 免费刷新次数
		self._flushData.free = tonumber(data.info.free or self._flushData.free)
		-- print("self._flushData.flushNumLimit ",self._flushData.flushNumLimit,"self._flushData.free ",self._flushData.free)
		-- 时间
		self._flushData.freeTime = tonumber(data.info.freeTime or self._flushData.freeTime)
		print("freeTime ",self._flushData.freeTime,"Player:",PlayerData:getTssTime(),"shijian:",3600 - (PlayerData:getTssTime() - self._flushData.freeTime))
		self._refreshBtn:setVisible(true)
		self._refreshBtn:setEnable(true)
		self:startTimer(3600 - (PlayerData:getTssTime() - self._flushData.freeTime))

		if self._flushData.free > 0 then
			-- 刷新消耗
			self._consumeIcon:setVisible( false )
			self._consumeNum:setText( "" )
			-- 刷新按钮
			self._refreshBtn:setString( LANGUAGE_SHOP.free_refresh )
			-- 倒计时
			self._freeRefresh:setPositionY( 45 )
		else
			-- 刷新消耗上限次数
			local consumeList = gameData.getDataFromCache("consume_price", {type = 29})
			local timesMax = 0
			for i, v in ipairs( consumeList ) do
				local _times = tonumber( v.subtype )
				if timesMax < _times then
					timesMax = _times
				end
			end
			-- 表里查询的刷新次数
			local times = tonumber( self._flushData.prestigeShopPayRefresh ) + 1
			times = times > timesMax and timesMax or times
			local consumeData = {}
			for i, v in ipairs( consumeList ) do
				if tonumber( v.subtype ) == times then
					consumeData = v
					break
				end
			end
			-- 刷新消耗
			self._consumeIcon:setTexture( zc.getImgByType( zc.dataTypeGold ) )
			self._consumeIcon:setVisible( true )
			self._consumeIcon:setPosition(5,self._refreshBtn:getContentSize().height/2)
			self._consumeNum:setText( consumeData.price or "" )
			-- 刷新按钮
			self._refreshBtn:setString( LANGUAGE_SHOP.refresh_space )
			-- 倒计时
			self._freeRefresh:setPositionY( 45 )
		end
	else
		-- 隐藏底部
		self._bottom:setVisible( false )
		self._goodsTableview:setViewSize( cc.size( 713, 460 ) )
		self._goodsTableview:setPosition( 162, 15 )
		self._goodsTableview:reloadData()
	end
end

-- 倒计时相关
function ShopLayer:startTimer( time )
	self:stopTimer()
	print("self._tabIndex:",self._tabIndex)
	if self._tabIndex == 4 then
		self._timer = scheduler.scheduleGlobal(function()
			time = ( time - 1 )
			self:tick( time )
		end, 1)
	else
		if self._tabIndex ~= 4 then
			if self._flushData.flushNumTime and self._tss and tonumber(self._tss) and self._flushData.flushNumTime == tonumber(self._tss) then
				--第一次由于时间对不上，手动处理恢复时间为一个间隔时间 
				time = self._flushData.recoverInterval
				self._tss = nil
			else
				time = time%self._flushData.recoverInterval
			end
		end
		self:tick( time )
	end
	-- if self._tabIndex == 4 then
	-- 	self._timer = scheduler.scheduleGlobal(function()
	-- 		time = ( time - 1 )
	-- 		self:tick( time )
	-- 	end, 1)
	-- else
	-- 	self._timer = scheduler.scheduleGlobal(function()
	-- 		time = ( time - 1 )%self._flushData.recoverInterval
	-- 		self:tick( time )
	-- 	end, 1)
	-- end
end

function ShopLayer:tick( time )
	-- print("tick time:",time)
	if self._tabData[self._tabIndex].db == "house_goods" then
		if self._flushData.freeFlushNum >= self._flushData.freeFlushNumLimit then
			-- 免费刷新次数达到上限
			self._freeRefresh:setString( self._flushData.freeFlushNum.."/"..self._flushData.freeFlushNumLimit )
			self._freeRefresh:setPositionX( 550 )
			self:stopTimer()
		else
			-- 继续恢复
			self._freeRefresh:setString( self._flushData.freeFlushNum.."/"..self._flushData.freeFlushNumLimit.."[color=ff4c1212]("..zc.getCdStringWithNumber( time )..")[/color]" )
			self._freeRefresh:setPositionX( 550 )
			if time == 0 then
				-- 倒计时结束，增加一次免费刷新次数
				self._flushData.freeFlushNum = self._flushData.freeFlushNum + 1
			end
		end
	elseif self._tabData[self._tabIndex].db == "prestigeShop" then -- 声望商店
		local function updateRefrersh()
			local _free = self._flushData.prestigeShopRefresh - self._flushData.prestigeShopPayRefresh
			local str = zc.lang("free")
			-- self._flushData.free 剩余免费刷新的次数
			-- print("prestigeShopRefresh:",self._flushData.prestigeShopRefresh,"prestigeShopPayRefresh:",self._flushData.prestigeShopPayRefresh)
			if self._flushData.free >= self._flushData.free_total_refresh_num then
				-- 免费刷新次数达到上限 停止计时器
				self._freeRefresh:setString( str.. self._flushData.free.."/".. self._flushData.free_total_refresh_num)
				self._freeRefresh:setPositionX( 540 )
				self:stopTimer()
			else
				-- 继续恢复
				-- print("继续恢复 time:",time)
				self._freeRefresh:setString( str..self._flushData.free.."/".. self._flushData.free_total_refresh_num.." [color=ff4c1212]("..zc.getCdStringWithNumber( time )..")[/color]" )
				self._freeRefresh:setPositionX( 540 )
				if time == 0 then
					-- 倒计时结束，增加刷新次数
					-- self._flushData.prestigeShopRefresh = self._flushData.prestigeShopRefresh + 1
					self._flushData.free = self._flushData.free + 1

					-- 倒计时
					self._freeRefresh:setPositionY( 45 )
					-- 刷新消耗
					self._consumeIcon:setVisible( false )
					self._consumeNum:setVisible( false )
					if self._refreshBtn then
						local _normal = self._refreshBtn:getStateNormal()
						local _select = self._refreshBtn:getStateSelected()
						if _normal and _normal:getChildByName("normalLab") then
							_normal:getChildByName("normalLab"):setPositionX(_normal:getContentSize().width/2)
						end
						if _select and _select:getChildByName("selectedLab") then
							_select:getChildByName("selectedLab"):setPositionX(_select:getContentSize().width/2)
						end
					end
				end
			end
		end
		local vip_ = PlayerData.info.vip
		if vip_ >= 10 then
			if self._flushData.flushNumLimit <= self._flushData.prestigeShopRefresh then
				-- self:stopTimer()
				-- 当前次数
				-- print("self._flushData.free ",self._flushData.free)
				-- if self._flushData.free > 0 then -- 说明这个还有免费的
				-- 	self._freeRefresh:setPositionY( 45 )
				-- else
				-- 	self._freeRefresh:setPositionY( 45 )
				-- 	-- 刷新消耗
					self._consumeIcon:setTexture( zc.getImgByType( zc.dataTypeGold ) )
					self._consumeIcon:setVisible( false )
					self._consumeIcon:setPosition(5,self._refreshBtn:getContentSize().height/2)
				-- end
				-- self._freeRefresh:setString( zc.lang("free").. self._flushData.free.."/".. self._flushData.free_total_refresh_num)
				self._refreshBtn:setEnable(false)
				updateRefrersh()
			else
				self._refreshBtn:setEnable(true)
				-- 已经刷新的次数减去元宝刷新的次数 就是免费刷新的次数
				updateRefrersh()
			end
		else
			updateRefrersh()
		end
	elseif self._tabData[self._tabIndex].db == "arena_goods" then
		local _befNum = self._flushData.freeFlushNumLimit + self._flushData.flushNum + self._flushData.arenaShopGoldFlush - self._flushData.goodsFlushNum
		local _aftNum = self._flushData.freeFlushNumLimit
		if _befNum < 0 then
			_befNum = 0
		end
		if self._flushData.flushNum >= self._flushData.goodsFlushNum then
			-- 免费刷新次数达到上限
			self._freeRefresh:setString( _befNum.."/".._aftNum )
			self._freeRefresh:setPositionX( 550 )
			self:stopTimer()
		else
			-- 继续恢复
			self._freeRefresh:setString( _befNum.."/".. _aftNum .."[color=ff4c1212]("..zc.getCdStringWithNumber( time )..")[/color]" )
			self._freeRefresh:setPositionX( 550 )
			if time == 0 then
				-- 倒计时结束，增加一次免费刷新次数
				self._flushData.flushNum = self._flushData.flushNum + 1

				-- 倒计时
				self._freeRefresh:setPositionY( 45 )
				-- 刷新消耗
				self._consumeIcon:setVisible( false )
				self._consumeNum:setVisible( false )
				if self._refreshBtn then
					local _normal = self._refreshBtn:getStateNormal()
					local _select = self._refreshBtn:getStateSelected()
					if _normal and _normal:getChildByName("normalLab") then
						_normal:getChildByName("normalLab"):setPositionX(_normal:getContentSize().width/2)
					end
					if _select and _select:getChildByName("selectedLab") then
						_select:getChildByName("selectedLab"):setPositionX(_select:getContentSize().width/2)
					end
				end
			end
		end
	end
end
function ShopLayer:stopTimer()
	if self._timer then
		scheduler.unscheduleGlobal( self._timer )
		self._timer = nil
	end
end
function ShopLayer:checkOpenstate(_id, _nextFun, _outFun)
    zc.sendHttp({
        apiName = "Activity_open_info",
        successCallback = function (net_data)
        	local _outDate = true
			if net_data and next(net_data) then
            	PlayerData:setActivityInfo(net_data)
            	local _activityInfo = PlayerData:getActivityInfoById(13)
	            if _activityInfo and _activityInfo["id"] and next(_activityInfo) == nil then
	                _outDate = true
	            else
	                _outDate = false
					if _nextFun then
						_nextFun()
					end
	            end
			end
    		if _outDate and _outFun then
        		_outFun()
        		zc.dispatchEvent({name = CUSTOM_EVENT.EVENT_REDORBUBBLE_CHANGE, data = {tag = "activityShow"}}) 
        	end
        end
    })
end
function ShopLayer:getGoodsName( goodsData )
	-- dump(goodsData, "--goodsData---")
	local goodsInfo = zc._getIconInfo({
		data = {
			mainType = goodsData.itemType,
			type = goodsData.itemId or 0,
			id = goodsData.itemId,
		},
	})
	goodsData.name = goodsInfo.name
end

function ShopLayer:onEnter()
	--功能引导
	local _currentGID = getGuideManager():getCurrentGID()
	if _currentGID == GuideKey.KEY_12207 then
		zc.setGuideFuncNextStep(_currentGID, false)
		self:onEnter()
	elseif inArray(_currentGID, GuideGroup.DUI_HUA) then
		local function _callBack()
            getGuideManager():setFuncGuideOver()
        end
        getGuideManager():setHeroFuncCallBack(_callBack)
        getGuideManager():startGuide()
	end
end

-- 刷新主城气泡
function ShopLayer:refreshBubble( data )
	local _tabData = self._tabData[self._tabIndex]
	local _tabType = _tabData.params.type
	if self._tabData[self._tabIndex].db  == "house_goods" then
		data.info[_tabType].flushNumLimit = self._flushData.flushNumLimit
		data.info[_tabType].recoverInterval = self._flushData.recoverInterval
	elseif self._tabData[self._tabIndex].db  == "arena_goods" then
		data.info[_tabType].flushLimit = self._flushData.freeFlushNumLimit
		data.info[_tabType].goodsFlushNumInterval = self._flushData.recoverInterval
	end
	PlayerData:setShopCanDo( self._tabIndex, data.info[_tabType] )
end

return ShopLayer